Attribute VB_Name = "ListViewUtil"
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit


Private Declare Function LockWindowUpdate Lib "user32" ( _
    ByVal hWnd As Long) As Long

Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" ( _
    ByVal hWnd As Long, _
    ByVal Msg As Long, _
    ByVal wParam As Long, _
    lParam As Any) As Long

Private Type LVITEM
    lMask As Long
    lItem As Long
    lSubItem As Long
    lState As Long
    lStateMask As Long
    sText As String
    lTextMax As Long
    lImage As Long
    lParam As Long
    lIndent As Long
End Type

Const LVM_FIRST = &H1000

Const LVM_GETITEM = (LVM_FIRST + 5)
Const LVM_SETITEM = (LVM_FIRST + 6)
Const LVM_GETCOLUMNWIDTH = (LVM_FIRST + 29)
Const LVM_SETCOLUMNWIDTH = (LVM_FIRST + 30)
Const LVM_SETTEXTBKCOLOR = (LVM_FIRST + 38)
Const LVM_SETBKIMAGE = (LVM_FIRST + 68)

Const LVSCW_AUTOSIZE = -1
Const LVSCW_AUTOSIZE_USEHEADER = -2

Const LVIF_INDENT = &H10&

Const WM_SETREDRAW = &HB

Public Sub FitColumnSize(oLVW As MSComctlLib.ListView, _
                         ByVal bUseHeader As Boolean, _
                         Optional ByVal nMaxVisible As Integer = 0)
    Dim lCol As Long
    Dim lWidth1 As Long
    Dim lWidth2 As Long
    
    With oLVW
        LockWindowUpdate .hWnd
'        Call SendMessage(.hWnd, WM_SETREDRAW, 0, ByVal 0&)
        If (nMaxVisible = 0 Or nMaxVisible > .ColumnHeaders.Count) Then
            nMaxVisible = .ColumnHeaders.Count
        End If

        For lCol = 1 To nMaxVisible
            Call SendMessage(.hWnd, LVM_SETCOLUMNWIDTH, lCol - 1, ByVal LVSCW_AUTOSIZE)
            If bUseHeader Then
                lWidth1 = SendMessage(.hWnd, LVM_GETCOLUMNWIDTH, lCol - 1, ByVal 0&)
            
                Call SendMessage(.hWnd, LVM_SETCOLUMNWIDTH, lCol - 1, ByVal LVSCW_AUTOSIZE_USEHEADER)
                lWidth2 = SendMessage(.hWnd, LVM_GETCOLUMNWIDTH, lCol - 1, ByVal 0&)
                
                If lWidth1 > lWidth2 Then
                    Call SendMessage(.hWnd, LVM_SETCOLUMNWIDTH, lCol - 1, ByVal lWidth1)
                Else
                    Call SendMessage(.hWnd, LVM_SETCOLUMNWIDTH, lCol - 1, ByVal lWidth2)
                End If
            End If
        Next
        If nMaxVisible < .ColumnHeaders.Count Then
            For lCol = (nMaxVisible + 1) To .ColumnHeaders.Count
                Call SendMessage(.hWnd, LVM_SETCOLUMNWIDTH, lCol - 1, ByVal 0)
            Next
        End If
'        Call SendMessage(.hWnd, WM_SETREDRAW, 1, ByVal 0&)
        LockWindowUpdate 0&
    End With
End Sub

Public Sub SetIndent(oLVW As MSComctlLib.ListView, _
                     ByVal lIndex As Long, _
                     Optional ByVal lIndent As Long = 1)
    
    Dim lvi As LVITEM

    With lvi
        .lItem = lIndex - 1
        .lMask = LVIF_INDENT
        .lIndent = lIndent
    End With
    
    Call SendMessage(oLVW.hWnd, LVM_SETITEM, 0&, lvi)
End Sub

Public Function GetListItemByKey(oLVW As MSComctlLib.ListView, _
                                 sKey As String) As MSComctlLib.ListItem
    On Error Resume Next
    Set GetListItemByKey = oLVW.ListItems("K" + sKey)
End Function

Public Function GetListItemByIndex(oLVW As MSComctlLib.ListView, _
                                 iIndex As Integer) As MSComctlLib.ListItem
    On Error Resume Next
    Set GetListItemByIndex = oLVW.ListItems(iIndex)
End Function

Public Sub SortColumn(oLVW As MSComctlLib.ListView, _
                      oCol As MSComctlLib.ColumnHeader)
    If oCol.Tag = lvwAscending Then
       oCol.Tag = lvwDescending
    Else
       oCol.Tag = lvwAscending
    End If
    With oLVW
        .SortOrder = oCol.Tag
        .SortKey = oCol.SubItemIndex
        .Sorted = True
    End With
End Sub

Public Function LVWKey(sName As String) As String
    LVWKey = "K" + sName
End Function
Public Function LVWName(sKey As String) As String
    LVWName = Mid$(sKey, 2)
End Function

