VERSION 5.00
Begin VB.UserControl FloodBar 
   AutoRedraw      =   -1  'True
   CanGetFocus     =   0   'False
   ClientHeight    =   495
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   6330
   ScaleHeight     =   33
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   422
End
Attribute VB_Name = "FloodBar"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

'''

Private Declare Function DrawText Lib "User32.dll" Alias "DrawTextA" ( _
    ByVal hdc As Long, _
    ByVal lpStr As String, _
    ByVal nCount As Long, _
    lpRect As RECT, _
    ByVal wFormat As Long) As Long

Const DT_LEFT = &H0
Const DT_TOP = &H0
Const DT_CENTER = &H1
Const DT_RIGHT = &H2
Const DT_VCENTER = &H4
Const DT_BOTTOM = &H8
Const DT_WORDBREAK = &H10
Const DT_SINGLELINE = &H20
Const DT_EXPANDTABS = &H40
Const DT_TABSTOP = &H80
Const DT_NOCLIP = &H100
Const DT_EXTERNALLEADING = &H200
Const DT_CALCRECT = &H400
Const DT_NOPREFIX = &H800
Const DT_INTERNAL = &H1000
Const DT_EDITCONTROL = &H2000
Const DT_PATH_ELLIPSIS = &H4000
Const DT_END_ELLIPSIS = &H8000
Const DT_MODIFYSTRING = &H10000
Const DT_RTLREADING = &H20000
Const DT_WORD_ELLIPSIS = &H40000

'''

Private Declare Function DrawEdge Lib "User32.dll" ( _
    ByVal hdc As Long, _
    lpRect As RECT, _
    ByVal edge As Long, _
    ByVal grfFlags As Long) As Long

Const BDR_RAISEDOUTER = &H1
Const BDR_SUNKENOUTER = &H2
Const BDR_RAISEDINNER = &H4
Const BDR_SUNKENINNER = &H8

Const EDGE_BUMP = (BDR_RAISEDOUTER Or BDR_SUNKENINNER)
Const EDGE_ETCHED = (BDR_SUNKENOUTER Or BDR_RAISEDINNER)
Const EDGE_RAISED = (BDR_RAISEDOUTER Or BDR_RAISEDINNER)
Const EDGE_SUNKEN = (BDR_SUNKENOUTER Or BDR_SUNKENINNER)

Const BF_LEFT = &H1
Const BF_TOP = &H2
Const BF_RIGHT = &H4
Const BF_BOTTOM = &H8
Const BF_TOPLEFT = (BF_TOP Or BF_LEFT)
Const BF_TOPRIGHT = (BF_TOP Or BF_RIGHT)
Const BF_BOTTOMLEFT = (BF_BOTTOM Or BF_LEFT)
Const BF_BOTTOMRIGHT = (BF_BOTTOM Or BF_RIGHT)
Const BF_RECT = (BF_LEFT Or BF_TOP Or BF_RIGHT Or BF_BOTTOM)
Const BF_DIAGONAL = &H10
Const BF_DIAGONAL_ENDBOTTOMLEFT = (BF_DIAGONAL Or BF_BOTTOM Or BF_LEFT)
Const BF_DIAGONAL_ENDBOTTOMRIGHT = (BF_DIAGONAL Or BF_BOTTOM Or BF_RIGHT)
Const BF_DIAGONAL_ENDTOPLEFT = (BF_DIAGONAL Or BF_TOP Or BF_LEFT)
Const BF_DIAGONAL_ENDTOPRIGHT = (BF_DIAGONAL Or BF_TOP Or BF_RIGHT)
Const BF_MIDDLE = &H800    ' Fill in the middle.
Const BF_SOFT = &H1000     ' Use for softer buttons.
Const BF_ADJUST = &H2000   ' Calculate the space left over.
Const BF_FLAT = &H4000     ' For flat rather than 3-D borders.
Const BF_MONO = &H8000     ' For monochrome borders.

'''

Private Type ColorBytesType
    bRed As Byte
    bGreen As Byte
    bBlue As Byte
    bDummy As Byte
End Type

Private Type ColorType
    lColor As Long
End Type

Private tColorBytes As ColorBytesType
Private tColor As ColorType

'''

Public Enum FloodBarBorderStyleConstants
    flbBorderStyleNone
    flbBorderStyleSingel
    flbBorderStyleSunken
    flbBorderStyleRaised
End Enum

Public Enum FloodBarDrawStyleConstants
    flbDrawStyleDefault
    flbDrawStyleHot
End Enum

'''

Private lBorderStyle As FloodBarBorderStyleConstants
Private lBackColor As OLE_COLOR
Private lFillColor As OLE_COLOR
Private lForeColor As OLE_COLOR
Private iPercent As Integer

Private lDrawStyle As FloodBarDrawStyleConstants

Private lMedColor As OLE_COLOR
Private lHotColor As OLE_COLOR

Private iMedPercent As Integer
Private iHotPercent As Integer

'''

'
' Properties
'
Public Property Get BorderStyle() As FloodBarBorderStyleConstants
    BorderStyle = lBorderStyle
End Property
Public Property Let BorderStyle(ByVal NewVal As FloodBarBorderStyleConstants)
    lBorderStyle = NewVal
    PropertyChanged "BorderStyle"
    DrawFlood
End Property

Public Property Get BackColor() As OLE_COLOR
    BackColor = lBackColor
End Property
Public Property Let BackColor(ByVal NewVal As OLE_COLOR)
    lBackColor = NewVal
    PropertyChanged "BackColor"
    UserControl.BackColor = lBackColor
    DrawFlood
End Property

Public Property Get FillColor() As OLE_COLOR
    FillColor = lFillColor
End Property
Public Property Let FillColor(ByVal NewVal As OLE_COLOR)
    lFillColor = NewVal
    PropertyChanged "FillColor"
    DrawFlood
End Property

Public Property Get ForeColor() As OLE_COLOR
    ForeColor = lForeColor
End Property
Public Property Let ForeColor(ByVal NewVal As OLE_COLOR)
    lForeColor = NewVal
    PropertyChanged "ForeColor"
    UserControl.ForeColor = lForeColor
    DrawFlood
End Property

Public Property Get Percent() As Integer
    Percent = iPercent
End Property
Public Property Let Percent(ByVal NewVal As Integer)
    iPercent = NewVal
    If iPercent < -1 Then iPercent = -1
    If iPercent > 100 Then iPercent = 100
    PropertyChanged "Percent"
    DrawFlood
End Property

Public Property Get DrawStyle() As FloodBarDrawStyleConstants
    DrawStyle = lDrawStyle
End Property
Public Property Let DrawStyle(ByVal NewVal As FloodBarDrawStyleConstants)
    lDrawStyle = NewVal
    PropertyChanged "DrawStyle"
    DrawFlood
End Property

Public Property Get MediumColor() As OLE_COLOR
    MediumColor = lMedColor
End Property
Public Property Let MediumColor(ByVal NewVal As OLE_COLOR)
    lMedColor = NewVal
    PropertyChanged "MediumColor"
    DrawFlood
End Property

Public Property Get HotColor() As OLE_COLOR
    HotColor = lHotColor
End Property
Public Property Let HotColor(ByVal NewVal As OLE_COLOR)
    lHotColor = NewVal
    PropertyChanged "HotColor"
    DrawFlood
End Property

Public Property Get MediumPercent() As Integer
    MediumPercent = iMedPercent
End Property
Public Property Let MediumPercent(ByVal NewVal As Integer)
    iMedPercent = NewVal
    If iMedPercent < -1 Then iMedPercent = -1
    If iMedPercent > 100 Then iMedPercent = 100
    If iMedPercent > iHotPercent Then iMedPercent = iHotPercent
    PropertyChanged "MediumPercent"
    DrawFlood
End Property

Public Property Get HotPercent() As Integer
    HotPercent = iHotPercent
End Property
Public Property Let HotPercent(ByVal NewVal As Integer)
    iHotPercent = NewVal
    If iHotPercent < -1 Then iHotPercent = -1
    If iHotPercent > 100 Then iHotPercent = 100
    If iHotPercent < iMedPercent Then iHotPercent = iMedPercent
    PropertyChanged "HotPercent"
    DrawFlood
End Property
'
' Events UserControl
'
Private Sub UserControl_Initialize()
    lBorderStyle = flbBorderStyleNone
    lBackColor = vb3DFace
    lFillColor = vbBlue
    lForeColor = vbWindowText
    iPercent = 0
    
    lDrawStyle = flbDrawStyleDefault
    lMedColor = vbYellow
    lHotColor = vbRed
    iMedPercent = 50
    iHotPercent = 75
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    On Error Resume Next
    With PropBag
        BorderStyle = .ReadProperty("BorderStyle", flbBorderStyleNone)
        BackColor = .ReadProperty("BackColor", vb3DFace)
        FillColor = .ReadProperty("FillColor", vbBlue)
        ForeColor = .ReadProperty("ForeColor", vbWindowText)
        Percent = .ReadProperty("Percent", 0)
        DrawStyle = .ReadProperty("DrawStyle", flbDrawStyleDefault)
        MediumColor = .ReadProperty("MediumColor", vbYellow)
        HotColor = .ReadProperty("HotColor", vbRed)
        MediumPercent = .ReadProperty("MediumPercent", 50)
        HotPercent = .ReadProperty("HotPercent", 75)
    End With
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    With PropBag
        .WriteProperty "BorderStyle", BorderStyle, flbBorderStyleNone
        .WriteProperty "BackColor", BackColor, vb3DFace
        .WriteProperty "FillColor", FillColor, vbBlue
        .WriteProperty "ForeColor", ForeColor, vbWindowText
        .WriteProperty "Percent", Percent, 0
        .WriteProperty "DrawStyle", DrawStyle, flbDrawStyleDefault
        .WriteProperty "MediumColor", MediumColor, vbYellow
        .WriteProperty "HotColor", HotColor, vbRed
        .WriteProperty "MediumPercent", MediumPercent, 50
        .WriteProperty "HotPercent", HotPercent, 75
    End With
End Sub

Private Sub UserControl_Resize()
    DrawFlood
End Sub
'
' Private Sub/Function
'
Private Sub DrawFlood()
    Dim sPercent As String
    Dim r As RECT
    
    Dim lWidthAll As Long
    Dim lWidthMed As Long
    Dim lWidthHot As Long
    Dim lWidthRight As Long
    
    Dim lWidthPercent As Long
    
    With r
        .Left = 0
        .Top = 0
        .Right = ScaleWidth
        .Bottom = ScaleHeight
    End With
    
    sPercent = iPercent & " %"
    
    UserControl.Cls
    
    UserControl.DrawMode = vbCopyPen
    
    DrawBorder r
    
    If iPercent = -1 Then Exit Sub
    
    DrawText UserControl.hdc, sPercent, Len(sPercent), r, DT_CENTER + DT_VCENTER + DT_SINGLELINE + DT_NOPREFIX
    
    UserControl.DrawMode = vbMergePenNot
    
    If iPercent > 0 Then
        Select Case lDrawStyle
            Case flbDrawStyleDefault:
                UserControl.Line (r.Left, r.Top)-(r.Left + (((r.Right - r.Left - 1)) * iPercent \ 100), r.Bottom - 1), lFillColor, BF
            
            Case flbDrawStyleHot:
                lWidthAll = r.Right - r.Left - 1
                lWidthMed = lWidthAll * iMedPercent \ 100
                lWidthHot = lWidthAll * iHotPercent \ 100
                lWidthRight = lWidthAll - lWidthHot
                lWidthPercent = lWidthAll * iPercent \ 100
                
                If lWidthPercent < lWidthMed Then
                    UserControl.Line (r.Left, r.Top)-(r.Left + lWidthPercent, r.Bottom - 1), lFillColor, BF
                Else
                    If lWidthPercent < lWidthHot Then
                        UserControl.Line (r.Left, r.Top)-(r.Left + lWidthMed, r.Bottom - 1), lFillColor, BF
                        DrawHotBar r.Left + lWidthMed + 1, r.Left + lWidthPercent - 1, lFillColor, lMedColor, lWidthHot - lWidthMed
                    Else
                        UserControl.Line (r.Left, r.Top)-(r.Left + lWidthMed, r.Bottom - 1), lFillColor, BF
                        DrawHotBar r.Left + lWidthMed + 1, r.Left + lWidthHot, lFillColor, lMedColor, lWidthHot - lWidthMed
                        DrawHotBar r.Left + lWidthHot + 1, r.Left + lWidthPercent, lMedColor, lHotColor, lWidthRight
                    End If
                End If
        End Select
    End If
    
    UserControl.DrawMode = vbCopyPen
End Sub

Private Sub DrawHotBar(ByVal lFirstPos As Long, _
                       ByVal lLastPos As Long, _
                       ByVal lFirstColor As Long, _
                       ByVal lLastColor As Long, _
                       ByVal lWidth As Long)
                    
    Dim sRedStep As Single
    Dim sGreenStep As Single
    Dim sBlueStep As Single
    
    Dim sRed As Single
    Dim sGreen As Single
    Dim sBlue As Single
    
    Dim lPos As Long
    Dim lColor As Long
    
    sRedStep = GetRedStep(lFirstColor, lLastColor, lWidth)
    sGreenStep = GetGreenStep(lFirstColor, lLastColor, lWidth)
    sBlueStep = GetBlueStep(lFirstColor, lLastColor, lWidth)

    sRed = CLng(GetRed(lFirstColor))
    sGreen = CLng(GetGreen(lFirstColor))
    sBlue = CLng(GetBlue(lFirstColor))

    For lPos = lFirstPos To lLastPos
        lColor = RGB(CLng(sRed), CLng(sGreen), CLng(sBlue))
        Line (lPos, 0)-(lPos, ScaleHeight), lColor
        sRed = sRed + sRedStep
        sGreen = sGreen + sGreenStep
        sBlue = sBlue + sBlueStep
    Next
End Sub

'''

Private Sub DrawBorder(r As RECT)
    Select Case lBorderStyle
        Case flbBorderStyleNone:
        Case flbBorderStyleSingel:
            UserControl.Line (r.Left, r.Top)-(r.Right - 1, r.Bottom - 1), vbBlack, B
            r.Left = r.Left + 1
            r.Top = r.Top + 1
            r.Right = r.Right - 1
            r.Bottom = r.Bottom - 1
        Case flbBorderStyleSunken:
            DrawEdge UserControl.hdc, r, EDGE_SUNKEN, BF_RECT + BF_SOFT
            r.Left = r.Left + 2
            r.Top = r.Top + 2
            r.Right = r.Right - 2
            r.Bottom = r.Bottom - 2
        Case flbBorderStyleRaised:
            DrawEdge UserControl.hdc, r, EDGE_RAISED, BF_RECT + BF_SOFT
            r.Left = r.Left + 2
            r.Top = r.Top + 2
            r.Right = r.Right - 2
            r.Bottom = r.Bottom - 2
    End Select
End Sub

'''

Private Function GetRedStep(ByVal lColor1 As Long, ByVal lColor2 As Long, ByVal lWidth As Long) As Single
    GetRedStep = (CSng(GetRed(lColor2) - GetRed(lColor1))) / CSng(lWidth)
End Function

Private Function GetGreenStep(ByVal lColor1 As Long, ByVal lColor2 As Long, ByVal lWidth As Long) As Single
    GetGreenStep = (CSng(GetGreen(lColor2) - GetGreen(lColor1))) / CSng(lWidth)
End Function

Private Function GetBlueStep(ByVal lColor1 As Long, ByVal lColor2 As Long, ByVal lWidth As Long) As Single
    GetBlueStep = (CSng(GetBlue(lColor2) - GetBlue(lColor1))) / CSng(lWidth)
End Function

Private Function GetRed(ByVal lColor As Long) As Long
    tColor.lColor = lColor
    LSet tColorBytes = tColor
    GetRed = CLng(tColorBytes.bRed)
End Function

Private Function GetGreen(ByVal lColor As Long) As Long
    tColor.lColor = lColor
    LSet tColorBytes = tColor
    GetGreen = CLng(tColorBytes.bGreen)
End Function

Private Function GetBlue(ByVal lColor As Long) As Long
    tColor.lColor = lColor
    LSet tColorBytes = tColor
    GetBlue = CLng(tColorBytes.bBlue)
End Function



