VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CmdLine"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private Const CML_PREFIX_SLASH = "/"
Private Const CML_PREFIX_MINUS = "-"
Private Const CML_PREFIX_PLUS = "+"

Public Enum CmdLinePrefixConstants
    cmlPrefixSlash
    cmlPrefixMinus
    cmlPrefixPlus
End Enum

Private Const QUOTE1 = """"
Private Const QUOTE2 = "'"

Private sCmdLine As String

Private oItems As New Collection

Public Property Get Text() As String
    Text = sCmdLine
End Property
Public Property Let Text(ByVal NewVal As String)
    sCmdLine = Trim(NewVal)
    GetCmdLineOpts
End Property

Public Property Get Count() As Long
    Count = oItems.Count
End Property

Public Property Get Item(ByVal Index As Variant) As CmdLineOpt
Attribute Item.VB_UserMemId = 0
    Set Item = oItems(Index)
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property
'
'
'
Public Sub Add(ByVal Prefix As CmdLinePrefixConstants, _
               ByVal Name As String, _
               ByVal IsOptional As Boolean, _
               ByVal MatchCase As Boolean, _
               ByVal ValueDesc As String, _
               ByVal Description As String)

    Dim oCmdLineOpt As New CmdLineOpt
    
    With oCmdLineOpt
        .Prefix = Prefix
        .Name = Name
        .IsOptional = IsOptional
        .ValueDesc = ValueDesc
        .Description = Description
        .InCmdLine = False
    End With
    
    oItems.Add oCmdLineOpt, oCmdLineOpt.Name
End Sub

Public Function Exists(ByVal Index As Variant) As Boolean
    Dim oCmdLineOpt As CmdLineOpt
    
    On Error Resume Next
    Set oCmdLineOpt = oItems(Index)
    Exists = (Err.Number = 0)
End Function

Public Sub Remove(ByVal Index As Variant)
    oItems.Remove Index
End Sub

Public Sub Clear()
    Dim l As Long
    For l = 1 To oItems.Count
        oItems.Remove 1
    Next
End Sub

'''

Private Sub GetCmdLineOpts()
    Dim oCmdLineOpt As CmdLineOpt
    Dim lLen As Long
    Dim lPos As Long
    Dim lPrefix As CmdLinePrefixConstants
    Dim sName As String
    Dim lNoNameNum As Long
    
    lLen = Len(sCmdLine)
    
    If lLen = 0 Then Exit Sub

    lNoNameNum = 1
    
    lPos = 1
    Do While lPos < lLen
        If IsPrefix(lPos, lPrefix) Then
            If lPos <= lLen Then
                sName = GetName(lPos, lLen)
                If Exists(sName) Then
                    Set oCmdLineOpt = oItems(sName)
                Else
                    Set oCmdLineOpt = New CmdLineOpt
                    oItems.Add oCmdLineOpt, sName
                End If
                
                With oCmdLineOpt
                    .Prefix = lPrefix
                    .Name = sName
                    If lPos <= lLen Then
                        .Value = GetValue(lPos, lLen)
                    End If
                    .InCmdLine = True
                End With
                
                Set oCmdLineOpt = Nothing
                
            End If
        Else
            If Mid$(sCmdLine, lPos, 1) <> " " Then
                sName = "O" + CStr(lNoNameNum)
                Set oCmdLineOpt = New CmdLineOpt
                oItems.Add oCmdLineOpt, sName
                
                lNoNameNum = lNoNameNum + 1
                
                With oCmdLineOpt
                    .Name = sName
                    If lPos <= lLen Then
                        .Value = GetValue(lPos, lLen, True)
                    End If
                    .InCmdLine = True
                End With
                
                Set oCmdLineOpt = Nothing
            Else
                lPos = lPos + 1
            End If
        End If
    Loop
End Sub

Private Function IsPrefix(lPos As Long, lPrefix As CmdLinePrefixConstants) As Boolean
    Dim bPrefix As Boolean
    
    bPrefix = False
    
    If lPos > 1 Then
        If Mid$(sCmdLine, lPos - 1, 1) = " " Then
            If lPos < Len(sCmdLine) Then
                If Mid$(sCmdLine, lPos, 1) = CML_PREFIX_MINUS Then
                    lPrefix = cmlPrefixMinus
                    bPrefix = True
                End If
            End If
        End If
    Else
        If Mid$(sCmdLine, lPos, 1) = CML_PREFIX_MINUS Then
            lPrefix = cmlPrefixMinus
            bPrefix = True
        End If
    End If
    
    If bPrefix Then
        lPos = lPos + 1
    End If
    
    IsPrefix = bPrefix
End Function

Private Function IsQuote(lPos As Long) As Boolean
    If Mid(sCmdLine, lPos, 1) = QUOTE1 Or Mid(sCmdLine, lPos, 1) = QUOTE2 Then
        lPos = lPos + 1
        IsQuote = True
    Else
        IsQuote = False
    End If
End Function

Private Function GetName(lPos As Long, lLen As Long) As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    
    Blanks lPos, lLen

    lPos1 = lPos
    Do While lPos <= lLen
        If Mid(sCmdLine, lPos, 1) <> " " Then
            lPos = lPos + 1
        Else
            Exit Do
        End If
    Loop
    
    lPos2 = lPos
    
    GetName = Mid(sCmdLine, lPos1, lPos2 - lPos1)
End Function

Private Function GetValue(lPos As Long, lLen As Long, Optional ByVal bBlankSep As Boolean = False) As String
    Dim sValue As String
    Dim bInQuote As Boolean
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim lPrefix As CmdLinePrefixConstants
    
    sValue = ""
    bInQuote = False
    
    Blanks lPos, lLen
    
    lPos1 = lPos
    
    Do While lPos <= lLen
        If IsQuote(lPos) Then
            bInQuote = Not bInQuote
        End If
        
        If IsPrefix(lPos, lPrefix) Then
            lPos = lPos - 1
            Exit Do
        End If
        
        If Mid$(sCmdLine, lPos, 1) <> " " Then
            lPos = lPos + 1
        Else
            If Not bBlankSep Then
                If bInQuote Then
                    lPos = lPos + 1
                Else
                    Blanks lPos, lLen
                End If
            Else
                If bInQuote Then
                    lPos = lPos + 1
                Else
                    Exit Do
                End If
            End If
        End If
    Loop
    
    lPos2 = lPos
    
    sValue = Trim(Mid(sCmdLine, lPos1, lPos2 - lPos1))
    
    If Left(sValue, 1) = QUOTE1 Or Left(sValue, 1) = QUOTE2 Then sValue = Mid(sValue, 2)
    If Right(sValue, 1) = QUOTE1 Or Right(sValue, 1) = QUOTE2 Then sValue = Left(sValue, Len(sValue) - 1)
    
    GetValue = sValue
End Function

Private Sub Blanks(lPos As Long, lLen As Long)
    Do While lPos <= lLen
        If Mid(sCmdLine, lPos, 1) = " " Then
            lPos = lPos + 1
        Else
            Exit Do
        End If
    Loop
End Sub

