Attribute VB_Name = "modCmdSet"
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Sub CommandsSet(oDB As DBMLib.Database)
    SetFile oDB
    SetEdit oDB
    SetView oDB
    SetInstance oDB
    SetHelp oDB
End Sub

Private Sub SetFile(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
        SetEnabled oDB, cmdFPrint, False
        SetEnabled oDB, cmdFPrintSetup, False
        
        Select Case CMDOBJ.ItemType
            Case cotCombo, cotTree:
                .Item(cmdFUnregister).Visible = False
                .Item(cmdFDelete).Visible = True
                .Item(cmdFRename).Enabled = True
                .Item(cmdFProperties).Enabled = True
            
            Case cotDatabase:
                .Item(cmdFUnregister).Visible = True
                .Item(cmdFDelete).Visible = False
                
                If oDB Is Nothing Then
                    SetEnabled oDB, cmdFUnregister, False
                    SetEnabled oDB, cmdFRename, False
                    SetEnabled oDB, cmdFProperties, False
                Else
                    If oDB.Busy Then
                        SetEnabled oDB, cmdFUnregister, False
                        SetEnabled oDB, cmdFRename, False
                        SetEnabled oDB, cmdFProperties, False
                    Else
                        SetEnabled oDB, cmdFUnregister, True
                        SetEnabled oDB, cmdFRename, True
                        SetEnabled oDB, cmdFProperties, True
                    End If
                End If
                
            Case Else:
                .Item(cmdFUnregister).Visible = False
                .Item(cmdFDelete).Visible = False
                SetEnabled oDB, cmdFRename, False
                SetEnabled oDB, cmdFProperties, False
        End Select
    End With
End Sub

Private Sub SetEdit(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
        SetEnabled oDB, cmdEUndo, False
        SetEnabled oDB, cmdECut, False
        SetEnabled oDB, cmdECopy, False
        SetEnabled oDB, cmdEPaste, False
        SetEnabled oDB, cmdEDelete, False
        
        If FRM.lvwReport.ListItems.Count > 0 Then
            .Item(cmdFMClearAll).Enabled = True
            If Not FRM.lvwReport.SelectedItem Is Nothing Then
                .Item(cmdFMProperties).Enabled = True
                .Item(cmdFMClearItem).Enabled = True
            Else
                .Item(cmdFMProperties).Enabled = False
                .Item(cmdFMClearItem).Enabled = False
            End If
        Else
            .Item(cmdFMProperties).Enabled = False
            .Item(cmdFMClearItem).Enabled = False
            .Item(cmdFMClearAll).Enabled = False
        End If
    End With
End Sub

Private Sub SetView(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
        If oDB Is Nothing Then
            SetEnabled oDB, cmdVRefresh, False
        Else
            If oDB.Busy Then
                SetEnabled oDB, cmdVRefresh, False
            Else
                SetEnabled oDB, cmdVRefresh, True
            End If
        End If
        
        If DOCS.Count > 1 Then
            .Item(cmdDocuments).Enabled = True
            .Item(cmdNextDocument).Enabled = True
            .Item(cmdPrevDocument).Enabled = True
        Else
            .Item(cmdDocuments).Enabled = False
            .Item(cmdNextDocument).Enabled = False
            .Item(cmdPrevDocument).Enabled = False
        End If
    End With
End Sub

Private Sub SetInstance(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
    
        If oDB Is Nothing Then
            SetEnabled oDB, cmdIViewDetails, False
            
            SetEnabled oDB, cmdIOffline, False
            SetEnabled oDB, cmdICold, False
            SetEnabled oDB, cmdIOnline, False
            
            SetEnabled oDB, cmdIRestartUntil, False
            
            SetEnabled oDB, cmdIDrop, False
            
            SetEnabled oDB, cmdICancel, False
            
'            SetEnabled oDB, cmdTSQLStudio, False
            
            SetEnabled oDB, cmdIDisconnect, False
            
            SetEnabled oDB, cmdDebugTrace, False
        Else
            If oDB.Busy Then
                SetEnabled oDB, cmdIViewDetails, False
                
                SetEnabled oDB, cmdIOffline, False
                SetEnabled oDB, cmdICold, False
                SetEnabled oDB, cmdIOnline, False
                
                SetEnabled oDB, cmdIRestartUntil, False
                
                SetEnabled oDB, cmdIDrop, False
                
                SetEnabled oDB, cmdICancel, True
            
'                SetEnabled oDB, cmdTSQLStudio, True
                
                SetEnabled oDB, cmdIDisconnect, False
                
                SetEnabled oDB, cmdDebugTrace, False
            Else
                If oDB.Check Then
                    Select Case oDB.state.Value
                                    
                        Case dbmDBStateOffline:
                            SetEnabled oDB, cmdIViewDetails, True
                            
                            SetEnabled oDB, cmdIOffline, False
                            SetEnabled oDB, cmdICold, True
                            SetEnabled oDB, cmdIOnline, True
        
                            SetEnabled oDB, cmdIRestartUntil, False
        
                            SetEnabled oDB, cmdIDrop, True
                            
                            SetEnabled oDB, cmdICancel, False
                            
'                            SetEnabled oDB, cmdTSQLStudio, False
                            
                        Case dbmDBStateCold:
                            SetEnabled oDB, cmdIViewDetails, True
                            
                            SetEnabled oDB, cmdIOffline, True
                            SetEnabled oDB, cmdICold, False
                            SetEnabled oDB, cmdIOnline, True
                            
                            SetEnabled oDB, cmdIRestartUntil, True
                            
                            SetEnabled oDB, cmdIDrop, False
                            
                            SetEnabled oDB, cmdICancel, False
                            
'                            SetEnabled oDB, cmdTSQLStudio, False
        
                        Case dbmDBStateWarm:
                            SetEnabled oDB, cmdIViewDetails, True
                            
                            SetEnabled oDB, cmdIOffline, True
                            SetEnabled oDB, cmdICold, True
                            SetEnabled oDB, cmdIOnline, False
            
                            SetEnabled oDB, cmdIRestartUntil, False
            
                            SetEnabled oDB, cmdIDrop, False
                            
                            SetEnabled oDB, cmdICancel, False
                            
'                            SetEnabled oDB, cmdTSQLStudio, True
        
                        Case dbmDBStateUnknown:
                            SetEnabled oDB, cmdIViewDetails, True
                            
                            SetEnabled oDB, cmdIOffline, True
                            SetEnabled oDB, cmdICold, True
                            SetEnabled oDB, cmdIOnline, True
                        
                            SetEnabled oDB, cmdIRestartUntil, True
                        
                            SetEnabled oDB, cmdIDrop, True
                            
                            SetEnabled oDB, cmdICancel, False
                            
'                            SetEnabled oDB, cmdTSQLStudio, False
                        
                        Case dbmDBStateStoppedIncorrectly:
                            SetEnabled oDB, cmdIViewDetails, True
                            
                            SetEnabled oDB, cmdIOffline, False
                            SetEnabled oDB, cmdICold, True
                            SetEnabled oDB, cmdIOnline, True
            
                            SetEnabled oDB, cmdIRestartUntil, True
            
                            SetEnabled oDB, cmdIDrop, True
                            
                            SetEnabled oDB, cmdICancel, False
                            
'                            SetEnabled oDB, cmdTSQLStudio, False
                    End Select
                    
                    SetEnabled oDB, cmdIDisconnect, True
                    
                    SetEnabled oDB, cmdDebugTrace, True
                Else
                    SetEnabled oDB, cmdIViewDetails, False
                    
                    SetEnabled oDB, cmdIOffline, False
                    SetEnabled oDB, cmdICold, False
                    SetEnabled oDB, cmdIOnline, False
                    
                    SetEnabled oDB, cmdIRestartUntil, False
                    
                    SetEnabled oDB, cmdIDrop, False
                    
                    SetEnabled oDB, cmdICancel, False
                    
'                    SetEnabled oDB, cmdTSQLStudio, False
                    
                    SetEnabled oDB, cmdIDisconnect, True
                    
                    SetEnabled oDB, cmdDebugTrace, False
                End If
            End If
        End If
    End With
    
    SetInstanceInfos oDB
    SetInstanceBackup oDB
    SetInstanceRecovery oDB
    SetInstanceTuning oDB
    SetInstanceCheck oDB
    SetInstanceConfiguration oDB
    SetInstanceAutosave oDB
End Sub

Private Sub SetInstanceInfos(oDB As DBMLib.Database)
    Dim bEnabled As Boolean

    If oDB Is Nothing Then
        bEnabled = False
        SetEnabled oDB, cmdIIHistory, False
    Else
        If oDB.Busy Then
            bEnabled = False
            SetEnabled oDB, cmdIIHistory, False
        Else
            If oDB.Check Then
                Select Case oDB.state.Value
                    Case dbmDBStateOffline, dbmDBStateStoppedIncorrectly:
                        bEnabled = False
                        SetEnabled oDB, cmdIIHistory, True
                    Case dbmDBStateCold:
                        bEnabled = False
                        SetEnabled oDB, cmdIIHistory, True
                    Case dbmDBStateWarm:
                        bEnabled = True
                        SetEnabled oDB, cmdIIHistory, True
                    Case dbmDBStateUnknown:
                        bEnabled = False
                        SetEnabled oDB, cmdIIHistory, True
                End Select
            Else
                bEnabled = False
                SetEnabled oDB, cmdIIHistory, False
            End If
        End If
    End If

    With FRM.CML.CommandItems
        SetEnabled oDB, cmdIICaches, bEnabled
        SetEnabled oDB, cmdIIData, bEnabled
        SetEnabled oDB, cmdIIIO, bEnabled
        SetEnabled oDB, cmdIILog, bEnabled
        SetEnabled oDB, cmdIILocks, bEnabled
        SetEnabled oDB, cmdIISessions, bEnabled
        SetEnabled oDB, cmdIIVersions, bEnabled
        
        SetEnabled oDB, cmdIIOMSCaches, bEnabled
        SetEnabled oDB, cmdIIOMSMonitor, bEnabled
        SetEnabled oDB, cmdIIOMSClasscontainers, bEnabled
    End With
End Sub

Private Sub SetInstanceBackup(oDB As DBMLib.Database)
    Dim oCtl As DBMDocBackup
    Dim iDoc As Integer
    
    With FRM.CML.CommandItems
        If oDB Is Nothing Then
            SetEnabled oDB, cmdIBComplete, False
            SetEnabled oDB, cmdIBIncremental, False
            SetEnabled oDB, cmdIBLog, False
            SetEnabled oDB, cmdIBAutoLog, False
        Else
            If oDB.Busy Then
                SetEnabled oDB, cmdIBComplete, False
                SetEnabled oDB, cmdIBIncremental, False
                SetEnabled oDB, cmdIBLog, False
                SetEnabled oDB, cmdIBAutoLog, False
            Else
                If oDB.Check Then
                
                    If DOCS.Exists(dbmDBackup, oDB, iDoc) Then
                        Set oCtl = DOCS.Item(iDoc).Control
                    Else
                        Set oCtl = Nothing
                    End If
                
                    Select Case oDB.state.Value
                        Case dbmDBStateOffline, dbmDBStateStoppedIncorrectly, dbmDBStateUnknown:
                            SetEnabled oDB, cmdIBComplete, False
                            SetEnabled oDB, cmdIBIncremental, False
                            SetEnabled oDB, cmdIBLog, False
                            SetEnabled oDB, cmdIBAutoLog, False
                            
                        Case dbmDBStateCold, dbmDBStateWarm:
                            If oCtl Is Nothing Then
                                SetEnabled oDB, cmdIBComplete, True
                                SetEnabled oDB, cmdIBIncremental, True
                                SetEnabled oDB, cmdIBLog, True
                                SetEnabled oDB, cmdIBAutoLog, (oDB.state.Value = dbmDBStateWarm)
                            Else
                                If oCtl.ActDialog = dbmBackupDlgMedia Then
                                    SetEnabled oDB, cmdIBComplete, True
                                    SetEnabled oDB, cmdIBIncremental, True
                                    SetEnabled oDB, cmdIBLog, True
                                    SetEnabled oDB, cmdIBAutoLog, (oDB.state.Value = dbmDBStateWarm)
                                Else
                                    SetEnabled oDB, cmdIBComplete, oCtl.Mode = dbmBackupModeData
                                    SetEnabled oDB, cmdIBIncremental, oCtl.Mode = dbmBackupModePages
                                    SetEnabled oDB, cmdIBLog, oCtl.Mode = dbmBackupModeLog
                                    SetEnabled oDB, cmdIBAutoLog, (oCtl.Mode = dbmBackupModeAuto And oDB.state.Value = dbmDBStateWarm)
                                End If
                            End If
                    End Select
                Else
                    SetEnabled oDB, cmdIBComplete, False
                    SetEnabled oDB, cmdIBIncremental, False
                    SetEnabled oDB, cmdIBLog, False
                    SetEnabled oDB, cmdIBAutoLog, False
                End If
            End If
        End If
    End With
End Sub

Private Sub SetInstanceRecovery(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
        If oDB Is Nothing Then
            SetEnabled oDB, cmdIRDatabase, False
            SetEnabled oDB, cmdIRIndexes, False
            SetEnabled oDB, cmdIRDevspaces, False
        Else
            If oDB.Busy Then
                SetEnabled oDB, cmdIRDatabase, False
                SetEnabled oDB, cmdIRIndexes, False
                SetEnabled oDB, cmdIRDevspaces, False
            Else
                If oDB.Check Then
                    Select Case oDB.state.Value
                        Case dbmDBStateOffline, dbmDBStateStoppedIncorrectly:
                            SetEnabled oDB, cmdIRDatabase, False
                            SetEnabled oDB, cmdIRIndexes, False
                            SetEnabled oDB, cmdIRDevspaces, False
                        Case dbmDBStateCold:
                            SetEnabled oDB, cmdIRDatabase, True
                            SetEnabled oDB, cmdIRIndexes, False
                            SetEnabled oDB, cmdIRDevspaces, False
                        Case dbmDBStateWarm:
                            SetEnabled oDB, cmdIRDatabase, False
                            SetEnabled oDB, cmdIRIndexes, True
                            SetEnabled oDB, cmdIRDevspaces, True
                        Case dbmDBStateUnknown:
                            SetEnabled oDB, cmdIRDatabase, False
                            SetEnabled oDB, cmdIRIndexes, False
                            SetEnabled oDB, cmdIRDevspaces, False
                    End Select
                Else
                    SetEnabled oDB, cmdIRDatabase, False
                    SetEnabled oDB, cmdIRIndexes, False
                    SetEnabled oDB, cmdIRDevspaces, False
                End If
            End If
        End If
    End With
End Sub

Private Sub SetInstanceTuning(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
        If oDB Is Nothing Then
            SetEnabled oDB, cmdITOptStat, False
            SetEnabled oDB, cmdITIndexUse, False
        Else
            If oDB.Busy Then
                SetEnabled oDB, cmdITOptStat, False
                SetEnabled oDB, cmdITIndexUse, False
            Else
                If oDB.Check Then
                    Select Case oDB.state.Value
                        Case dbmDBStateOffline, dbmDBStateStoppedIncorrectly:
                            SetEnabled oDB, cmdITOptStat, False
                            SetEnabled oDB, cmdITIndexUse, False
                        Case dbmDBStateCold:
                            SetEnabled oDB, cmdITOptStat, False
                            SetEnabled oDB, cmdITIndexUse, False
                        Case dbmDBStateWarm:
                            SetEnabled oDB, cmdITOptStat, True
                            SetEnabled oDB, cmdITIndexUse, True
                        Case dbmDBStateUnknown:
                            SetEnabled oDB, cmdITOptStat, True
                            SetEnabled oDB, cmdITIndexUse, True
                    End Select
                Else
                    SetEnabled oDB, cmdITOptStat, False
                    SetEnabled oDB, cmdITIndexUse, False
                End If
            End If
        End If
    End With
End Sub

Private Sub SetInstanceCheck(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
        If oDB Is Nothing Then
            SetEnabled oDB, cmdICFiles, False
            SetEnabled oDB, cmdICServer, False
            SetEnabled oDB, cmdICKernelTrace, False
            SetEnabled oDB, cmdICommandLine, False
            SetEnabled oDB, cmdISessions, False
            SetEnabled oDB, cmdICEventMonitor, False
            SetEnabled oDB, cmdICDatabase, False
            SetEnabled oDB, cmdICBackup, False
        Else
            If oDB.Busy Then
                SetEnabled oDB, cmdICFiles, False
                SetEnabled oDB, cmdICServer, False
                SetEnabled oDB, cmdICKernelTrace, False
                SetEnabled oDB, cmdICommandLine, False
                SetEnabled oDB, cmdISessions, False
                SetEnabled oDB, cmdICEventMonitor, False
                SetEnabled oDB, cmdICDatabase, False
                SetEnabled oDB, cmdICBackup, False
            Else
                If oDB.Check Then
                    Select Case oDB.state.Value
                        Case dbmDBStateOffline, dbmDBStateStoppedIncorrectly:
                            SetEnabled oDB, cmdICFiles, True
                            SetEnabled oDB, cmdICServer, False
                            SetEnabled oDB, cmdICKernelTrace, True
                            SetEnabled oDB, cmdICommandLine, True
                            SetEnabled oDB, cmdISessions, False
                            SetEnabled oDB, cmdICEventMonitor, False
                            SetEnabled oDB, cmdICDatabase, False
                            SetEnabled oDB, cmdICBackup, True
                        Case dbmDBStateCold:
                            SetEnabled oDB, cmdICFiles, True
                            SetEnabled oDB, cmdICServer, True
                            SetEnabled oDB, cmdICKernelTrace, True
                            SetEnabled oDB, cmdICommandLine, True
                            SetEnabled oDB, cmdISessions, False
                            SetEnabled oDB, cmdICEventMonitor, True
                            SetEnabled oDB, cmdICDatabase, True
                            SetEnabled oDB, cmdICBackup, True
                        Case dbmDBStateWarm:
                            SetEnabled oDB, cmdICFiles, True
                            SetEnabled oDB, cmdICServer, True
                            SetEnabled oDB, cmdICKernelTrace, True
                            SetEnabled oDB, cmdICommandLine, True
                            SetEnabled oDB, cmdISessions, True
                            SetEnabled oDB, cmdICEventMonitor, True
                            SetEnabled oDB, cmdICDatabase, True
                            SetEnabled oDB, cmdICBackup, True
                        Case dbmDBStateUnknown:
                            SetEnabled oDB, cmdICFiles, True
                            SetEnabled oDB, cmdICServer, True
                            SetEnabled oDB, cmdICKernelTrace, True
                            SetEnabled oDB, cmdICommandLine, True
                            SetEnabled oDB, cmdISessions, True
                            SetEnabled oDB, cmdICEventMonitor, False
                            SetEnabled oDB, cmdICDatabase, True
                            SetEnabled oDB, cmdICBackup, True
                    End Select
                Else
                    SetEnabled oDB, cmdICFiles, False
                    SetEnabled oDB, cmdICServer, False
                    SetEnabled oDB, cmdICKernelTrace, False
                    SetEnabled oDB, cmdICommandLine, False
                    SetEnabled oDB, cmdISessions, False
                    SetEnabled oDB, cmdICEventMonitor, False
                    SetEnabled oDB, cmdICDatabase, False
                    SetEnabled oDB, cmdICBackup, False
                End If
            End If
        End If
    End With
End Sub

Private Sub SetInstanceConfiguration(oDB As DBMLib.Database)
    Dim bParameters As Boolean
    Dim bDevSpaces As Boolean
    Dim iDoc As Integer

    With FRM.CML.CommandItems
        If oDB Is Nothing Then
            SetEnabled oDB, cmdICMedia, False
            SetEnabled oDB, cmdICParameters, False
            SetEnabled oDB, cmdICDevSpaceData, False
            SetEnabled oDB, cmdICDevSpaceLog, False
            SetEnabled oDB, cmdICTermcharSets, False
            SetEnabled oDB, cmdICLogMode, False
            SetEnabled oDB, cmdICMapcharSets, False
            SetEnabled oDB, cmdICLoadSysTabs, False
            SetEnabled oDB, cmdICUsers, False
            SetEnabled oDB, cmdICEvents, False
        Else
            If oDB.Busy Then
                SetEnabled oDB, cmdICMedia, False
                SetEnabled oDB, cmdICParameters, False
                SetEnabled oDB, cmdICDevSpaceData, False
                SetEnabled oDB, cmdICDevSpaceLog, False
                SetEnabled oDB, cmdICLogMode, False
                SetEnabled oDB, cmdICTermcharSets, False
                SetEnabled oDB, cmdICMapcharSets, False
                SetEnabled oDB, cmdICLoadSysTabs, False
                SetEnabled oDB, cmdICUsers, False
                SetEnabled oDB, cmdICEvents, False
            Else
                If oDB.Check Then
                    'bParameters = DOCS.Exists(dbmDParameters, oDB, iDoc)
                    'bDevSpaces = DOCS.Exists(dbmDDevSpaceData, oDB, iDoc) Or _
                    '             DOCS.Exists(dbmDDevSpaceLog, oDB, iDoc)
                    bParameters = False
                    bDevSpaces = False
                
                    Select Case oDB.state.Value
                        Case dbmDBStateOffline, dbmDBStateStoppedIncorrectly:
                            SetEnabled oDB, cmdICMedia, True
                            SetEnabled oDB, cmdICParameters, Not bDevSpaces
                            SetEnabled oDB, cmdICDevSpaceData, Not bParameters
                            SetEnabled oDB, cmdICDevSpaceLog, Not bParameters
                            SetEnabled oDB, cmdICLogMode, True
                            SetEnabled oDB, cmdICTermcharSets, False
                            SetEnabled oDB, cmdICMapcharSets, False
                            SetEnabled oDB, cmdICLoadSysTabs, False
                            SetEnabled oDB, cmdICUsers, True
                            SetEnabled oDB, cmdICEvents, False
                            
                        Case dbmDBStateCold:
                            SetEnabled oDB, cmdICMedia, True
                            SetEnabled oDB, cmdICParameters, Not bDevSpaces
                            SetEnabled oDB, cmdICDevSpaceData, Not bParameters
                            SetEnabled oDB, cmdICDevSpaceLog, Not bParameters
                            SetEnabled oDB, cmdICLogMode, True
                            SetEnabled oDB, cmdICTermcharSets, False
                            SetEnabled oDB, cmdICMapcharSets, False
                            SetEnabled oDB, cmdICLoadSysTabs, False
                            SetEnabled oDB, cmdICUsers, True
                            SetEnabled oDB, cmdICEvents, True
                        
                        Case dbmDBStateWarm:
                            SetEnabled oDB, cmdICMedia, True
                            SetEnabled oDB, cmdICParameters, Not bDevSpaces
                            SetEnabled oDB, cmdICDevSpaceData, Not bParameters
                            SetEnabled oDB, cmdICDevSpaceLog, Not bParameters
                            SetEnabled oDB, cmdICLogMode, True
                            If oDB.Version.Number < dbmversion74 Then
                                SetEnabled oDB, cmdICTermcharSets, True
                            Else
                                SetEnabled oDB, cmdICTermcharSets, False
                            End If
                            SetEnabled oDB, cmdICMapcharSets, True
                            SetEnabled oDB, cmdICLoadSysTabs, True
                            SetEnabled oDB, cmdICUsers, True
                            SetEnabled oDB, cmdICEvents, True
    
                        Case dbmDBStateUnknown:
                            SetEnabled oDB, cmdICMedia, False
                            SetEnabled oDB, cmdICParameters, False
                            SetEnabled oDB, cmdICDevSpaceData, False
                            SetEnabled oDB, cmdICDevSpaceLog, False
                            SetEnabled oDB, cmdICLogMode, False
                            SetEnabled oDB, cmdICTermcharSets, False
                            SetEnabled oDB, cmdICMapcharSets, False
                            SetEnabled oDB, cmdICLoadSysTabs, False
                            SetEnabled oDB, cmdICUsers, False
                            SetEnabled oDB, cmdICEvents, False
                    End Select
                Else
                    SetEnabled oDB, cmdICMedia, False
                    SetEnabled oDB, cmdICParameters, False
                    SetEnabled oDB, cmdICDevSpaceData, False
                    SetEnabled oDB, cmdICDevSpaceLog, False
                    SetEnabled oDB, cmdICLogMode, False
                    SetEnabled oDB, cmdICTermcharSets, False
                    SetEnabled oDB, cmdICMapcharSets, False
                    SetEnabled oDB, cmdICLoadSysTabs, False
                    SetEnabled oDB, cmdICUsers, False
                    SetEnabled oDB, cmdICEvents, False
                End If
            End If
        End If
    End With
End Sub

Public Sub SetInstanceAutosave(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
        If oDB Is Nothing Then
            SetEnabled oDB, cmdIAutoLogOn, False
            SetEnabled oDB, cmdIAutoLogOff, False
        Else
            If oDB.Busy Then
                SetEnabled oDB, cmdIAutoLogOn, False
                SetEnabled oDB, cmdIAutoLogOff, False
            Else
                If oDB.Check Then
                    Select Case oDB.state.Value
                        Case dbmDBStateOffline, dbmDBStateStoppedIncorrectly:
                            SetEnabled oDB, cmdIAutoLogOn, False
                            SetEnabled oDB, cmdIAutoLogOff, False
                        Case dbmDBStateCold:
                            SetEnabled oDB, cmdIAutoLogOn, False
                            SetEnabled oDB, cmdIAutoLogOff, False
                        Case dbmDBStateWarm:
                            If oDB.Autosave.state = dbmAutosaveOn Then
                                SetEnabled oDB, cmdIAutoLogOn, False
                                SetEnabled oDB, cmdIAutoLogOff, True
                            Else
                                SetEnabled oDB, cmdIAutoLogOn, True
                                SetEnabled oDB, cmdIAutoLogOff, False
                            End If
                        Case dbmDBStateUnknown:
                            SetEnabled oDB, cmdIAutoLogOn, False
                            SetEnabled oDB, cmdIAutoLogOff, False
                    End Select
                Else
                    SetEnabled oDB, cmdIAutoLogOn, False
                    SetEnabled oDB, cmdIAutoLogOff, False
                End If
            End If
        End If
    End With
End Sub

Private Sub SetHelp(oDB As DBMLib.Database)
    With FRM.CML.CommandItems
    End With
End Sub

'''

Private Sub SetEnabled(oDB As DBMLib.Database, _
                       ByVal lCommand As DBMGUICommandConstants, _
                       ByVal bEnabled As Boolean)
    Dim oItem As DBMCmd.CommandItem
    Dim bSet As Boolean
    
    Set oItem = FRM.CML.CommandItems.Item(lCommand)
    
    bSet = bEnabled
    
    With oItem
        If Not oDB Is Nothing Then
            If bSet Then
                If oDB.Version.Number < .VersionNumber Then
                    bSet = False
                End If
            End If
            If bSet Then
                If .InstanceType > 0 Then
                    If oDB.InstanceType <> dbmInstanceTypeNone Then
                        Select Case oDB.InstanceType
                            Case dbmInstanceTypeOLTP:   bSet = ((.InstanceType And CMD_INST_OLTP) <> 0)
                            Case dbmInstanceTypeLVC:
                                If oDB.Version.Number < dbmversion74 Then
                                    bSet = ((.InstanceType And CMD_INST_LVC) <> 0)
                                End If
                            Case dbmInstanceTypeCS:     bSet = ((.InstanceType And CMD_INST_CS) <> 0)
                            Case dbmInstanceTypeBW:     bSet = ((.InstanceType And CMD_INST_BW) <> 0)
                        End Select
                    End If
                End If
            End If
        Else
            bSet = False
        End If
        
        .Enabled = bSet
    End With
End Sub
                       


