VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RegDatabases"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

' Databases

Const KEY_DBS = "Databases"

'

Private sRoot As String
Private oReg As DBMReg.Registry
Private oKey As DBMReg.Key

'''

Public Property Let Root(NewVal As String)
    sRoot = NewVal
End Property

Public Property Set Registry(NewVal As DBMReg.Registry)
    Set oReg = NewVal
End Property

Public Property Get Key() As DBMReg.Key
    Set Key = oKey
End Property

'''

Public Sub Init()
    Set oKey = oReg.Keys.Add(HKEY_CURRENT_USER, sRoot & "\" & KEY_DBS)
    With oKey
        If Not .Check(False, False) Then
            If Not .Create(False, False) Then
                Exit Sub
            End If
        Else
            If Not .EnumKeys(False, False) Then
                Exit Sub
            Else
                Check
            End If
        End If
    End With
End Sub

Public Function Exists(Name As String) As Boolean
    Dim oKeyFound As DBMReg.Key

    Set oKeyFound = Nothing
    Set oKeyFound = oKey.FindKeyByPath(True, oKey.Path + "\" + Name)
    
    Exists = Not (oKeyFound Is Nothing)
End Function

Public Function Add(Name As String, _
                    Servername As String, _
                    Databasename As String, _
                    Username As String, _
                    Password As String, _
                    Version As String, _
                    Kernel As DBMLib.DBMDatabaseKernelConstants, _
                    InstanceType As DBMLib.DBMDatabaseInstanceTypeConstants) As DBMReg.Key
    Dim oNewKey As DBMReg.Key
    Dim oPwd As New DBMPwd.Crypt
    
    Set Add = Nothing
    
    Set oNewKey = oKey.Keys.Add(HKEY_CURRENT_USER, Name)
    
    AddValues oNewKey
    
    With oNewKey.Values
        .Item(REG_DB_VAL_SERVER).Value = Servername
        .Item(REG_DB_VAL_NAME).Value = Databasename
        .Item(REG_DB_VAL_USER).Value = Username
        .Item(REG_DB_VAL_PWD).Value = oPwd.Encrypt(Password)
        .Item(REG_DB_VAL_VER).Value = Version
        .Item(REG_DB_VAL_KERNEL).Value = Kernel
        .Item(REG_DB_VAL_AUTOREFRESH).Value = False
        .Item(REG_DB_VAL_INTERVAL).Value = 60000
        .Item(REG_DB_VAL_EVENTS).Value = False
        .Item(REG_DB_VAL_TRACE).Value = False
        .Item(REG_DB_VAL_INST).Value = InstanceType
        .Item(REG_DB_VAL_NOEXTBACKUP).Value = False
    End With
    
    If Not oNewKey.Create(False, True) Then
        Exit Function
    End If
    
    Set Add = oNewKey
End Function

Public Sub Rename(OldName As String, NewName As String)
    If oKey.Keys.Item(OldName).Rename(NewName) Then
        oKey.Keys.Rename OldName, NewName
    End If
End Sub

Public Sub Remove(Name As String)
    oKey.Keys.Item(Name).Delete
    oKey.Keys.Remove Name
End Sub

'''

Private Sub AddValues(oKeyAdd As DBMReg.Key)
    With oKeyAdd.Values
        .Add REG_DB_VAL_NAME, "", REG_SZ
        .Add REG_DB_VAL_SERVER, "", REG_SZ
        .Add REG_DB_VAL_USER, "", REG_SZ
        .Add REG_DB_VAL_PWD, "", REG_SZ
        .Add REG_DB_VAL_VER, "", REG_SZ
        .Add REG_DB_VAL_KERNEL, DBMLib.DBMDatabaseKernelConstants.dbmFastKernel, REG_DWORD
        .Add REG_DB_VAL_AUTOREFRESH, False, REG_DWORD
        .Add REG_DB_VAL_INTERVAL, 60000, REG_DWORD
        .Add REG_DB_VAL_EVENTS, False, REG_DWORD
        .Add REG_DB_VAL_TRACE, False, REG_DWORD
        .Add REG_DB_VAL_INST, 0, REG_DWORD
        .Add REG_DB_VAL_NOEXTBACKUP, False, REG_DWORD
    End With
End Sub

Private Sub Check()
    Dim oSubKey As DBMReg.Key
    Dim oValue As DBMReg.Value
    
    For Each oSubKey In oKey.Keys
    
        AddValues oSubKey
        
        With oSubKey
            If .OpenKey() Then
                For Each oValue In .Values
                    If Not oValue.CheckValue(.Key) Then
                        If Not oValue.CreateValue(.Key) Then
                            Exit Sub
                        End If
                    Else
                        If Not oValue.QueryValue(.Key) Then
                            Exit Sub
                        End If
                    End If
                Next
                .CloseKey
            End If
        End With
    Next
End Sub

