VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RegTree"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

' Tree

Const KEY_TREE = "Tree"
Const VAL_EXP = "Expanded"
Const VAL_SEL = "Selected"
Const VAL_DB = "DB"

'

Private sRoot As String
Private oReg As DBMReg.Registry
Private oRootKey As DBMReg.Key

'''

Public Property Let Root(NewVal As String)
    sRoot = NewVal
End Property

Public Property Set Registry(NewVal As DBMReg.Registry)
    Set oReg = NewVal
End Property

Public Property Get Key() As DBMReg.Key
    Set Key = oRootKey
End Property

'''

Public Sub Init()
    Set oRootKey = oReg.Keys.Add(HKEY_CURRENT_USER, sRoot + "\" + KEY_TREE)
    
    AddValues oRootKey
    
    With oRootKey
        If Not .Check(False, False) Then
            If Not .Create(False, False) Then
                Exit Sub
            End If
        Else
            If Not .EnumKeys(True, True) Then
                Exit Sub
            Else
                Check
            End If
        End If
    End With
End Sub

Public Function Exists(Key As DBMReg.Key, _
                       Name As String) As Boolean
    Dim oKeyFound As DBMReg.Key

    Set oKeyFound = Nothing
    Set oKeyFound = Key.FindKeyByPath(False, Key.Path + "\" + Name)
    
    Exists = Not (oKeyFound Is Nothing)
End Function

Public Function Find(Path As String) As DBMReg.Key
    Set Find = oRootKey.FindKeyByPath(True, Path)
End Function

Public Function Add(Key As DBMReg.Key, _
                    Name As String) As DBMReg.Key
    Dim oNewKey As DBMReg.Key
    
    Set Add = Nothing
    
    Set oNewKey = Key.Keys.Add(HKEY_CURRENT_USER, Name)
    
    AddValues oNewKey
    
    If Not oNewKey.Create(False, True) Then
        Exit Function
    End If
    
    Set Add = oNewKey
End Function

Public Function ExistsDB(Key As DBMReg.Key, _
                         Name As String) As Boolean
    Dim oValue As DBMReg.Value
    
    ExistsDB = False
    
    For Each oValue In Key.Values
        If Left$(oValue.Name, 2) = VAL_DB Then
            If UCase$(Trim$(Name)) = UCase(Trim$(oValue.Value)) Then
                ExistsDB = True
                Exit For
            End If
        End If
    Next
End Function

Public Sub AddDB(Key As DBMReg.Key, _
                 Name As String)
    Dim oValue As DBMReg.Value
    Dim iDB As Integer
    
    iDB = 0
        
    For Each oValue In Key.Values
        If Left$(oValue.Name, 2) = VAL_DB Then
            If Val(Mid$(oValue.Name, 3)) > iDB Then
                iDB = Val(Mid(oValue.Name, 3))
            End If
        End If
    Next
        
    iDB = iDB + 1
    
    Set oValue = Key.Values.Add(VAL_DB & CStr(iDB), Name, REG_SZ)
    
    If Key.OpenKey() Then
        oValue.CreateValue Key.Key
        Key.CloseKey
    End If
End Sub

Public Function FindDB(Name As String) As DBMReg.Key
    Dim oFindKey As DBMReg.Key
    
    Set oFindKey = Nothing
    
    FindDBRecursive oRootKey, Name, oFindKey
     
    Set FindDB = oFindKey
End Function

Public Sub FindDBRecursive(Key As DBMReg.Key, _
                           Name As String, _
                           FindKey As DBMReg.Key)
    Dim oValue As DBMReg.Value
    Dim oSubKey As DBMReg.Key
    
    If Not FindKey Is Nothing Then Exit Sub
    
    With Key
        For Each oValue In .Values
            If Left$(oValue.Name, 2) = VAL_DB Then
                If UCase$(Trim$(Name)) = UCase$(Trim$(oValue.Value)) Then
                    Set FindKey = Key
                    Exit For
                End If
            End If
        Next
    End With
        
    If FindKey Is Nothing Then
        For Each oSubKey In Key.Keys
            FindDBRecursive oSubKey, Name, FindKey
        Next
    End If
End Sub

Public Sub RemoveDB(Name As String)
    RemoveDBRecursive oRootKey, Name
End Sub

Public Sub RemoveDBRecursive(Key As DBMReg.Key, _
                             Name As String)
    Dim oValue As DBMReg.Value
    Dim oValueRemove As DBMReg.Value
    
    Dim oSubKey As DBMReg.Key
    
    With Key
        If .OpenKey() Then
            Set oValueRemove = Nothing
            For Each oValue In .Values
                If Left$(oValue.Name, 2) = VAL_DB Then
                    If UCase$(Trim$(Name)) = UCase$(Trim$(oValue.Value)) Then
                        If oValue.DeleteValue(Key.Key) Then
                            Set oValueRemove = oValue
                            Exit For
                        Else
                            Exit For
                        End If
                    End If
                End If
            Next
            
            If Not oValueRemove Is Nothing Then
                .Values.Remove oValueRemove.Name
            End If
            
            .CloseKey
        End If
        
        For Each oSubKey In Key.Keys
            RemoveDBRecursive oSubKey, Name
        Next
    End With
End Sub

Public Sub RenameDB(OldName As String, _
                    NewName As String)
    RenameDBRecursive oRootKey, OldName, NewName
End Sub

Public Sub RenameDBRecursive(Key As DBMReg.Key, _
                             OldName As String, _
                             NewName As String)
    Dim oValue As DBMReg.Value
    Dim oSubKey As DBMReg.Key
    
    With Key
        If .OpenKey() Then
            For Each oValue In .Values
                If Left$(oValue.Name, 2) = VAL_DB Then
                    If UCase$(Trim$(OldName)) = UCase$(Trim$(oValue.Value)) Then
                        oValue.Value = NewName
                        If Not oValue.SetValue(Key.Key) Then
                        End If
                    End If
                End If
            Next
            .CloseKey
        End If
        
        For Each oSubKey In Key.Keys
            RenameDBRecursive oSubKey, OldName, NewName
        Next
    End With
End Sub

'''

Private Sub Class_Terminate()
    If Not oRootKey.Check(True, False) Then
        If Not oRootKey.Create(True, True) Then
        
        End If
    Else
        If Not oRootKey.SetValues(True) Then
        
        End If
    End If
End Sub

Private Sub AddValues(Key As DBMReg.Key)
    Key.Values.Add VAL_EXP, CLng(True), REG_DWORD
    Key.Values.Add VAL_SEL, CLng(True), REG_DWORD
End Sub

Private Sub Check()
    CheckRecursive oRootKey
End Sub

Private Sub CheckRecursive(Key As DBMReg.Key)
    Dim oValue As DBMReg.Value
    Dim oSubKey As DBMReg.Key
    Dim bExpanded As Boolean
    Dim bSelected As Boolean
    
    bExpanded = False
    bSelected = False
    
    With Key
        If .OpenKey() Then
            For Each oValue In .Values
                Select Case oValue.Name
                    Case VAL_EXP: bExpanded = True
                    Case VAL_SEL: bSelected = True
                End Select
            Next
            
            If Not bExpanded Then
                Set oValue = .Values.Add(VAL_EXP, CLng(False), REG_DWORD)
                If Not oValue.CreateValue(.Key) Then
                    Exit Sub
                End If
            End If
                
            If Not bSelected Then
                Set oValue = .Values.Add(VAL_SEL, CLng(False), REG_DWORD)
                If Not oValue.CreateValue(.Key) Then
                    Exit Sub
                End If
            End If
            
            .CloseKey
        End If
        
        For Each oSubKey In .Keys
            CheckRecursive oSubKey
        Next
    End With
End Sub

