Attribute VB_Name = "TreeViewUtil"
' --- GPL ---
'
' Copyright (C) 1998-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private Declare Function SendMessage Lib "user32.dll" Alias "SendMessageA" ( _
    ByVal hWnd As Long, _
    ByVal Msg As Long, _
    ByVal wParam As Long, _
    lParam As Any) As Long

Const TV_FIRST = &H1100

Const TVM_GETITEMRECT = (TV_FIRST + 4)
Const TVM_GETNEXTITEM = (TV_FIRST + 10)
Const TVM_GETITEM = (TV_FIRST + 12)

Const TVGN_ROOT = &H0
Const TVGN_NEXT = &H1
Const TVGN_PREVIOUS = &H2
Const TVGN_PARENT = &H3
Const TVGN_CHILD = &H4
Const TVGN_FIRSTVISIBLE = &H5
Const TVGN_NEXTVISIBLE = &H6
Const TVGN_PREVIOUSVISIBLE = &H7
Const TVGN_DROPHILITE = &H8
Const TVGN_CARET = &H9
Const TVGN_LASTVISIBLE = &HA

Public Type TV_ITEM
   mask As Long
   hItem As Long
   state As Long
   stateMask As Long
   pszText As String
   cchTextMax As Long
   iImage As Long
   iSelectedImage As Long
   cChildren As Long
   lParam As Long
End Type


Public Sub GetItemRect(oTVW As MSComctlLib.TreeView, _
                       oNode As MSComctlLib.Node, _
                       r As RECT)
    Dim hItem As Long
    hItem = GetItem(oTVW, oNode)
    Call TreeView_GetItemRect(oTVW.hWnd, hItem, r, 1)
End Sub

'''

Public Function GetItem(oTVW As MSComctlLib.TreeView, _
                        oNode As MSComctlLib.Node) As Long
    Dim oTemp As MSComctlLib.Node
    Dim aSiblingPos() As Integer
    Dim iLevel As Integer
    Dim hItem As Long
    Dim i As Integer
    Dim iPos As Integer
    
    Set oTemp = oNode
    
    Do While (oTemp Is Nothing) = False
        iLevel = iLevel + 1
        ReDim Preserve aSiblingPos(iLevel)
        aSiblingPos(iLevel) = GetNodeSiblingPos(oTemp)
        Set oTemp = oTemp.Parent
    Loop

    hItem = TreeView_GetNextItem(oTVW.hWnd, 0, TVGN_ROOT)
    If hItem Then
        For i = iLevel To 1 Step -1
            iPos = aSiblingPos(i)
    
            Do While iPos > 1
                hItem = TreeView_GetNextItem(oTVW.hWnd, hItem, TVGN_NEXT)
                iPos = iPos - 1
            Loop
    
            If (i > 1) Then hItem = TreeView_GetNextItem(oTVW.hWnd, hItem, TVGN_CHILD)
        Next
    
        GetItem = hItem
    End If
End Function

Public Function GetNodeSiblingPos(oNode As MSComctlLib.Node) As Integer
    Dim oTemp As MSComctlLib.Node
    Dim iPos As Integer
    
    Set oTemp = oNode
    Do While (oTemp Is Nothing) = False
        iPos = iPos + 1
        Set oTemp = oTemp.Previous
    Loop
    
    GetNodeSiblingPos = iPos
End Function

'''

Private Function TreeView_GetItem(hWnd As Long, pitem As TV_ITEM) As Boolean
    TreeView_GetItem = SendMessage(hWnd, TVM_GETITEM, 0, pitem)
End Function

Private Function TreeView_GetNextItem(hWnd As Long, hItem As Long, flag As Long) As Long
    TreeView_GetNextItem = SendMessage(hWnd, TVM_GETNEXTITEM, ByVal flag, ByVal hItem)
End Function

Private Function TreeView_GetItemRect(hWnd As Long, hItem As Long, r As RECT, lItemRect As Long) As Boolean
    r.Left = hItem
    TreeView_GetItemRect = SendMessage(hWnd, TVM_GETITEMRECT, ByVal lItemRect, r)
End Function
