VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Autosave"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_SHOW = "AutosaveShow"
Const FUNC_ON = "AutosaveOn"
Const FUNC_OFF = "AutosaveOff"
Const FUNC_CANCEL = "AutosaveCancel"

'''

Const AUTOSAVE_ON = "AUTOSAVE IS ON"
Const AUTOSAVE_OFF = "AUTOSAVE IS OFF"

Public Enum DBMAutosaveStateConstants
    dbmAutosaveOn
    dbmAutosaveOff
    dbmAutosaveUnknown
End Enum

Private oParent As Object

Private dAt As Date

Private lState As DBMAutosaveStateConstants

Private sMediumName As String

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTAutosave
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get At() As Date
    At = dAt
End Property

Public Property Get State() As DBMAutosaveStateConstants
    State = lState
End Property
Friend Property Let State(ByVal NewVal As DBMAutosaveStateConstants)
    lState = NewVal
    dAt = Now()
End Property
'
' Methods
'
Public Function Refresh(Optional ByVal Asynchron As Boolean = False, _
                        Optional ByVal SenderID As Long = 0) As Boolean
    Refresh = AutosaveShow(Asynchron, SenderID)
End Function

Public Function AutosaveShow(Optional ByVal Asynchron As Boolean = False, _
                             Optional ByVal SenderID As Long = 0) As Boolean
    If Not Asynchron Then
        AutosaveShow = Execute(FUNC_SHOW, False, SenderID)
    Else
        QUEUE.QueueItems.Add Me, FUNC_SHOW, SenderID
        AutosaveShow = True
    End If
End Function

Public Function AutosaveOn(Optional ByVal MediumName As String = "", _
                           Optional ByVal Asynchron As Boolean = False, _
                           Optional ByVal SenderID As Long = 0) As Boolean
    sMediumName = MediumName
    
    If Not Asynchron Then
        AutosaveOn = Execute(FUNC_ON, False, SenderID)
    Else
        QUEUE.QueueItems.Add Me, FUNC_ON, SenderID
        AutosaveOn = True
    End If
End Function

Public Function AutosaveOff(Optional ByVal Asynchron As Boolean = False, _
                            Optional ByVal SenderID As Long = 0) As Boolean
    If Not Asynchron Then
        AutosaveOff = Execute(FUNC_OFF, False, SenderID)
    Else
        QUEUE.QueueItems.Add Me, FUNC_OFF, SenderID
        AutosaveOff = True
    End If
End Function

Public Function AutosaveCancel(Optional ByVal Asynchron As Boolean = False, _
                               Optional ByVal SenderID As Long = 0) As Boolean
    If Database.State.Value <> dbmDBStateWarm Then Exit Function

    If Not Asynchron Then
        AutosaveCancel = Execute(FUNC_CANCEL, False, SenderID)
    Else
        QUEUE.QueueItems.Add Me, FUNC_CANCEL, SenderID
        AutosaveCancel = True
    End If
End Function

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bRC As Boolean
    
    Database.Busy = True
    
    bRC = False
    
    Select Case FunctionName
        Case FUNC_SHOW:     bRC = ExecuteShow(Asynchron, SenderID)
        Case FUNC_ON:       bRC = ExecuteOn(Asynchron, SenderID)
        Case FUNC_OFF:      bRC = ExecuteOff(Asynchron, SenderID)
        Case FUNC_CANCEL:   bRC = ExecuteCancel(Asynchron, SenderID)
    End Select
    
    Execute = bRC
    
    Database.Busy = False
End Function

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
    dAt = Empty
    lState = dbmAutosaveUnknown
End Sub

Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Private Function ExecuteShow(ByVal bAsynchron As Boolean, ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim sState As String
    
    bOK = False
    
    With Database
        If .Check() Then
            If .UtilConnect Then
                If DBMSRVAutosave.AutosaveShow(.ServerObject, sState) Then
                    dAt = Now()
                    Select Case sState
                        Case AUTOSAVE_ON:   lState = dbmAutosaveOn
                        Case AUTOSAVE_OFF:  lState = dbmAutosaveOff
                    End Select
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
                .UtilRelease
            End If
        End If
        
        If bAsynchron Then
            .EventAutosaveAction lSenderID, Not bOK, dbmAutosaveActioShow
        End If
    End With
End Function

Private Function ExecuteOn(ByVal bAsynchron As Boolean, ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If .UtilConnect Then
                If DBMSRVAutosave.AutosaveOn(.ServerObject, sMediumName) Then
                    lState = dbmAutosaveOn
                    .State.Autosave = True
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
                .UtilRelease
            End If
        End If
        
        If bAsynchron Then
            .EventAutosaveAction lSenderID, Not bOK, dbmAutosaveActionOn
        End If
    End With
End Function

Private Function ExecuteOff(ByVal bAsynchron As Boolean, ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If .UtilConnect Then
                If DBMSRVAutosave.AutosaveOff(.ServerObject) Then
                    lState = dbmAutosaveOff
                    .State.Autosave = False
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
                .UtilRelease
            End If
        End If
        
        If bAsynchron Then
            .EventAutosaveAction lSenderID, Not bOK, dbmAutosaveActionOff
        End If
    End With
End Function

Private Function ExecuteCancel(ByVal bAsynchron As Boolean, ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If .UtilConnect Then
                If DBMSRVAutosave.AutosaveCancel(.ServerObject) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
                .UtilRelease
            End If
        End If
        
        If bAsynchron Then
            .EventAutosaveAction lSenderID, Not bOK, dbmAutosaveActionCancel
        End If
    End With
End Function

