Attribute VB_Name = "DBMSRVBackup"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Function Save(oBackup As DBMLib.Backup, _
                     oServerObject As DBMSOB.ServerObject, _
                     oMedia As Object, _
                     ByVal lSaveType As DBMBackupTypeConstants, _
                     ByVal lSaveFor As DBMBackupForConstants) As Boolean
    Dim sData As String
    
    Save = False
    
    If lSaveType = dbmBTAuto Then
        lSaveType = dbmBTLog
    End If
    
    If oServerObject.Command(DBMSRV_CMD_BACKUP + " " + GetName(oMedia) + " " + oBackup.BackupTypeString(lSaveType) + " " + oBackup.BackupForString(lSaveFor), sData) Then
        oMedia.Database.Backup.BackupResult.Data = sData
        Save = True
    End If
End Function

Public Function SaveReplace(oServerObject As DBMSOB.ServerObject, _
                            oMedium As DBMLib.Medium)
    Dim sCmd As String
    Dim sData As String
    
    SaveReplace = False
    
    sCmd = DBMSRV_CMD_BACKUP_REPLACE + " " + GetName(oMedium)
    If Len(oMedium.NewLocation) > 0 Then
        sCmd = sCmd + " " + """" + oMedium.NewLocation + """"
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        oMedium.Database.Backup.BackupResult.Data = sData
        SaveReplace = True
    End If
End Function

Public Function SaveIgnore(oServerObject As DBMSOB.ServerObject, _
                           oDB As DBMLib.Database)
    Dim sData As String
    
    SaveIgnore = False
    
    If oServerObject.Command(DBMSRV_CMD_BACKUP_IGNORE, sData) Then
        oDB.Backup.BackupResult.Data = sData
        SaveIgnore = True
    End If
End Function

Public Function SaveCancel(oServerObject As DBMSOB.ServerObject, _
                           oDB As DBMLib.Database)
    Dim sData As String
    
    SaveCancel = False
    
    If oServerObject.Command(DBMSRV_CMD_BACKUP_CANCEL, sData) Then
        oDB.Backup.BackupResult.Data = sData
        SaveCancel = True
    End If
End Function

Public Function SaveState(oServerObject As DBMSOB.ServerObject, _
                          oBackupState As DBMLib.BackupState)
    Dim sData As String
    
    SaveState = False
    
    If oServerObject.Command(DBMSRV_CMD_BACKUP_STATE, sData) Then
        oBackupState.Data = sData
        SaveState = True
    End If
End Function

'''

Public Function Restore(oBackup As DBMLib.Backup, _
                        oServerObject As DBMSOB.ServerObject, _
                        oMedia As Object, _
                        ByVal lRestoreType As DBMBackupTypeConstants, _
                        ByVal lFileNumber As Long, _
                        ByVal bUseUntilDate As Boolean, _
                        ByVal dUntilDate As Date) As Boolean
    Dim sCmd As String
    Dim sType As String
    Dim sData As String
    
    Restore = False
    
    If lRestoreType = dbmBTAuto Then
        lRestoreType = dbmBTLog
    End If
    
    sCmd = DBMSRV_CMD_RESTORE
    
    sCmd = sCmd + " " + GetName(oMedia)
           
    sType = oBackup.BackupTypeString(lRestoreType)
    
    sCmd = sCmd + " " + sType
           
    sCmd = sCmd + " " + GetExternalID(oMedia)
           
    If lFileNumber > 0 Then
        sCmd = sCmd + " " + Format$(lFileNumber, "000")
    End If
    
    If bUseUntilDate Then
        sCmd = sCmd + " UNTIL " + Format$(dUntilDate, "YYYYMMDD") + " " + Format$(dUntilDate, "HHNNSS")
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        oMedia.Database.Backup.BackupResult.Data = sData
        Restore = True
    End If
End Function

Public Function RestoreReplace(oServerObject As DBMSOB.ServerObject, _
                               oMedium As DBMLib.Medium, _
                               ByVal lFileNumber As Long)
    Dim sData As String
    Dim sCmd As String
    
    RestoreReplace = False
    
    sCmd = DBMSRV_CMD_RESTORE_REPLACE + " " + GetName(oMedium)
    
    sCmd = sCmd + " " + GetExternalID(oMedium)

    If Len(oMedium.NewLocation) > 0 Then
        sCmd = sCmd + " " + """" + oMedium.NewLocation + """"
    Else
        sCmd = sCmd + " " + """" + oMedium.Location + """"
    End If
        
    If lFileNumber > 0 Then
        sCmd = sCmd + " " + Format$(lFileNumber, "000")
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        oMedium.Database.Backup.BackupResult.Data = sData
        RestoreReplace = True
    End If
End Function

Public Function RestoreIgnore(oServerObject As DBMSOB.ServerObject, _
                              oDB As DBMLib.Database)
    Dim sData As String
    
    RestoreIgnore = False
    
    If oServerObject.Command(DBMSRV_CMD_RESTORE_IGNORE, sData) Then
        oDB.Backup.BackupResult.Data = sData
        RestoreIgnore = True
    End If
End Function

Public Function RestoreCancel(oServerObject As DBMSOB.ServerObject, _
                              oDB As DBMLib.Database)
    Dim sData As String
    
    RestoreCancel = False
    
    If oServerObject.Command(DBMSRV_CMD_RESTORE_CANCEL, sData) Then
        oDB.Backup.BackupResult.Data = sData
        RestoreCancel = True
    End If
End Function

Public Function RestoreState(oServerObject As DBMSOB.ServerObject, _
                             oBackupState As DBMLib.BackupState)
    Dim sCmd As String
    Dim sData As String
    
    RestoreState = False
    
    If oServerObject.Command(DBMSRV_CMD_RESTORE_STATE, sData) Then
        oBackupState.Data = sData
        RestoreState = True
    End If
End Function

'''

Public Function Check(oBackup As DBMLib.Backup, _
                      oServerObject As DBMSOB.ServerObject, _
                      oMedia As Object, _
                      ByVal lCheckType As DBMBackupTypeConstants, _
                      ByVal lFileNumber As Long) As Boolean
    Dim sCmd As String
    Dim sType As String
    Dim sData As String
    
    Check = False
    
    If lCheckType = dbmBTAuto Then
        lCheckType = dbmBTLog
    End If
    
    sCmd = DBMSRV_CMD_RESTORE_CHECK
    
    sCmd = sCmd + " " + GetName(oMedia)
           
    sType = oBackup.BackupTypeString(lCheckType)
    
    sCmd = sCmd + " " + sType
           
    If lFileNumber > 0 Then
        sCmd = sCmd + " " + Format$(lFileNumber, "000")
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        oMedia.Database.Backup.BackupResult.Data = sData
        Check = True
    End If
End Function

Public Function CheckReplace(oServerObject As DBMSOB.ServerObject, _
                             oMedium As DBMLib.Medium, _
                             ByVal lFileNumber As Long)
    Dim sData As String
    Dim sCmd As String
    
    CheckReplace = False
    
    sCmd = DBMSRV_CMD_RESTORE_REPLACE + " " + GetName(oMedium)
    If Len(oMedium.NewLocation) > 0 Then
        sCmd = sCmd + " " + """" + oMedium.NewLocation + """"
    Else
        sCmd = sCmd + " " + """" + oMedium.Location + """"
    End If
        
    If lFileNumber > 0 Then
        sCmd = sCmd + " " + Format$(lFileNumber, "000")
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        oMedium.Database.Backup.BackupResult.Data = sData
        CheckReplace = True
    End If
End Function

Public Function CheckIgnore(oServerObject As DBMSOB.ServerObject, _
                            oDB As DBMLib.Database)
    Dim sData As String
    
    CheckIgnore = False
    
    If oServerObject.Command(DBMSRV_CMD_RESTORE_IGNORE, sData) Then
        oDB.Backup.BackupResult.Data = sData
        CheckIgnore = True
    End If
End Function

Public Function CheckCancel(oServerObject As DBMSOB.ServerObject, _
                            oDB As DBMLib.Database)
    Dim sData As String
    
    CheckCancel = False
    
    If oServerObject.Command(DBMSRV_CMD_RESTORE_CANCEL, sData) Then
        oDB.Backup.BackupResult.Data = sData
        CheckCancel = True
    End If
End Function

Public Function CheckState(oServerObject As DBMSOB.ServerObject, _
                           oBackupState As DBMLib.BackupState)
    Dim sData As String
    
    CheckState = False
    
    If oServerObject.Command(DBMSRV_CMD_RESTORE_STATE_CHECK, sData) Then
        oBackupState.Data = sData
        CheckState = True
    End If
End Function

'''

Public Function BackupCommand(oBackup As DBMLib.Backup, _
                              oServerObject As DBMSOB.ServerObject, _
                              sCommand As String) As Boolean
    Dim sCmd As String
    Dim sType As String
    Dim sData As String
    
    BackupCommand = False
    
    sCmd = DBMSRV_CMD_BACKUP_COMMAND + " " + """" + sCommand + """"
           
    If oServerObject.Command(sCmd, sData) Then
        oBackup.BackupResult.Data = sData
        BackupCommand = True
    End If
End Function

'''

Public Function RestartInfo(oServerObject As DBMSOB.ServerObject, _
                            oRestartInfo As DBMLib.RestartInfo) As Boolean
    Dim sData As String
    
    RestartInfo = False
    
    If oServerObject.Command(DBMSRV_CMD_BACKUP_RESTART_INFO, sData) Then
        oRestartInfo.Data = sData
        RestartInfo = True
    End If
End Function

Public Function Label(oServerObject As DBMSOB.ServerObject, _
                      oMedium As DBMLib.Medium, _
                      ByVal lFileNumber As Long) As Boolean
    Dim sCmd As String
    Dim sData As String

    Label = False

    If DBMSRVUtil.Connect(oServerObject) Then
        
        sCmd = DBMSRV_CMD_BACKUP_LABEL + " """ + oMedium.ReadLabelName + """"
        
        If lFileNumber > 0 Then
            sCmd = sCmd + " " + Format$(lFileNumber, "000")
        End If
    
        If oServerObject.Command(sCmd, sData) Then
            oMedium.BackupResult.Data = sData
            Label = True
        End If
        DBMSRVUtil.Release oServerObject
    End If
End Function

Public Function LabelOffline(oServerObject As DBMSOB.ServerObject, _
                             oMedium As DBMLib.Medium, _
                             ByVal lFileNumber As Long) As Boolean
    Dim sCmd As String
    Dim sData As String

    LabelOffline = False

    sCmd = DBMSRV_CMD_BACKUP_LABELOFFLINE + " """ + oMedium.ReadLabelName + """"
    
    If lFileNumber > 0 Then
        sCmd = sCmd + " " + Format$(lFileNumber, "000")
    End If

    If oServerObject.Command(sCmd, sData) Then
        oMedium.BackupResult.Data = sData
        LabelOffline = True
    End If
End Function

Public Sub Cancel(oServerObject As DBMSOB.ServerObject)
    oServerObject.Cancel
End Sub

'''

Public Function RestoreConfig(oServerObject As DBMSOB.ServerObject, _
                              sMediumName As String) As Boolean
    Dim sCmd As String
    Dim sData As String

    RestoreConfig = False

    sCmd = DBMSRV_CMD_RESTORE_CONFIG + " " + sMediumName
    
    If oServerObject.Command(sCmd, sData) Then
        RestoreConfig = True
    End If
End Function

'''

Private Function GetName(oMed As Object) As String
    Dim sName As String
    
    sName = ""
    
    If Not oMed Is Nothing Then
        If oMed.ObjectType = dbmOTMediaGroup Then
            sName = oMed.Name
        Else
            If oMed.ObjectType = dbmOTMedium Then
                If Not oMed.Parent Is Nothing Then
                    If oMed.Parent.ObjectType = dbmOTMediaGroup Then
                        sName = oMed.Parent.Name + "\" + oMed.Name
                    Else
                        sName = oMed.Name
                    End If
                End If
            End If
        End If
        sName = """" + sName + """"
    End If
    
    GetName = sName
End Function

Private Function GetExternalID(oMed As Object) As String
    Dim sID As String
    Dim oMedium As Medium
    
    sID = ""
    
    If Not oMed Is Nothing Then
        If oMed.ObjectType = dbmOTMediaGroup Then
            For Each oMedium In oMed.Media
                If Not oMedium.ExternalID Is Nothing Then
                    sID = sID + oMedium.ExternalID.OptionText + ","
                End If
            Next
            If Len(sID) > 0 Then
                sID = Left$(sID, Len(sID) - 1)
            End If
        Else
            If oMed.ObjectType = dbmOTMedium Then
                If Not oMed.ExternalID Is Nothing Then
                    sID = oMed.ExternalID.OptionText
                End If
            End If
        End If
    End If
    
    If Len(sID) > 0 Then
        sID = "EBID """ + sID + """"
    End If
    
    GetExternalID = sID
End Function

