Attribute VB_Name = "DBMSRVEve"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const DBMSRV_CMD_EVENT_LIST = "event_list"

Public Const DBMSRV_CMD_EVENT_SET = "event_set"
Public Const DBMSRV_CMD_EVENT_DELETE = "event_delete"

Public Const EVENT_EMPTY = "EMPTY"
Public Const EVENT_ALIVE = "ALIVE"
Public Const EVENT_AUTOSAVE = "AUTO_SAVE"
Public Const EVENT_BACKUP_PAGES = "BACKUP_PAGES"
Public Const EVENT_DC_LOG_HITRATE = "DC_LOG_HITRATE"
Public Const EVENT_DC_OMSLOG_HITRATE = "DC_OMSLOG_HITRATE"
Public Const EVENT_DC_OMS_HITRATE = "DC_OMS_HITRATE"
Public Const EVENT_DB_ABOVE_LIMIT = "DB_ABOVE_LIMIT"
Public Const EVENT_DB_BELOW_LIMIT = "DB_BELOW_LIMIT"
Public Const EVENT_DB_FILL_CHANGE = "DB_FILL_CHANGE"
Public Const EVENT_DW_COMPRESS = "DW_COMPRESS"
Public Const EVENT_DW_DISPLACE = "DW_DISPLACE"
Public Const EVENT_EVENT = "EVENT"
Public Const EVENT_LOG_ABOVE_LIMIT = "LOG_ABOVE_LIMIT"
Public Const EVENT_LOG_FILL_CHANGE = "LOG_FILL_CHANGE"
Public Const EVENT_MIGRATION_PROG = "MIGRATION_PROGRESS"
Public Const EVENT_RESTART_SHUTDOWN = "RESTART_SHUTDOWN"
Public Const EVENT_SAVEPOINT = "SAVEPOINT"
Public Const EVENT_SESSIONS = "SESSIONS"
Public Const EVENT_UPD_STAT_WANTED = "UPDATE_STATISTICS"
Public Const EVENT_WARNING = "WARNING"
Public Const EVENT_ERROR = "ERROR"

Public Function EventList(oServerObject As DBMSOB.ServerObject, _
                          oEvents As DatabaseEvents) As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    
    Dim sName As String
    Dim sState As String
    Dim sPriority As String
    Dim sValue1 As String
    Dim sValue2 As String
    Dim sDescription As String
    
    Dim lValue1 As Long
    Dim lValue2 As Long
    
    Dim oEvent As DatabaseEvent
    
    EventList = False
        
    If oServerObject.Command(DBMSRV_CMD_EVENT_LIST, sData) Then
        EventList = True
        
        lPos1 = 1
        sLine = Trim(oServerObject.GetLine(sData, lPos1)) ' Headerline
        Do While lPos1 < Len(sData)
            sLine = Trim(oServerObject.GetLine(sData, lPos1))
            If Len(sLine) > 0 Then
                lPos2 = 1
                sName = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                sState = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                sPriority = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                sValue1 = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                sValue2 = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                sDescription = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                        
                If sValue1 = "" Then
                    lValue1 = -1
                Else
                    lValue1 = Val(sValue1)
                End If
                
                If sValue2 = "" Then
                    lValue2 = -1
                Else
                    lValue2 = Val(sValue2)
                End If
                
                Select Case sName
                    Case EVENT_DB_ABOVE_LIMIT:
                        If oEvents.GetStateValue(sState) = dbmDBEventStateOn Then
                            oEvents.EventDataAbove.AddItem sState, sPriority, lValue1, lValue2
                        End If

                    Case EVENT_DB_BELOW_LIMIT
                        If oEvents.GetStateValue(sState) = dbmDBEventStateOn Then
                            oEvents.EventDataBelow.AddItem sState, sPriority, lValue1, lValue2
                        End If
                    
                    Case EVENT_LOG_ABOVE_LIMIT:
                        If oEvents.GetStateValue(sState) = dbmDBEventStateOn Then
                            oEvents.EventLogAbove.AddItem sState, sPriority, lValue1, lValue2
                        End If
                    
                    Case Else:
                        Set oEvent = New DatabaseEvent
                        Set oEvent.Parent = oEvents
                        With oEvent
                            .Name = sName
                            .StateString = sState
                            .PriorityString = sPriority
                            .Value1 = lValue1
                            .Value2 = lValue2
                            .Description = sDescription
                            .EventType = GetEventType(sName)
                        End With
                        oEvents.Add oEvent
                End Select
            End If
        Loop
    End If
End Function

'''

Public Function GetEventType(sName As String) As DBMDatabaseEventTypeConstants
    Dim lType As DBMDatabaseEventTypeConstants
    
    Select Case UCase$(sName)
        Case EVENT_EMPTY: lType = dbmEventEMPTY
        Case EVENT_ALIVE: lType = dbmEventALIVE
        Case EVENT_AUTOSAVE: lType = dbmEventAUTOSAVE
        Case EVENT_BACKUP_PAGES: lType = dbmEventBACKUP_PAGES
        Case EVENT_DC_LOG_HITRATE: lType = dbmEventDC_LOG_HITRATE
        Case EVENT_DC_OMSLOG_HITRATE: lType = dbmEventDC_OMSLOG_HITRATE
        Case EVENT_DC_OMS_HITRATE: lType = dbmEventDC_OMS_HITRATE
        Case EVENT_DB_ABOVE_LIMIT: lType = dbmEventDB_ABOVE_LIMIT
        Case EVENT_DB_BELOW_LIMIT: lType = dbmEventDB_BELOW_LIMIT
        Case EVENT_DB_FILL_CHANGE: lType = dbmEventDB_FILL_CHANGE
        Case EVENT_DW_COMPRESS: lType = dbmEventDW_COMPRESS
        Case EVENT_DW_DISPLACE: lType = dbmEventDW_DISPLACE
        Case EVENT_EVENT: lType = dbmEventEVENT
        Case EVENT_LOG_ABOVE_LIMIT: lType = dbmEventLOG_ABOVE_LIMIT
        Case EVENT_LOG_FILL_CHANGE: lType = dbmEventLOG_FILL_CHANGE
        Case EVENT_MIGRATION_PROG: lType = dbmEventMIGRATION_PROGRESS
        Case EVENT_RESTART_SHUTDOWN: lType = dbmEventRESTART_SHUTDOWN
        Case EVENT_SAVEPOINT: lType = dbmEventSAVEPOINT
        Case EVENT_SESSIONS: lType = dbmEventSESSIONS
        Case EVENT_UPD_STAT_WANTED: lType = dbmEventUPD_STAT_WANTED
        Case EVENT_WARNING: lType = dbmEventWARNING
        Case EVENT_ERROR: lType = dbmEventERROR
        Case Else: lType = dbmEventUNKNOWN
    End Select
    
    GetEventType = lType
End Function
