Attribute VB_Name = "DBMSRVFile"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const DBMSRV_CMD_FILE_READFIRST = "file_readfirst"
Const DBMSRV_CMD_FILE_READNEXT = "file_readnext"
Const DBMSRV_CMD_FILE_WRITEFIRST = "file_writefirst"
Const DBMSRV_CMD_FILE_WRITENEXT = "file_writenext"
Const DBMSRV_CMD_FILE_CLOSE = "file_close"

Const DBMSRV_CMD_FILE_GETLIST = "file_getlist"
Const DBMSRV_CMD_FILE_GETFIRST = "file_getfirst"
Const DBMSRV_CMD_FILE_GETNEXT = "file_getnext"

Const DBMSRV_OPT_FILE_ASCII = " -a "
Const DBMSRV_OPT_FILE_BINARY = " -b "

Const DBMSRV_OPT_FILE_STATE_CON = "CONTINUE"
Const DBMSRV_OPT_FILE_STATE_END = "END"

'''

Private Type DBMSRV_FILE_READ_TYPE
    sFile As String
    bByID As Boolean
    lPos As Long
    lHandle As Long
    sState As String
    lSize As Long
    lPart As Long
    sData As String
    lDataLen As Long
End Type

Private tFileRead As DBMSRV_FILE_READ_TYPE

'''

Private Type DBMSRV_FILE_WRITE_TYPE
    lHandle As Long
End Type

Private tFileWrite As DBMSRV_FILE_WRITE_TYPE

'''

' FileRead

Public Function FileReadOpen(ByVal oServerObject As DBMSOB.ServerObject, _
                             ByVal sRemoteFile As String, _
                             Optional ByVal bByID As Boolean) As Boolean

    Dim sCmd As String
    
    FileReadOpen = False

    tFileRead.lHandle = 0
    tFileRead.sFile = sRemoteFile
    tFileRead.bByID = bByID
    
    If Not bByID Then
        sCmd = DBMSRV_CMD_FILE_READFIRST & DBMSRV_OPT_FILE_ASCII & sRemoteFile
    Else
        sCmd = DBMSRV_CMD_FILE_GETFIRST & " " & sRemoteFile
    End If
    
    If oServerObject.Command(sCmd, tFileRead.sData) Then
        FileReadPackedHead oServerObject
        FileReadOpen = True
    End If
End Function

Public Function FileReadEOF() As Boolean
    FileReadEOF = (tFileRead.lPos > Len(tFileRead.sData))
End Function

Public Function FileReadLine(ByVal oServerObject As DBMSOB.ServerObject, _
                             sLine As String) As Boolean

    Dim bEnd As Boolean
    Dim sRest As String

    FileReadLine = False

    If FileReadGetLine(sLine, bEnd) Then
        If bEnd Then
            If FileReadNext(oServerObject) Then
                If FileReadGetLine(sRest, bEnd) Then
                    sLine = sLine & sRest
                End If
            End If
        End If
        FileReadLine = True
    End If
End Function

Public Function FileReadText(ByVal oServerObject As DBMSOB.ServerObject, _
                             sText As String) As Boolean

    FileReadText = False

    If FileReadGetText(sText) Then
        If FileReadNext(oServerObject) Then
        End If
        FileReadText = True
    End If
End Function


Public Function FileReadClose(ByVal oServerObject As DBMSOB.ServerObject) As Boolean

    Dim sData As String
    
    FileReadClose = False

    If Not FileReadEOF() Then
        If oServerObject.Command(DBMSRV_CMD_FILE_CLOSE & " " & tFileRead.lHandle, sData) Then
            tFileRead.lHandle = 0
            FileReadClose = True
        End If
    Else
        FileReadClose = True
    End If
End Function

' FileWrite

Public Function FileWriteOpen(ByVal oServerObject As DBMSOB.ServerObject, _
                              ByVal sRemoteFile As String) As Boolean
    
    Dim sOption As String
    Dim sData As String
    Dim lPos As Long
    
    FileWriteOpen = False
    
    tFileWrite.lHandle = 0
    
    sOption = DBMSRV_OPT_FILE_ASCII
    
    If oServerObject.Command(DBMSRV_CMD_FILE_WRITEFIRST & sOption & sRemoteFile, sData) Then
        lPos = 1
        tFileWrite.lHandle = CLng(Trim(oServerObject.GetLine(sData, lPos)))
        FileWriteOpen = True
    End If
End Function

Public Function FileWriteLine(ByVal oServerObject As DBMSOB.ServerObject, _
                              ByVal sLine As String) As Boolean
    
    Dim sOption As String
    Dim sCommand As String
    Dim sData As String
    
    FileWriteLine = False
    
    sOption = DBMSRV_OPT_FILE_ASCII
    
    sCommand = DBMSRV_CMD_FILE_WRITENEXT & sOption & tFileWrite.lHandle & vbLf
    sCommand = sCommand & CStr(Len(sLine)) & vbLf & sLine
    
    If oServerObject.Command(sCommand, sData) Then
        FileWriteLine = True
    End If
End Function

Public Function FileWriteClose() As Boolean
    Dim sData As String

    FileWriteClose = False
    
    If Command(DBMSRV_CMD_FILE_CLOSE & " " & tFileWrite.lHandle, sData) Then
        tFileWrite.lHandle = 0
        FileWriteClose = True
    End If
End Function

'''

Public Function FileGetFiles(ByVal oServerObject As DBMSOB.ServerObject, _
                             ByVal oFiles As Files) As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    
    Dim sDate As String
    Dim sTime As String
    
    Dim oFile As File
    
    FileGetFiles = False
    
    If oServerObject.Command(DBMSRV_CMD_FILE_GETLIST, sData) Then
        FileGetFiles = True
        lPos1 = 1
        sLine = Trim(oServerObject.GetLine(sData, lPos1)) ' Headerline
        Do While lPos1 < Len(sData)
            sLine = Trim(oServerObject.GetLine(sData, lPos1))
            If Len(sLine) > 0 Then
                lPos2 = 1
                Set oFile = New File
                With oFile
                    .Name = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                    .FileTypeString = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                    .Size = Val(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
                    
                    sDate = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                    sTime = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                    .Modified = DateSerial(Val(Left(sDate, 4)), Val(Mid(sDate, 5, 2)), Val(Mid(sDate, 7, 2))) + _
                                TimeSerial(Val(Left(sTime, 2)), Val(Mid(sTime, 3, 2)), Val(Mid(sTime, 5, 2)))
                    .Description = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                    .Filename = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                End With
                oFiles.Add oFile, oFile.Name
            End If
        Loop
    End If
End Function

'''

Private Sub FileReadPackedHead(ByVal oServerObject As DBMSOB.ServerObject)

    Dim sLine As String
    
    With tFileRead
        .lPos = 1
        .lHandle = CLng(Trim(oServerObject.GetLine(.sData, .lPos)))
        .sState = Trim(oServerObject.GetLine(.sData, .lPos))
        sLine = oServerObject.GetLine(.sData, .lPos)
        .lSize = CLng(Trim(Left(sLine, 20)))
        .lPart = CLng(Trim(Right(sLine, 20)))
    End With
End Sub

Private Function FileReadNext(ByVal oServerObject As DBMSOB.ServerObject) As Boolean
    Dim sCmd As String
    
    FileReadNext = False

    If tFileRead.sState = DBMSRV_OPT_FILE_STATE_CON Then
        If Not tFileRead.bByID Then
            sCmd = DBMSRV_CMD_FILE_READNEXT & DBMSRV_OPT_FILE_ASCII & tFileRead.lHandle
        Else
            sCmd = DBMSRV_CMD_FILE_GETNEXT & " " & tFileRead.sFile & " " & tFileRead.lHandle
        End If
        If oServerObject.Command(sCmd, tFileRead.sData) Then
            FileReadPackedHead oServerObject
            FileReadNext = True
        End If
    End If
End Function

Private Function FileReadGetLine(sLine As String, bEnd As Boolean) As Boolean
    Dim lPos As Long
    
    FileReadGetLine = False
    
    sLine = ""
    bEnd = False
    
    With tFileRead
        If .lPos <= Len(.sData) Then
            lPos = InStr(.lPos, .sData, vbLf)
            If lPos > 0 Then
                If Mid(.sData, lPos - 1, 1) = vbCr Then
                    sLine = Mid(.sData, .lPos, lPos - .lPos - 1)
                Else
                    sLine = Mid(.sData, .lPos, lPos - .lPos)
                End If
                .lPos = lPos + 1
            Else
                sLine = Mid(.sData, .lPos)
                .lPos = Len(.sData) + 1
                bEnd = True
            End If
            FileReadGetLine = True
        End If
    End With
    
End Function

Private Function FileReadGetText(sText As String) As Boolean
    Dim lPos As Long
    
    FileReadGetText = False
    
    sText = ""
    
    With tFileRead
        If .lPos <= Len(.sData) Then
            sText = Mid(.sData, .lPos)
            .lPos = Len(.sData) + 1
            FileReadGetText = True
        End If
    End With
End Function


