Attribute VB_Name = "DBMSRVHistory"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Public Const BH_COL_SYSKEY = 1
Public Const BH_COL_LABEL = 2
Public Const BH_COL_ACTION = 3
Public Const BH_COL_DBSTAMP1 = 4
Public Const BH_COL_DBSTAMP2 = 5
Public Const BH_COL_STARTDATE = 6
Public Const BH_COL_STOPDATE = 7
Public Const BH_COL_FROMPAGE = 8
Public Const BH_COL_TOPAGE = 9
Public Const BH_COL_LOGREQUIRED = 10
Public Const BH_COL_MEDIANAME = 11
Public Const BH_COL_PAGESTRANSFERED = 12
Public Const BH_COL_VOLUMES = 13
Public Const BH_COL_RETURNCODE = 14
Public Const BH_COL_RETURNTEXT = 15

Const BH_COL_MAX = BH_COL_RETURNTEXT

Public BHColPos(1 To BH_COL_MAX) As Integer
Public BHColLen(1 To BH_COL_MAX) As Integer

Public Const BM_COL_SYSKEY = 1
Public Const BM_COL_LABEL = 2
Public Const BM_COL_MEDIANAME = 3
Public Const BM_COL_DEVICETYPE = 4
Public Const BM_COL_OVERWRITE = 5
Public Const BM_COL_SIZE = 6
Public Const BM_COL_BLOCKSIZE = 7
Public Const BM_71_COL_LOCATION = 8
Public Const BM_72_COL_ACTIONKIND = 8
Public Const BM_72_COL_LOCATION = 9

Const BM_COL_MAX = BM_72_COL_LOCATION

Public BMColPos(1 To BM_COL_MAX) As Integer
Public BMColLen(1 To BM_COL_MAX) As Integer

'''

Const DBMSRV_CMD_HISTORY_OPEN = "backup_history_open"
Const DBMSRV_CMD_HISTORY_LIST = "backup_history_list"
Const DBMSRV_CMD_HISTORY_LISTNEXT = "backup_history_listnext"
Const DBMSRV_CMD_HISTORY_CLOSE = "backup_history_close"
Const DBMSRV_CMD_HISTORY_DATE = "backup_history_date"

Const DBMSRV_OPT_HISTORY_CON = "CONTINUE"
Const DBMSRV_OPT_HISTORY_END = "END"

Public Function HistoryOpen(oDatabase As Database, _
                            lHistoryFor As DBMHistoryForConstants) As Boolean

    Dim sCommand As String
    Dim sData As String

    HistoryOpen = False

    sCommand = DBMSRV_CMD_HISTORY_OPEN

    If oDatabase.Version.Number >= dbmVersion723 Then
        Select Case lHistoryFor
            Case dbmHFView:
            Case dbmHFViewExt:
                If Not oDatabase.NoExternalBackup Then
                    sCommand = sCommand + " -e"
                End If
            Case dbmHFBackup:
            Case dbmHFRestoreInit:
            Case dbmHFRestore:
                If Not oDatabase.NoExternalBackup Then
                    sCommand = sCommand + " -e"
                End If
        End Select
    End If
    
    If oDatabase.ServerObject.Command(sCommand, sData) Then
        HistoryOpen = True
    End If
End Function

Public Function HistoryGet(oDatabase As Database, _
                           lHistoryFor As DBMHistoryForConstants, _
                           oHistory As History) As Boolean

    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long

    Dim bContinue As Boolean
    Dim sCommand As String
    Dim oHistoryItem As HistoryItem
    Dim oMedium As Medium
    Dim oExternalID As ExternalID

    Dim lLineNum As Long

    Dim bFirstHistoryLine As Boolean
    Dim bFirstMediaLine As Boolean

    Dim lItem As Long

    HistoryGet = False

    bContinue = True
    sCommand = DBMSRV_CMD_HISTORY_LIST

    Select Case lHistoryFor
        Case dbmHFView, dbmHFViewExt:
            sCommand = sCommand + " -m"
            If oDatabase.Version.Number >= dbmVersion723 And _
               Not oDatabase.NoExternalBackup _
            Then
                sCommand = sCommand + " -e"
            End If
            
        Case dbmHFBackup:
            sCommand = sCommand + " -a SAVE"
            
        Case dbmHFRestoreInit:
            sCommand = sCommand + " -a SAVE -l DAT"
        
        Case dbmHFRestore:
            sCommand = sCommand + " -a SAVE -m"
            If oDatabase.Version.Number >= dbmVersion723 And _
               Not oDatabase.NoExternalBackup _
            Then
                sCommand = sCommand + " -e"
            End If
    End Select

    lLineNum = 0

    bFirstHistoryLine = True
    bFirstMediaLine = True

    Set oHistoryItem = Nothing

    Do While bContinue
        If oDatabase.ServerObject.Command(sCommand, sData) Then
            sCommand = DBMSRV_CMD_HISTORY_LISTNEXT
            HistoryGet = True
            lPos1 = 1
            sLine = Trim(oDatabase.ServerObject.GetLine(sData, lPos1))
            bContinue = (UCase(sLine) = DBMSRV_OPT_HISTORY_CON)
            Do While lPos1 < Len(sData)
                sLine = RTrim(oDatabase.ServerObject.GetLine(sData, lPos1))
                If Left$(sLine, 2) = "M:" Then
                    sLine = oHistoryItem.SysKey + "|" + oHistoryItem.Label + "|" + Mid$(sLine, 3)
                    If bFirstMediaLine Then
                        MediaInitColumns sLine
                        bFirstMediaLine = False
                    End If
                    Set oMedium = New Medium
                    Set oMedium.Parent = oHistoryItem
                    oMedium.SetPropertiesFromHistoryLine sLine
                    oHistoryItem.Media.Add oMedium
                    Set oMedium = Nothing
                    
                ElseIf Left$(sLine, 2) = "E:" Then
                
                    If Left$(sLine, 6) <> "E:NONE" Then
                        sLine = Mid$(sLine, 3)
                        Set oExternalID = New ExternalID
                        Set oExternalID.Parent = oHistoryItem
                        oExternalID.SetPropertiesFromHistoryLine sLine
                        oHistoryItem.ExternalIDs.Add oExternalID
                        Set oExternalID = Nothing
                    End If
                    
                Else
                
                    If Not oHistoryItem Is Nothing Then
                        For Each oExternalID In oHistoryItem.ExternalIDs
                            If oExternalID.Index <= oHistoryItem.Media.Count Then
                                Set oExternalID.Medium = oHistoryItem.Media(oExternalID.Index)
                                Set oHistoryItem.Media(oExternalID.Index).ExternalID = oExternalID
                            End If
                        Next
                    End If
                    If bFirstHistoryLine Then
                        HistoryInitColumns sLine
                        bFirstHistoryLine = False
                    End If
                    Set oHistoryItem = Nothing
                    Set oHistoryItem = New HistoryItem
                    Set oHistoryItem.Parent = oHistory
                    lLineNum = lLineNum + 1
                    oHistoryItem.Line = sLine
                    oHistoryItem.LineNum = lLineNum
                    oHistory.HistoryItems.Add oHistoryItem
                End If
            Loop

            If Not oHistoryItem Is Nothing Then
                For Each oExternalID In oHistoryItem.ExternalIDs
                    If oExternalID.Index <= oHistoryItem.Media.Count Then
                        Set oExternalID.Medium = oHistoryItem.Media(oExternalID.Index)
                        Set oHistoryItem.Media(oExternalID.Index).ExternalID = oExternalID
                    End If
                Next
            End If

        Else
            HistoryGet = False
            Exit Do
        End If
    Loop
End Function

Public Function HistoryClose(oDatabase As Database) As Boolean
    Dim sData As String
    
    HistoryClose = False
    
    If oDatabase.ServerObject.Command(DBMSRV_CMD_HISTORY_CLOSE, sData) Then
        HistoryClose = True
    End If
End Function

Public Function HistoryDate(oDatabase As Database, _
                            vDate As Date) As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    
    HistoryDate = False
    
    If oDatabase.ServerObject.Command(DBMSRV_CMD_HISTORY_DATE, sData) Then
        lPos1 = 1
        sLine = Trim(oDatabase.ServerObject.GetLine(sData, lPos1))
        vDate = DateSerial(Val(Left(sLine, 4)), Val(Mid(sLine, 5, 2)), Val(Mid(sLine, 7, 2))) + _
                TimeSerial(Val(Mid(sLine, 9, 2)), Val(Mid(sLine, 11, 2)), Val(Mid(sLine, 13, 2)))
        HistoryDate = True
    End If

End Function

'''

Private Sub HistoryInitColumns(sLine As String)
    Dim iCol As Integer
    Dim iPos1 As Integer
    Dim iPos2 As Integer
    Dim iLen As Integer
    
    iCol = 1
    iPos1 = 1
    iLen = Len(sLine)
    Do
        iPos2 = InStr(iPos1, sLine, "|")
        If iPos2 > 0 Then
            BHColPos(iCol) = iPos1
            BHColLen(iCol) = iPos2 - iPos1
            iPos1 = iPos2 + 1
        Else
            iPos2 = Len(sLine) + 1
            BHColPos(iCol) = iPos1
            BHColLen(iCol) = iPos2 - iPos1
        End If
        iCol = iCol + 1
    Loop While iPos2 < iLen And iCol <= BH_COL_MAX
End Sub

Private Sub MediaInitColumns(sLine As String)
    Dim iCol As Integer
    Dim iPos1 As Integer
    Dim iPos2 As Integer
    Dim iLen As Integer
    
    iCol = 1
    iPos1 = 1
    iLen = Len(sLine)
    Do
        iPos2 = InStr(iPos1, sLine, "|")
        If iPos2 > 0 Then
            BMColPos(iCol) = iPos1
            BMColLen(iCol) = iPos2 - iPos1
            iPos1 = iPos2 + 1
        Else
            iPos2 = Len(sLine) + 1
            BMColPos(iCol) = iPos1
            BMColLen(iCol) = iPos2 - iPos1
        End If
        iCol = iCol + 1
    Loop While iPos2 < iLen And iCol <= BM_COL_MAX
End Sub


