Attribute VB_Name = "DBMSRVInfo"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const MARK_CONTINUE = "CONTINUE"
Const MARK_END = "END"

Public Function Info(oServerObject As DBMSOB.ServerObject, _
                     oInfo As Info) As Boolean
    Dim sInfo As String
    Dim sData As String
    Dim sLine As String
    Dim lPos As Long
    
    Info = False
    
    Select Case oInfo.InfoType
        Case dbmInfoState:              sInfo = "STATE"
        Case dbmInfoCaches:             sInfo = "CACHES"
        Case dbmInfoUsers:              sInfo = "USERS"
        Case dbmInfoData:               sInfo = "DATA"
        Case dbmInfoDevSpaces:          sInfo = "DEVSPACES"
        Case dbmInfoIO:                 sInfo = "IO"
        Case dbmInfoLog:                sInfo = "LOG"
        Case dbmInfoLocks:              sInfo = "LOCKS"
        Case dbmInfoVersions:           sInfo = "VERSIONS"
        Case dbmInfoBadDevspaces:       sInfo = "BADDEVSPACES"
        Case dbmInfoOMSCaches:          sInfo = "OMSCACHES"
        Case dbmInfoOMSMonitor:         sInfo = "OMSMON"
        Case dbmInfoOMSClasscontainers: sInfo = "CLSCON"
    End Select
    
    If oServerObject.Command(DBMSRV_CMD_INFO + " " + sInfo + " -d", sData) Then
        Info = True
        lPos = 1
        sLine = Trim$(oServerObject.GetLine(sData, lPos))
        oInfo.Continue = (UCase$(sLine) = "CONTINUE")
        oInfo.Description = Trim$(oServerObject.GetLine(sData, lPos))
        oInfo.Header = oServerObject.GetLine(sData, lPos)
        sLine = oServerObject.GetLine(sData, lPos)
        oInfo.Text = Mid$(sData, lPos)
    End If
End Function

Public Function InfoSelect(oServerObject As DBMSOB.ServerObject, _
                           sName As String, _
                           sWhere As String, _
                           sHeader As String, _
                           sResult As String, _
                           bContinue As Boolean) As Boolean
    Dim sCmd As String
    Dim sData As String
    
    InfoSelect = False
    
    sCmd = DBMSRV_CMD_INFO + " " + sName
    If Len(sWhere) > 0 Then
        sCmd = sCmd + " -w " + sWhere
    End If
    
    If oServerObject.Command(sCmd, sData) Then
        Call ReadResult(oServerObject, sData, sHeader, sResult, bContinue)
        InfoSelect = True
    End If
End Function

Public Function InfoFetch(oServerObject As DBMSOB.ServerObject, _
                          sHeader As String, _
                          sResult As String, _
                          bContinue As Boolean) As Boolean
    Dim sData As String
    
    InfoFetch = False
    
    If oServerObject.Command(DBMSRV_CMD_INFO_NEXT, sData) Then
        Call ReadResult(oServerObject, sData, sHeader, sResult, bContinue)
        InfoFetch = True
    End If
End Function

Private Sub ReadResult(oServerObject As DBMSOB.ServerObject, _
                       sData As String, _
                       sHeader As String, _
                       sResult As String, _
                       bContinue As Boolean)
    Dim sContinue As String
    Dim lDataPos As Long
    Dim lDataLen As Long
    Dim sLine As String
    
    bContinue = False
    sHeader = ""
    sResult = ""
    lDataPos = 1
    lDataLen = Len(sData)
    
    If (lDataLen > 0) Then
        sContinue = Trim$(oServerObject.GetLine(sData, lDataPos))
        bContinue = (sContinue = MARK_CONTINUE)
        
        If lDataPos < lDataLen Then
            sHeader = oServerObject.GetLine(sData, lDataPos)
            sLine = oServerObject.GetLine(sData, lDataPos)
            If lDataPos < lDataLen Then
                sResult = Mid$(sData, lDataPos)
            End If
        End If
    End If
End Sub
