Attribute VB_Name = "DBMSRVMedia"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private Const KEY_BACKSLASH = "\"
Private Const KEY_SLASH = "/"

Public Function GetMedia(oServerObject As DBMSOB.ServerObject, _
                         oBackup As Backup) As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim iPos3 As Long
    Dim sField As String
      
    Dim sGroup As String
    Dim sName As String
    Dim sLocation As String
    Dim lDeviceType As DBMDeviceTypeConstants
    Dim lSaveType As DBMBackupTypeConstants
    Dim lSizePages As Long
    Dim lBlockSize As Long
    Dim lOverwrite As DBMOverwriteConstants
    Dim lAutoLoader As DBMAutoLoaderConstants
    Dim sOSCommand As String
    Dim dCreated As Date
    Dim dModified As Date
    Dim bOldDefinition As Boolean
    
    Dim oMediaGroup As MediaGroup
    Dim oMedium As Medium
    
    GetMedia = False
    
    If oServerObject.Command(DBMSRV_CMD_MEDIA_GETALL, sData) Then
        GetMedia = True
        lPos1 = 1
        Do While lPos1 < Len(sData)
            sLine = Trim(oServerObject.GetLine(sData, lPos1))
            If Len(sLine) > 0 Then
                lPos2 = 1
                
                sName = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                sLocation = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                lDeviceType = oBackup.DeviceType(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
                lSaveType = oBackup.BackupTypeConstant(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
                lSizePages = Val(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
                lBlockSize = Val(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
                lOverwrite = oBackup.Overwrite(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
                lAutoLoader = oBackup.AutoLoaderConstant(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
                sOSCommand = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                
                sField = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                dCreated = DateSerial(Val(Left(sField, 4)), Val(Mid(sField, 5, 2)), Val(Mid(sField, 7, 2))) + _
                           TimeSerial(Val(Mid(sField, 9, 2)), Val(Mid(sField, 11, 2)), Val(Mid(sField, 13, 2)))
                
                sField = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
                dModified = DateSerial(Val(Left(sField, 4)), Val(Mid(sField, 5, 2)), Val(Mid(sField, 7, 2))) + _
                            TimeSerial(Val(Mid(sField, 9, 2)), Val(Mid(sField, 11, 2)), Val(Mid(sField, 13, 2)))
                
                bOldDefinition = (UCase(Trim(oServerObject.GetField(sLine, vbTab, lPos2))) = "OLD")
                
                iPos3 = InStr(1, sName, KEY_BACKSLASH)
                If iPos3 = 0 Then
                    iPos3 = InStr(1, sName, KEY_SLASH)
                End If
                
                If iPos3 > 0 Then
                    sGroup = Trim(Left(sName, iPos3 - 1))
                    sName = Trim(Mid(sName, iPos3 + 1))
                    
                    Set oMediaGroup = oBackup.MediaGroups.ItemByName(sGroup)
                    If oMediaGroup Is Nothing Then
                        Set oMediaGroup = oBackup.CreateMediaGroup(sGroup)
                    End If
                    
                    Set oMedium = oMediaGroup.Media.ItemByName(sName)
                    If oMedium Is Nothing Then
                        Set oMedium = oMediaGroup.CreateMedium(sName, sLocation, lDeviceType, lSaveType)
                    End If
                Else
                    Set oMedium = oBackup.Media.ItemByName(sName)
                    If oMedium Is Nothing Then
                        Set oMedium = oBackup.CreateMedium(sName, sLocation, lDeviceType, lSaveType)
                    End If
                End If
                
                If Not oMedium Is Nothing Then
                    With oMedium
                        .SizePages = lSizePages
                        .Blocksize = lBlockSize
                        .Overwrite = lOverwrite
                        .AutoLoader = lAutoLoader
                        .OSCommand = sOSCommand
                        .Created = dCreated
                        .Modified = dModified
                        .OldDefinition = bOldDefinition
                    End With
                End If

            End If
        Loop
    End If
End Function

Public Function GetMedium(oServerObject As DBMSOB.ServerObject, _
                          oMedium As Medium) As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim sField As String
      
    Dim sGroup As String
    Dim sName As String
    Dim sLocation As String
    Dim lDeviceType As DBMDeviceTypeConstants
    Dim lSaveType As DBMBackupTypeConstants
    Dim lSizePages As Long
    Dim lBlockSize As Long
    Dim lOverwrite As DBMOverwriteConstants
    Dim lAutoLoader As DBMAutoLoaderConstants
    Dim sOSCommand As String
    Dim dCreated As Date
    Dim dModified As Date
    Dim bOldDefinition As Boolean
    
    GetMedium = False
    
    If oMedium.Parent.ObjectType = dbmOTMediaGroup Then
        sName = oMedium.Parent.Name & KEY_BACKSLASH
    End If
            
    sName = sName & oMedium.Name
    sName = """" & sName & """"
    
    If oServerObject.Command(DBMSRV_CMD_MEDIA_GET & " " & sName, sData) Then
        GetMedium = True
        lPos1 = 1
        sLine = Trim(oServerObject.GetLine(sData, lPos1))
        If Len(sLine) > 0 Then
            lPos2 = 1
            
            sName = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
            sLocation = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
            lDeviceType = oMedium.Database.Backup.DeviceType(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
            lSaveType = oMedium.Database.Backup.BackupTypeConstant(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
            lSizePages = Val(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
            lBlockSize = Val(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
            lOverwrite = oMedium.Database.Backup.Overwrite(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
            lAutoLoader = oMedium.Database.Backup.AutoLoaderConstant(Trim(oServerObject.GetField(sLine, vbTab, lPos2)))
            sOSCommand = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
            
            sField = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
            dCreated = DateSerial(Val(Left(sField, 4)), Val(Mid(sField, 5, 2)), Val(Mid(sField, 7, 2))) + _
                       TimeSerial(Val(Mid(sField, 9, 2)), Val(Mid(sField, 11, 2)), Val(Mid(sField, 13, 2)))
            
            sField = Trim(oServerObject.GetField(sLine, vbTab, lPos2))
            dModified = DateSerial(Val(Left(sField, 4)), Val(Mid(sField, 5, 2)), Val(Mid(sField, 7, 2))) + _
                        TimeSerial(Val(Mid(sField, 9, 2)), Val(Mid(sField, 11, 2)), Val(Mid(sField, 13, 2)))
            
            bOldDefinition = (UCase(Trim(oServerObject.GetField(sLine, vbTab, lPos2))) = "OLD")
            
            With oMedium
                .Location = sLocation
                .DeviceType = lDeviceType
                .SaveType = lSaveType
                .SizePages = lSizePages
                .Blocksize = lBlockSize
                .Overwrite = lOverwrite
                .AutoLoader = lAutoLoader
                .OSCommand = sOSCommand
                .Created = dCreated
                .Modified = dModified
                .OldDefinition = bOldDefinition
            End With
        End If
    End If
End Function


Public Function PutMedia(oServerObject As DBMSOB.ServerObject, _
                         oBackup As Backup) As Boolean
    Dim bPutMedia As Boolean
    
    Dim oMediaGroup As MediaGroup
    Dim oMedium As Medium

    bPutMedia = True
    
    For Each oMediaGroup In oBackup.MediaGroups
        For Each oMedium In oMediaGroup.Media
            If Not oMedium.OldDefinition Then
                If Not PutMedium(oServerObject, oMedium) Then
                    bPutMedia = False
                    Exit For
                End If
            End If
        Next
    Next
    
    If bPutMedia = True Then
        For Each oMedium In oBackup.Media
            If Not oMedium.OldDefinition Then
                If Not PutMedium(oServerObject, oMedium) Then
                    bPutMedia = False
                    Exit For
                End If
            End If
        Next
    End If
    
    PutMedia = bPutMedia
End Function

Public Function PutMedium(oServerObject As DBMSOB.ServerObject, _
                          oMedium As Medium) As Boolean
    Dim sDef As String
    
    Dim sData As String
    
    PutMedium = False
    
    sDef = GetMediumDefinition(oMedium)
    If oServerObject.Command(DBMSRV_CMD_MEDIA_PUT & " " & sDef, sData) Then
        PutMedium = True
    End If
End Function

Public Function DeleteMedium(oServerObject As DBMSOB.ServerObject, _
                             oMedium As Medium) As Boolean
    Dim sName As String
    
    Dim sData As String
    
    DeleteMedium = False
    
    If oMedium.Parent.ObjectType = dbmOTMediaGroup Then
        sName = oMedium.Parent.Name & KEY_BACKSLASH
    End If
            
    sName = sName & oMedium.Name
    sName = """" & sName & """"
    
    If oServerObject.Command(DBMSRV_CMD_MEDIA_DELETE & " " & sName, sData) Then
        DeleteMedium = True
    End If
End Function

Public Function MediaDate(oServerObject As DBMSOB.ServerObject, _
                          vDate As Date) As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    
    MediaDate = False
    
    If oServerObject.Command(DBMSRV_CMD_MEDIA_DATE, sData) Then
        lPos1 = 1
        sLine = Trim(oServerObject.GetLine(sData, lPos1))
        vDate = DateSerial(Val(Left(sLine, 4)), Val(Mid(sLine, 5, 2)), Val(Mid(sLine, 7, 2))) + _
                TimeSerial(Val(Mid(sLine, 9, 2)), Val(Mid(sLine, 11, 2)), Val(Mid(sLine, 13, 2)))
        MediaDate = True
    End If
End Function

'''

Private Function GetMediumDefinition(oMedium As Medium) As String
    Dim sName As String
    Dim sDef As String
    
    sName = ""
    sDef = ""
    
    If oMedium.Parent.ObjectType = dbmOTMediaGroup Then
        sName = oMedium.Parent.Name & KEY_BACKSLASH
    End If
        
    sName = sName & oMedium.Name
    
    sDef = """" & sName & """"

    sDef = sDef & " " & """" & oMedium.Location & """"
    sDef = sDef & " " & oMedium.DeviceTypeName
    sDef = sDef & " " & oMedium.SaveTypeString
    sDef = sDef & " " & CStr(oMedium.SizePages)
    sDef = sDef & " " & CStr(oMedium.Blocksize)
    sDef = sDef & " " & oMedium.OverwriteString
    sDef = sDef & " " & oMedium.AutoLoaderString
    sDef = sDef & " " & """" & oMedium.OSCommand & """"
    
    GetMediumDefinition = sDef
End Function

