VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "History"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_REFRESH = "RefreshHistory"

Private oParent As Object

Public Enum DBMHistoryForConstants
    dbmHFView
    dbmHFViewExt
    dbmHFBackup
    dbmHFRestoreInit
    dbmHFRestore
End Enum

Private lHistoryFor As DBMHistoryForConstants

Private dLastDate As Date

Private dAt As Date

Private oHistoryItems As HistoryItems

Private bAlways As Boolean

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTHistory
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

'''

Public Property Get HistoryFor() As DBMHistoryForConstants
    HistoryFor = lHistoryFor
End Property

Public Property Get At() As Date
    At = dAt
End Property

Public Property Get HistoryItems() As HistoryItems
    If oHistoryItems Is Nothing Then
        Set oHistoryItems = New HistoryItems
    End If
    Set HistoryItems = oHistoryItems
End Property

Public Property Get LastSaveData() As HistoryItem
    Dim lCount As Long
    Dim lItem As Long
    Dim oItem As HistoryItem
    
    Set LastSaveData = Nothing
    
    If Not oHistoryItems Is Nothing Then
        lCount = oHistoryItems.Count
        
        For lItem = lCount To 1 Step -1
            
            Set oItem = oHistoryItems.Item(lItem)
            
            If oItem.ActionType = dbmBAHistLost Then
                Exit For
            Else
                If oItem.ActionType = dbmBASave And _
                   oItem.BackupType = dbmBTData And _
                   oItem.ReturnCode = 0 _
                Then
                    Set LastSaveData = oItem
                    Exit For
                End If
            End If
        Next
    End If
End Property

Public Property Get ItemBySysKeyAndLabel(SysKey As String, Label As String) As HistoryItem
    Dim lCount As Long
    Dim lItem As Long
    Dim oItem As HistoryItem
    
    Set ItemBySysKeyAndLabel = Nothing
    
    If Not oHistoryItems Is Nothing Then
        lCount = oHistoryItems.Count
        
        For lItem = lCount To 1 Step -1
            
            Set oItem = oHistoryItems.Item(lItem)
            
            If oItem.SysKey = SysKey Then
                If oItem.Label = Label Then
                    Set ItemBySysKeyAndLabel = oItem
                    Exit For
                End If
            End If
        Next
    End If
End Property
'
' Methods
'
Public Function Refresh(Optional ByVal HistoryFor As DBMHistoryForConstants = dbmHFView, _
                        Optional ByVal Always As Boolean = False, _
                        Optional ByVal Asynchron As Boolean = False, _
                        Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        lHistoryFor = HistoryFor
        bAlways = Always
        If Not Asynchron Then
            bOK = Execute(FUNC_REFRESH, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_REFRESH, SenderID
            bOK = True
        End If
    End If
    
    Refresh = bOK
End Function

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim dCurrentDate As Date
    Dim bGet As Boolean
    Dim bOpen As Boolean
    
    bOK = False
    
    dCurrentDate = Empty
    bGet = False
    
    Database.Busy = True
            
    With Database
        If .Check() Then
            If bAlways = True Then
                bGet = True
            Else
                If dLastDate = Empty Then
                    bGet = True
                Else
                    If DBMSRVHistory.HistoryDate(Database, dCurrentDate) Then
                        If dCurrentDate <> Empty Then
                            If dLastDate < dCurrentDate Then
                                bGet = True
                            Else
                                bOK = True
                            End If
                        Else
                            bGet = True
                        End If
                    Else
                        bGet = True
                    End If
                End If
            End If
            
            If bGet Then
                HistoryItems.Clear
                
                bOpen = False
                
                .ServerObject.ErrorRaise = False
                
                If DBMSRVHistory.HistoryOpen(Database, lHistoryFor) Then
                    .ServerObject.ErrorRaise = True
                
                    bOpen = True
                    
                    If DBMSRVHistory.HistoryGet(Database, lHistoryFor, Me) Then
                        dLastDate = dCurrentDate
                        dAt = Now()
                        bOK = True
                    Else
                        .LastError.SetByServerObject .ServerObject
                    End If
                Else
                    .ServerObject.ErrorRaise = True
                    If .ServerObject.ReturnCode = DBMSOB_ERR_NOHISTORY Then
                        dLastDate = dCurrentDate
                        dAt = Now()
                        bOK = True
                    Else
                        .ServerObject.RaiseError
                        .LastError.SetByServerObject .ServerObject
                    End If
                End If
                
                If bOpen Then
                    DBMSRVHistory.HistoryClose Database
                End If
            Else
                bOK = True
            End If
        
        End If
        
        If Asynchron Then
            .EventHistoryAction SenderID, Not bOK, dbmHistoryActionRefresh
        End If
    End With
    
    Database.Busy = False
    
    Execute = bOK
End Function

Public Sub CleanUp()
    If Not oHistoryItems Is Nothing Then oHistoryItems.CleanUp
    
    Set oHistoryItems = Nothing
    
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
    
    lHistoryFor = dbmHFView
    
    dAt = Empty
    dLastDate = Empty
    
    Set oHistoryItems = Nothing
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub
