VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Indexes"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_OPEN = "OpenIndexes"
Const FUNC_FETCH = "FetchIndexes"
Const FUNC_CLOSE = "CloseIndexes"
Const FUNC_RECREATE = "RecreateIndex"
Const FUNC_DROP = "DropIndex"
Const FUNC_ENABLE = "EnableIndex"

Private oParent As Object

Private sSchemaName As String
Private sTableName As String
Private sIndexname As String
Private bUnused As Boolean
Private bDisabled As Boolean
Private bBad As Boolean
Private bEnable As Boolean

Private bOpen As Boolean

Private sHeader As String
Private sResult As String
Private bContinue As Boolean

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTIndexes
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

'''

Public Property Get SchemaName() As String
    SchemaName = sSchemaName
End Property
Public Property Get TableName() As String
    TableName = sTableName
End Property
Public Property Get IndexName() As String
    IndexName = sIndexname
End Property

'''

Public Property Get Header() As String
    Header = sHeader
End Property
Public Property Get Result() As String
    Result = sResult
End Property
Public Property Get Continue() As Boolean
    Continue = bContinue
End Property

'''

Public Function OpenIndexes(SchemaName As String, _
                            TableName As String, _
                            IndexName As String, _
                            Unused As Boolean, _
                            Disabled As Boolean, _
                            Bad As Boolean, _
                            Optional ByVal Asynchron As Boolean = False, _
                            Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        
        sSchemaName = SchemaName
        sTableName = TableName
        sIndexname = IndexName
        bUnused = Unused
        bDisabled = Disabled
        bBad = Bad
        
        If Not Asynchron Then
            bOK = Execute(FUNC_OPEN, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_OPEN, SenderID
            bOK = True
        End If
    End If
    
    OpenIndexes = bOK
End Function

Public Function FetchIndexes(Optional ByVal Asynchron As Boolean = False, _
                             Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_FETCH, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_FETCH, SenderID
            bOK = True
        End If
    End If
    
    FetchIndexes = bOK
End Function

Public Function CloseIndexes(Optional ByVal Asynchron As Boolean = False, _
                             Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_CLOSE, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_CLOSE, SenderID
            bOK = True
        End If
    End If
    
    CloseIndexes = bOK
End Function

Public Function RecreateIndex(Optional Schema As String = "", _
                              Optional Table As String = "", _
                              Optional Index As String = "", _
                              Optional ByVal Asynchron As Boolean = False, _
                              Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        
        sSchemaName = Schema
        sTableName = Table
        sIndexname = Index
        
        If Not Asynchron Then
            bOK = Execute(FUNC_RECREATE, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_RECREATE, SenderID
            bOK = True
        End If
    End If

    RecreateIndex = bOK
End Function

Public Function EnableIndex(Schema As String, _
                            Table As String, _
                            Index As String, _
                            Enable As Boolean, _
                            Optional ByVal Asynchron As Boolean = False, _
                            Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        
        sSchemaName = Schema
        sTableName = Table
        sIndexname = Index
        bEnable = Enable
        
        If Not Asynchron Then
            bOK = Execute(FUNC_ENABLE, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_ENABLE, SenderID
            bOK = True
        End If
    End If
    
    EnableIndex = bOK
End Function

Public Function DropIndex(Schema As String, _
                          Table As String, _
                          Index As String, _
                          Optional ByVal Asynchron As Boolean = False, _
                          Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        
        sSchemaName = Schema
        sTableName = Table
        sIndexname = Index
        
        If Not Asynchron Then
            bOK = Execute(FUNC_DROP, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_DROP, SenderID
            bOK = True
        End If
    End If
    
    DropIndex = bOK
End Function

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

Private Sub Class_Initialize()
    InitObject ObjectType
    bOpen = False
End Sub

Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    
    Database.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_OPEN:     bOK = ExecuteOpenIndexes(Asynchron, SenderID)
        Case FUNC_FETCH:    bOK = ExecuteFetchIndexes(Asynchron, SenderID)
        Case FUNC_CLOSE:    bOK = ExecuteCloseIndexes(Asynchron, SenderID)
        Case FUNC_RECREATE: bOK = ExecuteRecreateIndex(Asynchron, SenderID)
        Case FUNC_DROP:     bOK = ExecuteDropIndex(Asynchron, SenderID)
        Case FUNC_ENABLE:   bOK = ExecuteEnableIndex(Asynchron, SenderID)
    End Select
            
    Database.Busy = False
    
    Execute = bOK
End Function

Private Function ExecuteOpenIndexes(ByVal bAsynchron As Boolean, _
                                    ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim sInfo As String
    Dim sWhere As String
    Dim sLike As String
    
    bOK = False
    
    sWhere = ""
    
    With Database
        If .Check() Then
            sInfo = "INDEXES"
    
            If Len(sSchemaName) > 0 Then
                sLike = Replace(sSchemaName, "*", "%")
                If sLike <> "%" Then
                    sWhere = sWhere + " ""Owner"" LIKE '" + sLike + "' AND"
                End If
            End If
            If Len(sTableName) > 0 Then
                sLike = Replace(sTableName, "*", "%")
                If sLike <> "%" Then
                    sWhere = sWhere + " ""Table Name"" LIKE '" + sLike + "' AND"
                End If
            End If
            If Len(sIndexname) > 0 Then
                sLike = Replace(sIndexname, "*", "%")
                If sLike <> "%" Then
                    sWhere = sWhere + " ""Index Name"" LIKE '" + sLike + "' AND"
                End If
            End If
            
            If Right(sWhere, 3) = "AND" Then
                sWhere = Left$(sWhere, Len(sWhere) - 4)
            End If
            
            If bUnused Then
                sInfo = "UNUSEDIDX"
                bDisabled = False
                bBad = False
            End If
            
            If bDisabled Then
                sInfo = "DISABLEDIDX"
                bUnused = False
                bBad = False
            End If
            
            If bBad Then
                sInfo = "BADIDX"
                bUnused = False
                bDisabled = False
            End If
            
            If DBMSRVSQL.Connect(.ServerObject) Then
                If DBMSRVInfo.InfoSelect(.ServerObject, sInfo, sWhere, sHeader, sResult, bContinue) Then
                    bOpen = True
                    bOK = True
                Else
                    DBMSRVSQL.Release .ServerObject
                End If
            End If
            
            If Not bOK Then
                .LastError.SetByServerObject .ServerObject
            End If
            
        End If
        
        If bAsynchron Then
            .EventIndexAction lSenderID, Not bOK, dbmIndexActionOpen
        End If
            
    End With
    
    ExecuteOpenIndexes = bOK
End Function

Private Function ExecuteFetchIndexes(ByVal bAsynchron As Boolean, _
                                     ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If bOpen Then
                If DBMSRVInfo.InfoFetch(.ServerObject, sHeader, sResult, bContinue) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
            End If
        End If
        
        If bAsynchron Then
            .EventIndexAction lSenderID, Not bOK, dbmIndexActionFetch
        End If
    End With
    
    ExecuteFetchIndexes = bOK
End Function

Private Function ExecuteCloseIndexes(ByVal bAsynchron As Boolean, _
                                     ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean

    bOK = False
    
    With Database
        If .Check() Then
            If bOpen Then
                If DBMSRVSQL.Release(.ServerObject) Then
                    bOpen = False
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
            End If
        End If
        
        If bAsynchron Then
            .EventIndexAction lSenderID, Not bOK, dbmIndexActionClose
        End If
    End With
    
    ExecuteCloseIndexes = bOK
End Function

Private Function ExecuteRecreateIndex(ByVal bAsynchron As Boolean, _
                                      ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If DBMSRVSQL.RecreateIndex(.ServerObject, sSchemaName, sTableName, sIndexname) Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bAsynchron Then
            .EventIndexAction lSenderID, Not bOK, dbmIndexActionRecreate
        End If
    End With

    ExecuteRecreateIndex = bOK
End Function

Private Function ExecuteEnableIndex(ByVal bAsynchron As Boolean, _
                                    ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim sCmd As String
    
    bOK = False
    
    With Database
        If .Check() Then
            If DBMSRVSQL.Connect(.ServerObject) Then
                sCmd = "ALTER INDEX """ + sIndexname + """ ON """ + sSchemaName + """.""" + sTableName + """ "
                If bEnable Then
                    sCmd = sCmd + "ENABLE"
                Else
                    sCmd = sCmd + "DISABLE"
                End If
                If DBMSRVSQL.Execute(.ServerObject, sCmd) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
                
                DBMSRVSQL.Release .ServerObject
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bAsynchron Then
            .EventIndexAction lSenderID, Not bOK, dbmIndexActionEnable
        End If
    End With

    ExecuteEnableIndex = bOK
End Function

Private Function ExecuteDropIndex(ByVal bAsynchron As Boolean, _
                                  ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim sCmd As String
    
    bOK = False
    
    With Database
        If .Check() Then
            If DBMSRVSQL.Connect(.ServerObject) Then
                sCmd = "DROP INDEX """ + sIndexname + """ ON """ + sSchemaName + """.""" + sTableName + """"
                If DBMSRVSQL.Execute(.ServerObject, sCmd) Then
                    bOK = True
                Else
                    sCmd = "DROP INDEX """ + sSchemaName + """.""" + sTableName + """.""" + sIndexname + """"
                    If DBMSRVSQL.Execute(.ServerObject, sCmd) Then
                        bOK = True
                    Else
                        .LastError.SetByServerObject .ServerObject
                    End If
                End If
                DBMSRVSQL.Release .ServerObject
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bAsynchron Then
            .EventIndexAction lSenderID, Not bOK, dbmIndexActionDrop
        End If
    End With
End Function

