VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Info"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_REFRESH = "RefreshInfo"

Private oParent As Object

Private lIndex As Long

Private dAt As Date

Private lType As DBMInfoTypeConstants

Private bContinue As Boolean
Private sDescription As String
Private sHeader As String
Private sText As String

Private lPosReadLines As Long
Private lPosReadColumns As Long
Private lNumReadColumns As Integer

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTInfo
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get Index() As Long
    Index = lIndex
End Property
Friend Property Let Index(ByVal NewVal As Long)
    lIndex = NewVal
End Property


'''

Public Property Get At() As Date
    At = dAt
End Property

Public Property Get InfoType() As DBMInfoTypeConstants
    InfoType = lType
End Property
Friend Property Let InfoType(ByVal NewVal As DBMInfoTypeConstants)
    lType = NewVal
End Property

Public Property Get Continue() As Boolean
    Continue = bContinue
End Property
Friend Property Let Continue(ByVal NewVal As Boolean)
    bContinue = NewVal
End Property

Public Property Get Description() As String
    Description = sDescription
End Property
Friend Property Let Description(NewVal As String)
    sDescription = NewVal
End Property

Public Property Get Header() As String
    Header = sHeader
End Property
Friend Property Let Header(NewVal As String)
    sHeader = NewVal
End Property

Public Property Get Text() As String
    Text = sText
End Property
Friend Property Let Text(NewVal As String)
    sText = NewVal
End Property

'''

Public Function Refresh(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Database.State.Value = dbmDBStateWarm Then
            If Not Asynchron Then
                bOK = Execute(FUNC_REFRESH, False)
            Else
                QUEUE.QueueItems.Add Me, FUNC_REFRESH
                bOK = True
            End If
        End If
    End If
    
    Refresh = bOK
End Function

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    Database.Busy = True
            
    With Database
        If .Check() Then
            If DBMSRVSQL.Connect(.ServerObject) Then
                If DBMSRVInfo.Info(.ServerObject, Me) Then
                    dAt = Now()
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
                DBMSRVSQL.Release .ServerObject
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If Asynchron Then
            .EventInfoRefreshed Not bOK, lType
        End If
    End With
    
    Database.Busy = False
    
    Execute = bOK
End Function

'''

Public Sub BeginReadColumns()
    lPosReadColumns = 1
    lNumReadColumns = 1
End Sub

Public Function ReadColumn(ColNum As Integer, ColType As DBMInfoColumnTypeConstants, ColPos As Integer, ColLen As Integer) As String
    Dim lPos As Long
    Dim sColumn As String
    
    sColumn = ""
    
    If lPosReadColumns <= Len(sHeader) Then
        lPos = InStr(lPosReadColumns, sHeader, "|")
        If lPos > 0 Then
            ColNum = lNumReadColumns
            ColType = GetColumnType(lNumReadColumns)
            ColPos = lPosReadColumns
            ColLen = lPos - lPosReadColumns
            sColumn = Mid$(sHeader, ColPos, ColLen)
            lPosReadColumns = lPos + 1
            lNumReadColumns = lNumReadColumns + 1
        Else
            ColNum = lNumReadColumns
            ColType = GetColumnType(lNumReadColumns)
            ColPos = lPosReadColumns
            sColumn = Mid$(sHeader, ColPos)
            ColLen = Len(sColumn)
            lPosReadColumns = Len(sHeader) + 1
            lNumReadColumns = lNumReadColumns + 1
        End If
    End If
        
    ReadColumn = Trim(sColumn)
End Function

Public Function EndOfReadColumns() As Boolean
    EndOfReadColumns = (lPosReadColumns > Len(sHeader))
End Function

'''

Public Sub BeginReadLines()
    lPosReadLines = 1
End Sub

Public Function ReadLine() As String
    Dim lPos As Long
    Dim sLine As String
    
    sLine = ""
    
    If lPosReadLines <= Len(sText) Then
        lPos = InStr(lPosReadLines, sText, vbLf)
        If lPos > 0 Then
            sLine = Mid$(sText, lPosReadLines, lPos - lPosReadLines)
            lPosReadLines = lPos + 1
        Else
            sLine = Mid$(sText, lPosReadLines)
            lPosReadLines = Len(sText) + 1
        End If
    End If
    
    lPos = InStr(1, sLine, Chr(0))
    If lPos > 0 Then sLine = Left$(sLine, lPos - 1)
        
    ReadLine = sLine
End Function

Public Function EndOfReadLines() As Boolean
    EndOfReadLines = (lPosReadLines > Len(sText))
End Function

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
    
    dAt = Empty
    
    bContinue = False
    sHeader = ""
    sText = ""
    
    lPosReadLines = 1
    lPosReadColumns = 1
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

Private Function GetColumnType(ByVal iCol As Integer) As DBMInfoColumnTypeConstants
    Dim sColType As String
    Dim lColType As DBMInfoColumnTypeConstants
    
    sColType = "-"
    
    If iCol > 0 And iCol <= Len(sDescription) Then
        sColType = Mid$(sDescription, iCol, 1)
    End If
    
    Select Case sColType
        Case "N":   lColType = dbmInfoColTypeNumeric
        Case "S":   lColType = dbmInfoColTypeString
        Case "D":   lColType = dbmInfoColTypeDate
        Case "B":   lColType = dbmInfoColTypeBoolean
        Case Else:  lColType = dbmInfoColTypeUnknown
    End Select
    
    GetColumnType = lColType
End Function
