VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "KernelTraceOptions"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const OPT_DEF = "Default"
Const OPT_IND = "Index"
Const OPT_LOC = "Lock"
Const OPT_LON = "Long"
Const OPT_PAG = "Pages"
Const OPT_STA = "Standard"
Const OPT_TAB = "Table"
Const OPT_OBJ = "Objects"
Const OPT_ADD = "Objects add"
Const OPT_GET = "Objects get"
Const OPT_ALT = "Objects alter"
Const OPT_FRE = "Objects free"
Const OPT_SEL = "Select"
Const OPT_INS = "Insert"
Const OPT_UPD = "Update"
Const OPT_DEL = "Delete"
Const OPT_ORD = "Order"
Const OPT_OPT = "Optimizer"
Const OPT_TIM = "Time"
Const OPT_CHE = "Check"
Const OPT_COM = "Commands"

Const DES_DEF = "Includes Index, Lock, Long, Pages, Standard, Table"
Const DES_IND = "Interfaces according accesses to indexes"
Const DES_LOC = "Lock collisions and their elimination"
Const DES_LON = "Accesses to LONG columns"
Const DES_PAG = "I/O"
Const DES_STA = "Beginning of all parts of the packet interface"
Const DES_TAB = "Accesses to primary data"
Const DES_OBJ = "Accesses to objects (includes add, get, alter, free)"
Const DES_ADD = "Accesses to objects"
Const DES_GET = "Accesses to objects"
Const DES_ALT = "Accesses to objects"
Const DES_FRE = "Accesses to objects"
Const DES_SEL = "Complete message buffer of SELECT, GET, FETCH, SELECT ROW and UNION"
Const DES_INS = "Complete message buffer of INSERT"
Const DES_UPD = "Complete message buffer of UPDATE"
Const DES_DEL = "Complete message buffer of DELETE"
Const DES_ORD = "Complete order packet"
Const DES_OPT = "Optimizer output"
Const DES_TIM = "Elapsed time in kernel underneath of the message buffer"
Const DES_CHE = "Additional checks on correct state of the regions"
Const DES_COM = "All order packets (including internal packets)"

Const FUNC_REFRESH = "Refresh"
Const FUNC_OPTIONSON = "OptionsOn"
Const FUNC_OPTIONSOFF = "OptionsOff"

Private oParent As Object

Private oItems As New Collection

Private dAt As Date

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTKernelTraceOptions
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get Count() As Integer
    Count = oItems.Count
End Property

Public Property Get Item(Index As Variant) As KernelTraceOption
Attribute Item.VB_UserMemId = 0
    Set Item = oItems(Index)
End Property

Public Property Get At() As Date
    At = dAt
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property

'''

Friend Sub Add(KernelTraceOption As KernelTraceOption, _
               Optional Key As Variant)
               
    Set KernelTraceOption.Parent = Me
               
    If IsMissing(Key) Then
        oItems.Add KernelTraceOption
    Else
        oItems.Add KernelTraceOption, Key
    End If
    
    KernelTraceOption.Index = oItems.Count
End Sub

Public Sub Clear()
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = 1 To lCount
        oItems(1).CleanUp
        oItems.Remove 1
    Next
End Sub

Public Sub CleanUp()
    Clear
    Set oParent = Nothing
End Sub

Public Function Exists(Index As Variant) As Boolean
    Dim oKernelTraceOption As KernelTraceOption
    
    On Error Resume Next

    Set oKernelTraceOption = oItems(Index)
    
    If Err.Number Then
        Exists = False
    Else
        Exists = True
    End If
End Function

Public Function Refresh(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_REFRESH, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_REFRESH
            bOK = True
        End If
    End If
    
    Refresh = bOK
End Function

Public Function OptionsOn(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_OPTIONSON, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_OPTIONSON
            bOK = True
        End If
    End If
    
    OptionsOn = bOK
End Function

Public Function OptionsOff(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_OPTIONSOFF, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_OPTIONSOFF
            bOK = True
        End If
    End If
    
    OptionsOff = bOK
End Function

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim oDB As DBMLib.Database
    
    Set oDB = Database
    
    oDB.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_REFRESH:      bOK = ExecuteRefresh(Asynchron)
        Case FUNC_OPTIONSON:    bOK = ExecuteOptionsOn(Asynchron)
        Case FUNC_OPTIONSOFF:   bOK = ExecuteOptionsOff(Asynchron)
    End Select
            
    oDB.Busy = False
    
    Execute = bOK
End Function

Private Function ExecuteRefresh(ByVal bAsynchron As Boolean) As Boolean
    Dim bOK As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    
    Dim sName As String
    Dim sState As String
    Dim sLevel As String
    Dim sDescription As String
    
    Dim oKernelTraceOption As KernelTraceOption
    
    bOK = False
    
    Clear
            
    With Database
        If .Check() Then
            If .ServerObject.Command(DBMSRV_CMD_TRACE_SHOW + " verbose", sData) Then
                bOK = True
                lPos1 = 1
                sLine = Trim(.ServerObject.GetLine(sData, lPos1)) ' Headerline
                Do While lPos1 < Len(sData)
                    sLine = Trim(.ServerObject.GetLine(sData, lPos1))
                    If Len(sLine) > 0 Then
                        lPos2 = 1
                        Set oKernelTraceOption = New KernelTraceOption
                        sName = Trim(.ServerObject.GetField(sLine, vbTab, lPos2))
                        sState = Trim(.ServerObject.GetField(sLine, vbTab, lPos2))
                        sLevel = Trim(.ServerObject.GetField(sLine, vbTab, lPos2))
                        sDescription = Trim(.ServerObject.GetField(sLine, vbTab, lPos2))
                        With oKernelTraceOption
                            .Name = sName
                            .State = (UCase$(sState) = "ON")
                            .Level = sLevel
                            .Description = sDescription
                            .Checked = False
                        End With
                        Add oKernelTraceOption, oKernelTraceOption.Name
                    End If
                Loop
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bAsynchron Then
            .EventKernelTraceAction Not bOK, dbmKernelTraceActionRefresh
        End If
    End With
    
    ExecuteRefresh = bOK
End Function

Private Function ExecuteOptionsOn(ByVal bAsynchron As Boolean) As Boolean
    Dim bOK As Boolean
    Dim sSpec As String
    Dim sData As String
    
    With Database
        If .Check() Then
            sSpec = GetOptionSpec(True)
            If Len(sSpec) > 0 Then
                If .ServerObject.Command(DBMSRV_CMD_TRACE_ON + sSpec, sData) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
            End If
        End If
        
        If bAsynchron Then
            .EventKernelTraceAction Not bOK, dbmKernelTraceActionOptionsOn
        End If
    End With
    
    ExecuteOptionsOn = bOK
End Function

Private Function ExecuteOptionsOff(ByVal bAsynchron As Boolean) As Boolean
    Dim bOK As Boolean
    Dim sSpec As String
    Dim sData As String
    
    With Database
        If .Check() Then
            sSpec = GetOptionSpec(False)
            If Len(sSpec) > 0 Then
                If .ServerObject.Command(DBMSRV_CMD_TRACE_OFF + sSpec, sData) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
            End If
        End If
        
        If bAsynchron Then
            .EventKernelTraceAction Not bOK, dbmKernelTraceActionOptionsOff
        End If
    End With
    
    ExecuteOptionsOff = bOK
End Function

'''

Private Sub Class_Initialize()
    InitObject ObjectType
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Private Function GetOptionSpec(ByVal bWithLevel As Boolean) As String
    Dim oKernelTraceOption As KernelTraceOption
    Dim sSpec As String
    Dim lCount As Long
    Dim l As Long
    
    sSpec = ""
    
    lCount = oItems.Count
    
    For l = 1 To lCount
        Set oKernelTraceOption = oItems(l)
        If oKernelTraceOption.Checked Then
            sSpec = sSpec + " " + oKernelTraceOption.Name
            If bWithLevel Then
                If oKernelTraceOption.Level >= "1" And _
                   oKernelTraceOption.Level <= "9" _
                Then
                    sSpec = sSpec + " " + oKernelTraceOption.Level
                End If
            End If
        End If
    Next
    
    GetOptionSpec = sSpec
End Function
