VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Parameters"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "RVB_UniqueId" ,"33F01EFE01F3"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_OPENPARAMS = "OpenParamaters"
Const FUNC_CHECKPARAMS = "CheckParameters"
Const FUNC_SAVEPARAMS = "SaveParameters"
Const FUNC_CLOSEPARAMS = "CloseParameters"

Public Enum DBMParameterGroupsConstants
    dbmPGGeneral = 0
    dbmPGExtended = 1
    dbmPGSupport = 2
    dbmPGInfo = 3
End Enum

Const GRP_GEN = "GENERAL"
Const GRP_EXT = "EXTENDED"
Const GRP_SUP = "SUPPORT"
Const GRP_INF = "INFO"

Const PARAM_PARAMCHANGED = "__PARAM_CHANGED___"

Private oParent As Object

Private oItems As New Collection

Private bOpen As Boolean

'''

Private dAt As Date

Private bSessionOnly As Boolean
Private bInit As Boolean
Private lInstanceType As DBMDatabaseInstanceTypeConstants
Private sSourceDatabaseName As String
Private lGroups As DBMParameterGroupsConstants

'''

Private bAbort As Boolean

'''

Private lStatus As DBMParameterStatusConstants
Private sBadParam As String
Private sUserValue As String
Private sCompValue As String

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTParameters
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get Count() As Long
    Count = oItems.Count
End Property

Public Property Get Item(Index As Variant, Optional ByVal Refresh As Boolean = False) As Parameter
Attribute Item.VB_UserMemId = 0
    Dim oParameter As Parameter
    Dim bNew As Boolean
        
    ' *** special Error
    On Error Resume Next
    Set oParameter = oItems(Index)
    If Err.Number <> 0 Then
        Set oParameter = Nothing
        bNew = True
    Else
        bNew = False
    End If
      
    If oParameter Is Nothing Then
        Set oParameter = New Parameter
        Refresh = True
        If Not oParameter Is Nothing Then
            Add Index, oParameter
            Set oParameter.Parent = Me
            oParameter.Name = Index
        End If
    End If
    
    If Not oParameter Is Nothing Then
        If Refresh Then
          If Not oParameter.Refresh() Then
              Set oParameter = Nothing
              Me.Remove Index
          End If
        End If
    End If
    
    Set Item = oParameter
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property

Public Property Get At() As Date
    At = dAt
End Property

Public Property Get Groups() As DBMParameterGroupsConstants
    Groups = lGroups
End Property

Public Property Get Changed() As Boolean
    Dim oParameter As Parameter
    
    Changed = False
    
    Set oParameter = Item(PARAM_PARAMCHANGED, True)
    If (Not oParameter Is Nothing) Then
        Changed = (Val(oParameter.Value) <> 0)
    End If
End Property

Public Property Get Dirty() As Boolean
    Dim oParameter As Parameter
    Dim lCount As Long
    Dim lItem As Long

    Dirty = False
    
    lCount = oItems.Count
    For lItem = 1 To lCount
        Set oParameter = oItems(lItem)
        If oParameter.Dirty Then
            Dirty = True
            Exit For
        End If
    Next
End Property

Public Property Get IsOpen() As Boolean
    IsOpen = bOpen
End Property

'''

Public Property Get Status() As DBMParameterStatusConstants
    Status = lStatus
End Property
Public Property Get BadParam() As String
    BadParam = sBadParam
End Property
Public Property Get UserValue() As String
    UserValue = sUserValue
End Property
Public Property Get CompValue() As String
    CompValue = sCompValue
End Property

'''

Friend Sub Add(ByVal Key As String, _
               ByVal Parameter As Parameter)
    oItems.Add Parameter, Key
End Sub

Friend Sub Remove(Index As Variant)
    oItems(Index).CleanUp
    oItems.Remove Index
End Sub

Friend Sub Clear()
    Dim lCount As Long
    Dim lItem As Long

    lCount = oItems.Count
    For lItem = 1 To lCount
        oItems(1).CleanUp
        oItems.Remove 1
    Next
End Sub

Public Sub CleanUp()
    Clear
    Set oParent = Nothing
End Sub

'''

Public Function GetValue(Name As String, Optional ByVal ErrorRaise As Boolean = True) As String
    Dim sData As String
    Dim lPos As Long
    Dim sValue As String
    Dim bOldErrorRaise As Boolean
        
    sValue = ""
    
    With Database
        bOldErrorRaise = .ServerObject.ErrorRaise
        .ServerObject.ErrorRaise = ErrorRaise
        If .ServerObject.CommandNorm(DBMSRV_CMD_PARAM_GETVALUE + " " + Name, sData) Then
            lPos = 1
            sValue = Trim$(.ServerObject.GetLine(sData, lPos))
        Else
            sValue = "Error: " + CStr(.ServerObject.ReturnCode) + " " + .ServerObject.ErrorText
        End If
        .ServerObject.ErrorRaise = bOldErrorRaise
    End With
    
    GetValue = sValue
End Function

Public Function OpenParameters(Optional ByVal SessionOnly As Boolean = False, _
                               Optional ByVal Init As Boolean = False, _
                               Optional ByVal InstanceType As DBMDatabaseInstanceTypeConstants = dbmInstanceTypeNone, _
                               Optional ByVal SourceDatabaseName As String = "", _
                               Optional ByVal Groups As DBMParameterGroupsConstants = dbmPGGeneral, _
                               Optional ByVal Asynchron As Boolean = False) As Boolean

    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
    
        bSessionOnly = SessionOnly
        bInit = Init
        lInstanceType = InstanceType
        sSourceDatabaseName = SourceDatabaseName
        lGroups = Groups
        
        If Not Asynchron Then
            bOK = Execute(FUNC_OPENPARAMS, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_OPENPARAMS
            bOK = True
        End If
    End If

    OpenParameters = bOK
End Function

Public Function CheckParameters(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        
        lStatus = Status
        sBadParam = BadParam
        sUserValue = UserValue
        sCompValue = CompValue
    
        If Not Asynchron Then
            bOK = Execute(FUNC_CHECKPARAMS, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_CHECKPARAMS
            bOK = True
        End If
    End If
    
    CheckParameters = bOK
End Function

Public Function SaveParamaters(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_SAVEPARAMS, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_SAVEPARAMS
            bOK = True
        End If
    End If

    SaveParamaters = bOK
End Function

Public Function CloseParameters(Optional ByVal Abort As Boolean = False, _
                                Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        
        bAbort = Abort
        
        If Not Asynchron Then
            bOK = Execute(FUNC_CLOSEPARAMS, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_CLOSEPARAMS
            bOK = True
        End If
    End If
                                
    CloseParameters = bOK
End Function

'''

Public Function AddDevspace(ByVal Number As Long, _
                            ByVal mode As String, _
                            ByVal Name As String, _
                            ByVal DeviceType As String, _
                            Optional ByVal Size As String = "") As Boolean
    Dim bOK As Boolean
        
    Database.Busy = True
    
    bOK = DBMSRVParam.AddDevspace(Database, Number, mode, Name, DeviceType, Size)
    
    If Not bOK Then
        Database.LastError.SetByServerObject Database.ServerObject
    End If
    
    Database.Busy = False
    
    AddDevspace = bOK
End Function

Public Function DeleteDevspace(ByVal Number As Long, _
                               ByVal mode As String) As Boolean
    Dim bOK As Boolean
        
    Database.Busy = True
    
    bOK = DBMSRVParam.DeleteDevspace(Database, Number, mode)
    
    If Not bOK Then
        Database.LastError.SetByServerObject Database.ServerObject
    End If
    
    Database.Busy = False
    
    DeleteDevspace = bOK
End Function

'''

Public Function FileExists(Optional ByVal Version As Long = 0) As Boolean
    Database.Busy = True
    FileExists = DBMSRVParam.FileExists(Database.ServerObject, Version)
    Database.Busy = False
End Function

Public Function FileRemove() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    Database.Busy = True
    
    If bOpen Then
        CloseParameters True
    End If
    
    Clear
    
    bOK = DBMSRVParam.RemoveFile(Database.ServerObject)
    
    If Not bOK Then
        Database.LastError.SetByServerObject Database.ServerObject
    End If
    
    FileRemove = bOK
    
    Database.Busy = False
End Function

Public Function FileRestore(ByVal Version As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    Database.Busy = True
    
    If bOpen Then
        CloseParameters True
    End If
    
    Clear
    
    bOK = DBMSRVParam.Restore(Database.ServerObject, Version)
    
    If Not bOK Then
        Database.LastError.SetByServerObject Database.ServerObject
    End If
    
    FileRestore = bOK
    
    Database.Busy = False
End Function

'''

Private Sub Class_Initialize()
    InitObject ObjectType
    lGroups = 0
    bOpen = False
End Sub

Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    
    Database.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_OPENPARAMS:   bOK = ExecuteOpenParameters(Asynchron)
        Case FUNC_CHECKPARAMS:  bOK = ExecuteCheckParameters(Asynchron)
        Case FUNC_SAVEPARAMS:   bOK = ExecuteSaveParameters(Asynchron)
        Case FUNC_CLOSEPARAMS:  bOK = ExecuteCloseParameters(Asynchron)
    End Select
            
    Database.Busy = False
    
    Execute = bOK
End Function

Private Function ExecuteOpenParameters(ByVal Asynchron As Boolean) As Boolean
    Dim bOK As Boolean
    Dim sGroups As String
    Dim sInstanceType As String
    
    bOK = False
    
    With Database
        If .Check() Then
    
            If bOpen Then
                DBMSRVParam.AbortSession .ServerObject
                bOpen = False
            End If
        
            If bInit Then
                If DBMSRVParam.FileExists(.ServerObject) Then
                    bOK = DBMSRVParam.RemoveFile(.ServerObject)
                Else
                    bOK = True
                End If
            Else
                bOK = True
            End If
        
            If bOK Then
                bOK = False
                
                If DBMSRVParam.StartSession(.ServerObject) Then
                    sGroups = GetGroupStr(lGroups)
                
                    If Not bSessionOnly Then
                        Clear
                        If bInit Then
                            If Len(Trim$(sSourceDatabaseName)) = 0 Then
                            
                                Select Case lInstanceType
                                    Case dbmInstanceTypeNone: sInstanceType = ""
                                    Case dbmInstanceTypeOLTP: sInstanceType = "OLTP"
                                    Case dbmInstanceTypeLVC:  sInstanceType = "LVC"
                                    Case dbmInstanceTypeCS:   sInstanceType = "CS"
                                    Case dbmInstanceTypeBW:   sInstanceType = "BW"
                                End Select
                            
                                If DBMSRVParam.Init(.ServerObject, sInstanceType) Then
                                    bOK = ExtGetAll(sGroups)
                                End If
                            Else
                                If DBMSRVParam.Copy(.ServerObject, sSourceDatabaseName) Then
                                    bOK = ExtGetAll(sGroups)
                                End If
                            End If
                        Else
                            bOK = ExtGetAll(sGroups)
                        End If
                    Else
                        bOK = True
                    End If
                
                    bOpen = True
                End If
            End If
        
            If Not bOK Then
                .LastError.SetByServerObject .ServerObject
                DBMSRVParam.AbortSession Database.ServerObject
                bOpen = False
            End If
        
        End If
        
        If bOK Then dAt = Now()
        
        If Asynchron Then
            .EventParametersAction Not bOK, dbmParametersActionOpen
        End If
    End With
    
    ExecuteOpenParameters = bOK
End Function

Private Function ExecuteCheckParameters(ByVal Asynchron As Boolean) As Boolean
    Dim bOK As Boolean
    Dim lItem As Long
    Dim oParameter As Parameter
    
    bOK = False
    
    With Database
        If .Check() Then
    
            ' init return
            lStatus = dbmPSOK
            sBadParam = ""
            sUserValue = ""
            sCompValue = ""
        
            bOK = True
        
            ' in loop put all parameters
            lItem = 1
            While lItem <= oItems.Count And bOK
                If Item(lItem).Dirty Then
                    bOK = Item(lItem).PutOnly()
                End If
                lItem = lItem + 1
            Wend
        
            ' check all
            If bOK Then
                bOK = DBMSRVParam.CheckAll(.ServerObject, GetCheckModeStr(lGroups), lStatus, sBadParam, sUserValue, sCompValue)
                If Not bOK Then
                    .LastError.SetByServerObject .ServerObject
                End If
            End If
            
        End If
        
        If Asynchron Then
            .EventParametersAction Not bOK, dbmParametersActionCheck
        End If
    End With
    
    ExecuteCheckParameters = bOK
End Function

Private Function ExecuteSaveParameters(ByVal Asynchron As Boolean) As Boolean
    Dim bOK As Boolean
    Dim lCount As Long
    Dim lItem As Long
    Dim oParam As Parameter
    
    bOK = False
    
    With Database
        If .Check() Then
            bOK = DBMSRVParam.CommitSession(.ServerObject, lStatus, sBadParam, sUserValue, sCompValue, True)
    
            If bOK Then
                lCount = oItems.Count
                For lItem = 1 To lCount
                    Set oParam = oItems(lItem)
                    oParam.Dirty = False
                Next
            End If

            If Not bOK Then
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If Asynchron Then
            .EventParametersAction Not bOK, dbmParametersActionSave
        End If
    End With

    ExecuteSaveParameters = bOK
End Function

Private Function ExecuteCloseParameters(ByVal Asynchron As Boolean) As Boolean
    Dim bOK As Boolean
    Dim lStatus As DBMParameterStatusConstants
    Dim sBadParam As String
    Dim sUserValue As String
    Dim sCompValue As String
    
    bOK = False
    
    If bOpen Then
        With Database
            If .Check() Then
                If bAbort Then
                    bOK = DBMSRVParam.AbortSession(.ServerObject)
                Else
                    If DBMSRVParam.CheckAll(.ServerObject, GetCheckModeStr(lGroups), lStatus, sBadParam, sUserValue, sCompValue) Then
                        If DBMSRVParam.CommitSession(.ServerObject, lStatus, sBadParam, sUserValue, sCompValue) Then
                            bOK = True
                        End If
                    End If
                End If
            
                If Not bOK Then
                    .LastError.SetByServerObject .ServerObject
                End If
            End If
            
            If Asynchron Then
                .EventParametersAction Not bOK, dbmParametersActionClose
            End If
        End With
    Else
        bOK = True
    End If
    
    bOpen = False
    
    ExecuteCloseParameters = bOK
End Function

'''

Private Function GetGroupStr(ByVal lGroups As DBMParameterGroupsConstants) As String
    Dim sGroups As String
    
    sGroups = "GROUP=" + GRP_GEN
    
    If lGroups >= dbmPGExtended Then
        sGroups = sGroups + "," + GRP_EXT
    End If
    
    If lGroups >= dbmPGSupport Then
        sGroups = sGroups + "," + GRP_SUP
    End If

    GetGroupStr = sGroups
End Function

Private Function GetCheckModeStr(ByVal lGroups As DBMParameterGroupsConstants) As String
    Dim sMode As String
    
    Select Case lGroups
        Case dbmPGGeneral:  sMode = GRP_GEN
        Case dbmPGExtended: sMode = GRP_EXT
        Case dbmPGSupport:  sMode = GRP_SUP
    End Select
    
    GetCheckModeStr = sMode
End Function

Private Function ExtGetAll(sSelection As String) As Boolean
    Dim bOK As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lLen As Long
    Dim lPos As Long
    Dim oParameter As Parameter

    bOK = False
    
    If DBMSRVParam.ExtGetAll(Database.ServerObject, sSelection, sData) Then
        bOK = True
        lLen = Len(sData)
        lPos = 1
        While lLen > lPos And bOK
            sLine = Trim$(Database.ServerObject.GetLine(sData, lPos))
            If (Len(sLine) > 0) Then
                Set oParameter = Item(Trim$(Database.ServerObject.GetField(sLine, vbTab, 1)), True)
                If (oParameter Is Nothing) Then
                    bOK = False
                End If
            End If
        Wend
    End If
    
    ExtGetAll = bOK
End Function
