Attribute VB_Name = "QUEUE"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private Declare Function GetTickCount Lib "kernel32.dll" ( _
    ) As Long

Private Declare Function SetTimer Lib "user32.dll" ( _
    ByVal hwnd As Long, _
    ByVal nIDEvent As Long, _
    ByVal uElapse As Long, _
    ByVal lpTimerFunc As Long) As Long
    
Private Declare Function KillTimer Lib "user32.dll" ( _
    ByVal hwnd As Long, _
    ByVal nIDEvent As Long) As Long

'''

Private oQueueItems As QueueItems
Private bEnabled As Boolean
Private lInterval As Long
Private lTimerID As Long

Private bInPollQueue As Boolean

'''

Public Property Get QueueItems() As QueueItems
    If oQueueItems Is Nothing Then
        Set oQueueItems = New QueueItems
    End If
    Set QueueItems = oQueueItems
End Property
Public Property Set QueueItems(ByVal NewVal As QueueItems)
    Set oQueueItems = NewVal
End Property

Public Property Get Enabled() As Boolean
    Enabled = bEnabled
End Property
Public Property Let Enabled(ByVal NewVal As Boolean)
    If NewVal <> bEnabled Then
        bEnabled = NewVal
        If bEnabled Then
            StartTimer
        Else
            StopTimer
        End If
    End If
End Property

Public Property Get Interval() As Long
    Interval = lInterval
End Property
Public Property Let Interval(ByVal NewVal As Long)
    lInterval = NewVal
    If lTimerID <> 0 Then
        ChangeInterval
    End If
End Property
'
' Methods
'
Public Sub Initialize()
    bEnabled = False
    lInterval = 100
    lTimerID = 0
    Set oQueueItems = Nothing
End Sub

Public Sub Terminate()
    Enabled = False
    
    If Not oQueueItems Is Nothing Then oQueueItems.CleanUp
    
    Set oQueueItems = Nothing
End Sub

'''

Private Sub StartTimer()
    lTimerID = SetTimer(0, 0, lInterval, AddressOf TimerProc)
End Sub

Private Sub StopTimer()
    Dim lResult As Long
    If lTimerID <> 0 Then
        lResult = KillTimer(0, lTimerID)
        lTimerID = 0
    End If
End Sub

Private Sub ChangeInterval()
    Dim lResult As Long
    lResult = SetTimer(0, lTimerID, lInterval, AddressOf TimerProc)
End Sub

Private Sub TimerProc(ByVal lHwnd As Long, _
                      ByVal lMsg As Long, _
                      ByVal lTimerID As Long, _
                      ByVal lTime As Long)
                      
    On Error Resume Next
    
    If bEnabled And (Not bInPollQueue) Then
        bInPollQueue = True
        PollQueue
        bInPollQueue = False
    End If
End Sub

Private Sub PollQueue()
    Dim oQueueItem As QueueItem
    Dim bRC As Boolean
    
    If oQueueItems.Count > 0 Then
        Set oQueueItem = oQueueItems.Item(1)
        
        #If DBG Then
        T "Execute Item 1 (" + oQueueItem.FunctionName + ") Count = " + CStr(oQueueItems.Count)
        #End If
        
        bRC = oQueueItem.Execute()

        oQueueItems.Remove 1
    End If
End Sub

'''

#If DBG Then

Private Sub T(ByVal sText As String)
    If Not oDbgTrace Is Nothing Then
        oDbgTrace.Text "DBMLIB(x0" + Hex(App.ThreadID) + ").Queue: " + sText
    End If
End Sub

#End If

