VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Sessions"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_REFRESH = "Refresh"
Const FUNC_KILL = "Kill"

Private oParent As Object

Private oItems As New Collection

Private oInfo As Info

Private iColPos() As Integer
Private iColLen() As Integer
Private iColCount As Integer

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTSessions
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As DBMLib.Database
    Set Database = oParent.Database
End Property

Public Property Get Count() As Integer
    Count = oItems.Count
End Property

Public Property Get Item(Index As Variant) As DBMLib.Session
Attribute Item.VB_UserMemId = 0
    Set Item = oItems(Index)
End Property

Public Property Get At() As Date
    At = Info.At
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property

Private Property Get Info() As Info
    If oInfo Is Nothing Then
        Set oInfo = New Info
        Set oInfo.Parent = Me
        oInfo.InfoType = dbmInfoUsers
    End If
    Set Info = oInfo
End Property

'''

Friend Sub Add(Session As DBMLib.Session, _
               Optional Key As Variant)
               
    Set Session.Parent = Me
               
    If IsMissing(Key) Then
        oItems.Add Session
    Else
        oItems.Add Session, Key
    End If
    
    Session.Index = oItems.Count
End Sub

Public Sub Clear()
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = 1 To lCount
        oItems(1).CleanUp
        oItems.Remove 1
    Next
End Sub

Public Sub CleanUp()
    Clear
    
    If Not oInfo Is Nothing Then oInfo.CleanUp
    
    Set oInfo = Nothing
    Set oParent = Nothing
End Sub

Public Sub Remove(Index As Variant)
    Dim lStart As Long
    
    lStart = oItems(Index).Index
    
    oItems(Index).CleanUp
    oItems.Remove Index
    
    SetIndex lStart
End Sub

Public Function Exists(Index As Variant) As Boolean
    Dim oSession As DBMLib.Session
    
    On Error Resume Next

    Set oSession = oItems(Index)
    
    If Err.Number Then
        Exists = False
    Else
        Exists = True
    End If
End Function

Public Function Refresh(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_REFRESH, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_REFRESH
            bOK = True
        End If
    End If
    
    Refresh = bOK
End Function

Public Function Kill(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_KILL, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_KILL
            bOK = True
        End If
    End If
    
    Kill = bOK
End Function

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim oDB As DBMLib.Database
    
    Set oDB = Database
    
    oDB.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_REFRESH:  bOK = ExecuteRefresh(Asynchron)
        Case FUNC_KILL:     bOK = ExecuteKill(Asynchron)
    End Select
            
    oDB.Busy = False
    
    Execute = bOK
End Function

Public Function ExecuteRefresh(ByVal bAsynchron As Boolean) As Boolean
    Dim oSession As DBMLib.Session
    Dim iType As DBMLib.DBMInfoColumnTypeConstants
    Dim sCol As String
    Dim iCol As Integer
    Dim iPos As Integer
    Dim iLen As Integer
    Dim iMaxCol As Integer
    Dim sLine As String
    Dim bOK As Boolean
    
    bOK = False
    
    Clear
    
    If Info.Refresh Then
        bOK = True
        
        Info.BeginReadColumns
        Do While Not Info.EndOfReadColumns
            sCol = Trim$(Info.ReadColumn(iCol, iType, iPos, iLen))
            If iCol > UBound(iColPos) Then
                iMaxCol = UBound(iColPos) + 10
                ReDim Preserve iColPos(1 To iMaxCol)
                ReDim Preserve iColLen(1 To iMaxCol)
            End If
            iColPos(iCol) = iPos
            iColLen(iCol) = iLen
        Loop
        iColCount = iCol
    
        Info.BeginReadLines
        Do While Not Info.EndOfReadLines
            sLine = Info.ReadLine
            
            Set oSession = New Session
            With oSession
                .Name = Trim$(Mid$(sLine, iColPos(1), iColLen(1)))
                .TerminalID = Trim$(Mid$(sLine, iColPos(2), iColLen(2)))
                .TaskID = Trim$(Mid$(sLine, iColPos(3), iColLen(3)))
                .SessionID = Trim$(Mid$(sLine, iColPos(4), iColLen(4)))
                .Node = Trim$(Mid$(sLine, iColPos(5), iColLen(5)))
                .CatalogCacheSize = Trim$(Mid$(sLine, iColPos(6), iColLen(6)))
                .Checked = False
            End With
            Add oSession, oSession.TaskID
        Loop
    End If
    
    If bAsynchron Then
        Database.EventSessionAction Not bOK, dbmSessionActionRefresh
    End If
    
    ExecuteRefresh = bOK
End Function

Public Function ExecuteKill(ByVal bAsynchron As Boolean) As Boolean
    Dim oSession As DBMLib.Session
    Dim l As Long
    Dim bOK As Boolean
    
    Dim sData As String
    
    With Database
        If .Check() Then
            bOK = True
            For l = 1 To oItems.Count
                Set oSession = oItems(l)
                If oSession.Checked Then
                    If Not .ServerObject.Command(DBMSRV_CMD_DB_CONS_KILL + " " + oSession.TaskID, sData) Then
                        bOK = False
                        Exit For
                    End If
                End If
            Next
            
            If Not bOK Then
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bAsynchron Then
            .EventSessionAction Not bOK, dbmSessionActionKill
        End If
    End With
    
    ExecuteKill = bOK
End Function

'''

Private Sub Class_Initialize()
    InitObject ObjectType
    
    ReDim iColPos(1 To 10) As Integer
    ReDim iColLen(1 To 10) As Integer
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

Private Sub SetIndex(ByVal lStart As Long)
    Dim oSession As DBMLib.Session
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = lStart To lCount
        Set oSession = oItems(l)
        oSession.Index = l
    Next
End Sub


