VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Shows"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const SHOW_IO = "IO"
Const SHOW_AIO = "AIO"
Const SHOW_STORAGE = "STORAGE"
Const SHOW_TASKS = "TASKS"
Const SHOW_ACTIVE = "ACTIVE"
Const SHOW_RUNNABLE = "RUNNABLE"
Const SHOW_T_C = "T_C"
Const SHOW_VERSIONS = "VERSIONS"
Const SHOW_REGIONS = "REGIONS"
Const SHOW_STATE = "STATE"
Const SHOW_RTE = "RTE"
Const SHOW_QUEUES = "QUEUES"
Const SHOW_SUSPENDS = "SUSPENDS"
Const SHOW_SLEEP = "SLEEP"
Const SHOW_THRD_TIMES = "THRD_TIMES"
Const SHOW_PSE_STAT = "PSE_STAT"
Const SHOW_PSE_DATA = "PSE_DATA"
Const SHOW_ALL = "ALL"

Const DESC_IO = "Input/output activities in regular database operation"
Const DESC_AIO = "Input/output activities for data backup"
Const DESC_STORAGE = "Configuration and current status of the various storage areas"
Const DESC_TASKS = "List of tasks"
Const DESC_ACTIVE = "Active tasks for [Datawriter-Task|Server-Task|User-Task]"
Const DESC_RUNNABLE = "Runnable tasks (waiting for CPU)"
Const DESC_T_C = "Task-specific information for [Datawriter_Task|Server-Task|User-Task|Task Nr.x]"
Const DESC_VERSIONS = "Current variant of database kernel and runtime environment"
Const DESC_REGIONS = "Information about the critical sections used in the kernel"
Const DESC_STATE = "Mode of database instance"
Const DESC_RTE = "Runtime environment"
Const DESC_QUEUES = "Queues of runnable tasks (waiting for CPU)"
Const DESC_SUSPENDS = "Information about suspend states anywhere in the overall system"
Const DESC_SLEEP = "The CPU load from user kernel threads measured by the database kernel"
Const DESC_THRD_TIMES = "Information from the system about the CPU load from user kernel threads"
Const DESC_PSE_STAT = "PSE statistics"
Const DESC_PSE_DATA = "PSE data page array"
Const DESC_ALL = "All of the information provided by the console"

Const FUNC_OPEN = "OpenShows"

Private oParent As Object

Private oItems As New Collection

Private dAt As Date

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTShows
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

Public Property Get Count() As Integer
    Count = oItems.Count
End Property

Public Property Get Item(Index As Variant) As Show
Attribute Item.VB_UserMemId = 0
    Set Item = oItems(Index)
End Property

Public Property Get At() As Date
    At = dAt
End Property

Public Property Get NewEnum()
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = oItems.[_NewEnum]
End Property

'''

Friend Sub Add(Show As Show, _
               Optional Key As Variant)
    Set Show.Parent = Me
               
    If IsMissing(Key) Then
        oItems.Add Show
    Else
        oItems.Add Show, Key
    End If
    
    Show.Index = oItems.Count
End Sub

Public Sub Clear()
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = 1 To lCount
        oItems(1).CleanUp
        oItems.Remove 1
    Next
End Sub

Public Sub CleanUp()
    Clear
    Set oParent = Nothing
End Sub

Public Sub Remove(Index As Variant)
    Dim lStart As Long
    
    lStart = oItems(Index).Index
    
    oItems(Index).CleanUp
    oItems.Remove Index
    
    SetIndex lStart
End Sub

Public Function Exists(Index As Variant) As Boolean
    Dim oShow As Show
    
    On Error Resume Next

    Set oShow = oItems(Index)
    
    If Err.Number Then
        Exists = False
    Else
        Exists = True
    End If
End Function

Public Function OpenShows(Optional ByVal Asynchron As Boolean = False) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_OPEN, False)
        Else
            QUEUE.QueueItems.Add Me, FUNC_OPEN
            bOK = True
        End If
    End If
    
    OpenShows = bOK
End Function

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim sData As String
    Dim sLine As String
    Dim lPos1 As Long
    Dim lPos2 As Long
    Dim sDate As String
    Dim sTime As String
    Dim oShow As Show
    
    bOK = False
    
    Database.Busy = True
            
    Clear
            
    With Database
        If .Check() Then
            If .ServerObject.Command(DBMSRV_CMD_SHOW_LIST, sData) Then
                dAt = Now()
                bOK = True
                lPos1 = 1
                Do While lPos1 < Len(sData)
                    sLine = Trim(.ServerObject.GetLine(sData, lPos1))
                    If Len(sLine) > 0 Then
                        Set oShow = New Show
                        oShow.Name = sLine
                        Select Case UCase(sLine)
                            Case SHOW_IO: oShow.Description = DESC_IO
                            Case SHOW_AIO: oShow.Description = DESC_AIO
                            Case SHOW_STORAGE: oShow.Description = DESC_STORAGE
                            Case SHOW_TASKS: oShow.Description = DESC_TASKS
                            Case SHOW_ACTIVE: oShow.Description = DESC_ACTIVE
                            Case SHOW_RUNNABLE: oShow.Description = DESC_RUNNABLE
                            Case SHOW_T_C: oShow.Description = DESC_T_C
                            Case SHOW_VERSIONS: oShow.Description = DESC_VERSIONS
                            Case SHOW_REGIONS: oShow.Description = DESC_REGIONS
                            Case SHOW_STATE: oShow.Description = DESC_STATE
                            Case SHOW_RTE: oShow.Description = DESC_RTE
                            Case SHOW_QUEUES: oShow.Description = DESC_QUEUES
                            Case SHOW_SUSPENDS: oShow.Description = DESC_SUSPENDS
                            Case SHOW_SLEEP: oShow.Description = DESC_SLEEP
                            Case SHOW_THRD_TIMES: oShow.Description = DESC_THRD_TIMES
                            Case SHOW_PSE_STAT: oShow.Description = DESC_PSE_STAT
                            Case SHOW_PSE_DATA: oShow.Description = DESC_PSE_DATA
                            Case SHOW_ALL: oShow.Description = DESC_ALL
                        End Select
                        
                        Add oShow, oShow.Name
                    End If
                Loop
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        .EventShowsOpen Not bOK
    End With
    
    Database.Busy = False
    
    Execute = bOK
End Function

'''

Private Sub Class_Initialize()
    InitObject ObjectType
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

Private Sub SetIndex(ByVal lStart As Long)
    Dim oShow As Show
    Dim lCount As Long
    Dim l As Long
    
    lCount = oItems.Count
    
    For l = lStart To lCount
        Set oShow = oItems(l)
        oShow.Index = l
    Next
End Sub


