VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "TermcharSet"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "RVB_UniqueId" ,"33F01F0D0000"
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Private oParent As Object

Private Const TERMCHAR_SQL_STATE1 = "SELECT DISTINCT UPPER(ASCII(CODE)), UPPER(ASCII(STATE)) FROM TERMCHARSETS WHERE TERMCHARSETNAME = UPPER('"
Private Const TERMCHAR_SQL_STATE2 = "')"
Private Const TERMCHAR_SQL_VALUES1 = "SELECT HEX(INTERN), HEX(EXTERN), ASCII(COMMENT) FROM TERMCHARSETS WHERE TERMCHARSETNAME = UPPER('"
Private Const TERMCHAR_SQL_VALUES2 = "')"

Private Const TERMCHAR_UTL_DROP = "DROP TERMCHAR SET "
Private Const TERMCHAR_UTL_ALTER = "ALTER TERMCHAR SET "
Private Const TERMCHAR_UTL_CREATE = "CREATE TERMCHAR SET "

Private Const TERMCHAR_UTL_AS = " AS "
Private Const TERMCHAR_UTL_ENABLE = " ENABLE "
Private Const TERMCHAR_UTL_WITH = " WITH "

Private Const TERMCHAR_UTL_DIGIT1 = " X'"
Private Const TERMCHAR_UTL_DIGIT2 = "' TO X'"
Private Const TERMCHAR_UTL_DIGIT3 = "'"

Private Const TERMCHAR_UTL_COMMENT1 = " FOR '"
Private Const TERMCHAR_UTL_COMMENT2 = "'"

Private Const TERMCHAR_UTL_COMMA = ","

Private Const TERMCHAR_COUNT_MAX = 128
Private Const TERMCHAR_INTERN_LEN = 2
Private Const TERMCHAR_EXTERN_LEN = 2
Private Const TERMCHAR_COMMENT_LEN = 8

Private Const TERMCHAR_STATE_DISABLED = "DISABLED"
Private Const TERMCHAR_STATE_ENABLED = "ENABLED"
Private Const TERMCHAR_TYPE_ASCII = "ASCII"
Private Const TERMCHAR_TYPE_EBCDIC = "EBCDIC"

Public Enum TermcharCodeTypeConstants
  dbmTermcharCodeTypeASCII
  dbmTermcharCodeTypeEBCDIC
End Enum

Private Type TermcharElement
    sIntern As String * TERMCHAR_INTERN_LEN
    sExtern As String * TERMCHAR_EXTERN_LEN
    sComment As String * TERMCHAR_COMMENT_LEN
End Type

Private m_sName As String
Private m_sCode As String
Private m_sEnabled As String
Private m_bExistInDb As Boolean
Private m_bToBeDeleted As Boolean

Private aCodeList(1 To TERMCHAR_COUNT_MAX) As TermcharElement

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTTermcharSet
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

'''

Public Property Get CodeType() As TermcharCodeTypeConstants
    If (m_sCode = TERMCHAR_TYPE_EBCDIC) Then
        CodeType = dbmTermcharCodeTypeEBCDIC
    Else
        CodeType = dbmTermcharCodeTypeASCII
    End If
End Property
Public Property Let CodeType(nValue As TermcharCodeTypeConstants)
    If (nValue = dbmTermcharCodeTypeEBCDIC) Then
        m_sCode = TERMCHAR_TYPE_EBCDIC
    Else
        m_sCode = TERMCHAR_TYPE_ASCII
    End If
End Property

Public Property Get CodeTypeStr() As String
    CodeTypeStr = m_sCode
End Property
Public Property Let CodeTypeStr(NewVal As String)
    Select Case UCase(NewVal)
        Case TERMCHAR_TYPE_EBCDIC:   m_sCode = NewVal
        Case TERMCHAR_TYPE_ASCII:    m_sCode = NewVal
        Case Else:                   m_sCode = TERMCHAR_TYPE_ASCII
    End Select
End Property

'''

Public Property Get Enabled() As Boolean
    Enabled = (m_sEnabled = TERMCHAR_STATE_ENABLED)
End Property
Public Property Let Enabled(bValue As Boolean)
    If (bValue) Then
        m_sEnabled = TERMCHAR_STATE_ENABLED
    Else
        m_sEnabled = TERMCHAR_STATE_DISABLED
    End If
End Property

Public Property Let ToDelete(bValue As Boolean)
    m_bToBeDeleted = bValue
End Property

Public Property Get MaxCode() As Integer
    MaxCode = TERMCHAR_COUNT_MAX
End Property

Public Property Get Name() As String
    Name = m_sName
End Property
Public Property Let Name(ByVal sValue As String)
    m_sName = sValue
End Property

'''

Public Function Refresh() As Boolean
    Clear
    Refresh = GetValuesFromDB()
End Function

Public Sub Clear()
    Dim i As Integer
    
    For i = 1 To TERMCHAR_COUNT_MAX
        aCodeList(i).sComment = ""
        aCodeList(i).sExtern = ""
        aCodeList(i).sIntern = ""
    Next
End Sub

Public Function SetCode(Index As Long, _
                        Comment As String, _
                        Extern As String, _
                        Intern As String) As Boolean
    SetCode = False
    
    If Index >= 0 And _
       Index <= TERMCHAR_COUNT_MAX And _
       Len(Comment) <= TERMCHAR_COMMENT_LEN And _
       Len(Extern) <= TERMCHAR_EXTERN_LEN And _
       Len(Intern) <= TERMCHAR_INTERN_LEN _
    Then
        aCodeList(Index).sComment = Comment
        aCodeList(Index).sExtern = Extern
        aCodeList(Index).sIntern = Intern
    
        SetCode = True
    End If
End Function

Public Function GetCode(Index As Long, _
                        Comment As String, _
                        Extern As String, _
                        Intern As String) As Boolean
    GetCode = False
    
    If Index >= 0 And _
       Index <= TERMCHAR_COUNT_MAX _
    Then
        Comment = Trim(aCodeList(Index).sComment)
        Extern = Trim(aCodeList(Index).sExtern)
        Intern = Trim(aCodeList(Index).sIntern)
    
        GetCode = True
    End If
End Function

Public Function Save() As Boolean
    If (m_bExistInDb) Then
        If (m_bToBeDeleted) Then
            Save = Drop()
        Else
            Save = Alter()
        End If
    Else
        If (Not m_bToBeDeleted) Then
            Save = Create()
        End If
    End If
    m_bExistInDb = Save
End Function

'''

Private Function Create() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .UtilConnect() Then
            If .UtilExecute(TERMCHAR_UTL_CREATE & Name & _
                            TERMCHAR_UTL_AS & IIf((CodeType = dbmTermcharCodeTypeEBCDIC), TERMCHAR_TYPE_EBCDIC, TERMCHAR_TYPE_ASCII) & _
                            IIf(Enabled, TERMCHAR_UTL_ENABLE, "") & _
                            TERMCHAR_UTL_WITH & TermcharList) _
            Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
            .UtilRelease
        End If
    End With
    
    Create = bOK
End Function

Private Function Alter() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .UtilConnect() Then
            If .UtilExecute(TERMCHAR_UTL_ALTER & Name & _
                            TERMCHAR_UTL_AS & IIf((CodeType = dbmTermcharCodeTypeEBCDIC), TERMCHAR_TYPE_EBCDIC, TERMCHAR_TYPE_ASCII) & _
                            IIf(Enabled, TERMCHAR_UTL_ENABLE, "") & _
                            TERMCHAR_UTL_WITH & TermcharList) _
            Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
            .UtilRelease
        End If
    End With
    
    Alter = bOK
End Function

Private Function Drop() As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .UtilConnect() Then
            If .UtilExecute(TERMCHAR_UTL_DROP & Name) _
            Then
                bOK = True
            Else
                .LastError.SetByServerObject .ServerObject
            End If
            .UtilRelease
        End If
    End With
    
    Drop = bOK
End Function

Private Function GetValuesFromDB() As Boolean
    Dim bOK As Boolean
    Dim bSQLCon As Boolean
    Dim sResult As String
    Dim sRow As String
    Dim sIntern As String
    Dim sExtern As String
    Dim sComment As String
    Dim nIndex As Long
    Dim bContinue As Boolean
    
    bOK = False
    
    bSQLCon = False
    
    With Database
        If DBMSRVSQL.Connect(.ServerObject) Then
            bSQLCon = True
            If DBMSRVSQL.Execute(.ServerObject, TERMCHAR_SQL_STATE1 & Name & TERMCHAR_SQL_STATE2, sResult) Then
                sRow = DBMSRVSQL.NextRow(sResult)
                If Len(sRow) > 0 Then
                    CodeType = IIf((DBMSRVSQL.NextField(sRow) = TERMCHAR_TYPE_EBCDIC), dbmTermcharCodeTypeEBCDIC, dbmTermcharCodeTypeASCII)
                    Enabled = (DBMSRVSQL.NextField(sRow) = TERMCHAR_STATE_ENABLED)
                    bOK = True
                End If
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        Else
            .LastError.SetByServerObject .ServerObject
        End If
        
        ' retrieve codes
        If bOK And DBMSRVSQL.Execute(.ServerObject, TERMCHAR_SQL_VALUES1 & Name & TERMCHAR_SQL_VALUES2, sResult, bContinue) Then
            sRow = DBMSRVSQL.NextRow(sResult)
            nIndex = 1
        
            While Len(sRow) > 0 And bOK
                sIntern = DBMSRVSQL.NextField(sRow)
                sExtern = DBMSRVSQL.NextField(sRow)
                sComment = DBMSRVSQL.NextField(sRow)
                bOK = SetCode(nIndex, sComment, sExtern, sIntern)
                sRow = DBMSRVSQL.NextRow(sResult)
                nIndex = nIndex + 1
            Wend
        
            While bOK And bContinue
                bOK = DBMSRVSQL.Fetch(.ServerObject, sResult, bContinue)
        
                If bOK Then
                    sRow = DBMSRVSQL.NextRow(sResult)
                    nIndex = 1
        
                    While Len(sRow) > 0 And bOK
                        sIntern = DBMSRVSQL.NextField(sRow)
                        sExtern = DBMSRVSQL.NextField(sRow)
                        sComment = DBMSRVSQL.NextField(sRow)
                        bOK = SetCode(nIndex, sComment, sExtern, sIntern)
                        sRow = DBMSRVSQL.NextRow(sResult)
                        nIndex = nIndex + 1
                    Wend
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
        
            Wend
        Else
            .LastError.SetByServerObject .ServerObject
            bOK = False
        End If
        
        If bSQLCon Then DBMSRVSQL.Release .ServerObject
    End With
        
    m_bExistInDb = bOK
    
    GetValuesFromDB = bOK
End Function

Private Function TermcharList() As String
    Dim sList As String
    Dim nIndex As Integer
    
    sList = ""
    For nIndex = 1 To TERMCHAR_COUNT_MAX
    
        If Trim(aCodeList(nIndex).sIntern) <> "" And _
           Trim(aCodeList(nIndex).sExtern) <> "" _
        Then
            sList = sList & IIf(nIndex > 1, TERMCHAR_UTL_COMMA, "")
            sList = sList & _
                TERMCHAR_UTL_DIGIT1 & UCase(aCodeList(nIndex).sIntern) & _
                TERMCHAR_UTL_DIGIT2 & UCase(aCodeList(nIndex).sExtern) & _
                TERMCHAR_UTL_DIGIT3 & _
                IIf((Trim(aCodeList(nIndex).sComment) <> ""), TERMCHAR_UTL_COMMENT1 & Trim(aCodeList(nIndex).sComment) & TERMCHAR_UTL_COMMENT2, "")
        End If
    Next
    
    TermcharList = sList
End Function

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

'''

Private Sub Class_Initialize()
    InitObject ObjectType
    
    m_sName = ""
    m_sCode = TERMCHAR_TYPE_ASCII
    m_sEnabled = TERMCHAR_STATE_DISABLED
    m_bExistInDb = False
    m_bToBeDeleted = False
End Sub
Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

