VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "UpdStat"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' --- GPL ---
'
' Copyright (C) 1997-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
Option Explicit

Const FUNC_OPEN = "OpenUpdStat"
Const FUNC_FETCH = "FetchUpdStat"
Const FUNC_CLOSE = "CloseUpdStat"
Const FUNC_EXECUTE = "ExecuteUpdStat"

Private oParent As Object

Private sSchemaName As String
Private sTableName As String
Private bWanted As String

Private sSpec As String
Private sSpecCols As String

Private bOpen As Boolean

'''

Private sHeader As String
Private sResult As String
Private bContinue As Boolean

'''

Public Property Get ObjectType() As DBMObjectTypeConstants
    ObjectType = dbmOTUpdStat
End Property

Public Property Get Parent() As Object
    Set Parent = oParent
End Property
Friend Property Set Parent(NewVal As Object)
    Set oParent = NewVal
End Property

Public Property Get Database() As Database
    Set Database = oParent.Database
End Property

'''

Public Property Get SchemaName() As String
    SchemaName = sSchemaName
End Property

Public Property Get TableName() As String
    TableName = sTableName
End Property

Public Property Get Wanted() As Boolean
    Wanted = bWanted
End Property

'''

Public Property Get Header() As String
    Header = sHeader
End Property

Public Property Get Result() As String
    Result = sResult
End Property

Public Property Get Continue() As Boolean
    Continue = bContinue
End Property

'''

Public Function OpenUpdStat(SchemaName As String, _
                            TableName As String, _
                            Optional ByVal Wanted As Boolean = True, _
                            Optional ByVal Asynchron As Boolean = False, _
                            Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        sSchemaName = SchemaName
        sTableName = TableName
        bWanted = Wanted
        
        If Not Asynchron Then
            bOK = Execute(FUNC_OPEN, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_OPEN, SenderID
            bOK = True
        End If
    End If
    
    OpenUpdStat = bOK
End Function

Public Function FetchUpdStat(Optional ByVal Asynchron As Boolean = False, _
                             Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_FETCH, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_FETCH, SenderID
            bOK = True
        End If
    End If
    
    FetchUpdStat = bOK
End Function

Public Function CloseUpdStat(Optional ByVal Asynchron As Boolean = False, _
                             Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
        If Not Asynchron Then
            bOK = Execute(FUNC_CLOSE, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_CLOSE, SenderID
            bOK = True
        End If
    End If
    
    CloseUpdStat = bOK
End Function

Public Function ExecuteUpdStat(Spec As String, _
                               Optional SpecCols As String = "", _
                               Optional ByVal Asynchron As Boolean = False, _
                               Optional ByVal SenderID As Long = 0) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    If Database.Check() Then
    
        sSpec = Spec
        sSpecCols = SpecCols
        
        If Not Asynchron Then
            bOK = Execute(FUNC_EXECUTE, False, SenderID)
        Else
            QUEUE.QueueItems.Add Me, FUNC_EXECUTE, SenderID
            bOK = True
        End If
    End If
    
    ExecuteUpdStat = bOK
End Function

'''

Public Sub CleanUp()
    Set oParent = Nothing
End Sub

Private Sub Class_Initialize()
    InitObject ObjectType
    bOpen = False
End Sub

Private Sub Class_Terminate()
    TermObject ObjectType
End Sub

'''

Public Function Execute(ByVal FunctionName As String, ByVal Asynchron As Boolean, Optional ByVal SenderID As Long) As Boolean
    Dim bOK As Boolean
    
    Database.Busy = True
    
    bOK = False
    
    Select Case FunctionName
        Case FUNC_OPEN:     bOK = ExecuteOpenUpdStat(Asynchron, SenderID)
        Case FUNC_FETCH:    bOK = ExecuteFetchUpdStat(Asynchron, SenderID)
        Case FUNC_CLOSE:    bOK = ExecuteCloseUpdStat(Asynchron, SenderID)
        Case FUNC_EXECUTE:  bOK = ExecuteExecuteUpdStat(Asynchron, SenderID)
    End Select
            
    Database.Busy = False
    
    Execute = bOK
End Function

Private Function ExecuteOpenUpdStat(ByVal bAsynchron As Boolean, _
                                    ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    Dim sInfo As String
    Dim sWhere As String
    Dim sLike As String
    
    bOK = False
    
    If bWanted Then
        sInfo = "UPDSTATW"
    Else
        sInfo = "UPDSTAT"
    End If
    
    sWhere = ""
    
    With Database
        If .Check() Then
            If Len(sSchemaName) > 0 Then
                sLike = Replace(sSchemaName, "*", "%")
                If sLike <> "%" Then
                    sWhere = sWhere + " ""Owner"" LIKE '" + sLike + "' AND"
                End If
            End If
            If Len(sTableName) > 0 Then
                sLike = Replace(sTableName, "*", "%")
                If sLike <> "%" Then
                    sWhere = sWhere + " ""Table Name"" LIKE '" + sLike + "' AND"
                End If
            End If
        
            If Right(sWhere, 3) = "AND" Then
                sWhere = Left$(sWhere, Len(sWhere) - 4)
            End If
        
            If DBMSRVSQL.Connect(.ServerObject) Then
                If DBMSRVInfo.InfoSelect(.ServerObject, sInfo, sWhere, sHeader, sResult, bContinue) Then
                    bOpen = True
                    bOK = True
                Else
                    DBMSRVSQL.Release .ServerObject
                End If
                
            End If
            
            If Not bOK Then
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bAsynchron Then
            .EventUpdStatAction lSenderID, Not bOK, dbmUpdStatActionOpen
        End If
        
    End With
    
    ExecuteOpenUpdStat = bOK
End Function

Private Function ExecuteFetchUpdStat(ByVal bAsynchron As Boolean, _
                                     ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If bOpen Then
                If DBMSRVInfo.InfoFetch(.ServerObject, sHeader, sResult, bContinue) Then
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
            End If
        End If
        
        If bAsynchron Then
            .EventUpdStatAction lSenderID, Not bOK, dbmUpdStatActionFetch
        End If
    End With
    
    ExecuteFetchUpdStat = bOK
End Function

Private Function ExecuteCloseUpdStat(ByVal bAsynchron As Boolean, _
                                     ByVal lSenderID As Long) As Boolean
    Dim bOK As Boolean

    bOK = False
    
    With Database
        If .Check() Then
            If bOpen Then
                If DBMSRVSQL.Release(.ServerObject) Then
                    bOpen = False
                    bOK = True
                Else
                    .LastError.SetByServerObject .ServerObject
                End If
            End If
        End If
        
        If bAsynchron Then
            .EventUpdStatAction lSenderID, Not bOK, dbmUpdStatActionClose
        End If
    End With
    
    ExecuteCloseUpdStat = bOK
End Function

Private Function ExecuteExecuteUpdStat(ByVal bAsynchron As Boolean, _
                                       ByVal lSenderID As Long) As Boolean

    Dim bOK As Boolean
    
    bOK = False
    
    With Database
        If .Check() Then
            If DBMSRVSQL.UpdateStat(.ServerObject, sSpec) Then
                If Len(sSpecCols) > 0 Then
                    If DBMSRVSQL.UpdateStat(.ServerObject, sSpecCols) Then
                        bOK = True
                    Else
                        .LastError.SetByServerObject .ServerObject
                    End If
                Else
                    bOK = True
                End If
            Else
                .LastError.SetByServerObject .ServerObject
            End If
        End If
        
        If bAsynchron Then
            .EventUpdStatAction lSenderID, Not bOK, dbmUpdStatActionExecute
        End If
    End With
    
    ExecuteExecuteUpdStat = bOK
End Function





